/*
 SOS Project Classes.

 Copyright (C) 2006-2008 SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sos.framework;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.LineBorder;

import sos.framework.ressources.Icons;
import sos.framework.ressources.Lang;
import sos.framework.simulation.SEnv;
import sos.lib.logger.Logger;
import sos.lib.uicommon.UI;

/**
 * Classe qui represente le simulateur sous forme d'Applet.
 * 
 * @author Camille AMBROSI
 *
 */
public class SOSApplet extends JPanel implements ActionListener
{
	
	public static int APPLET_WIDTH						= 540;
	public static int APPLET_HEIGHT						= 420;
	public static Color APPLET_COLOR					= new Color(255, 255, 255);

	private static final long	serialVersionUID	= 1L;

	private JButton				btnMemGc			= null;
	private JButton				btnMemUpdate		= null;
	private JButton				btnSimul			= null;

	private JLabel				lblMemory			= null;
	private JPanel				pnlConfigStdC		= null;

	private JPanel				pnlConfigStd		= null;



	public SOSApplet()
	{
		super(new BorderLayout());
		try{
	
			// on initialise l'interface graphique
			UI.init();
			
			// langue par défaut
			Lang.loadFrench();
			
			// on construit les conteneurs
			buildMainPanel();
			
			// affichage de la memoire
			parseMemory();
			
			// affiche le tout
			this.setBackground(APPLET_COLOR);
			this.add(pnlConfigStdC, BorderLayout.CENTER);
			this.add(UI.createLabel(SOSApplication.APPLET_TITLE, UI.FONT_PLAIN_ITALIC), BorderLayout.SOUTH);
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "SOSApplet()", ex);}
	}



	private final void buildMainPanel()
	{
		try{
			JPanel pnlTmp;
			pnlConfigStd = UI.createPanel(null);
			pnlConfigStd.setLayout(new BoxLayout(pnlConfigStd, BoxLayout.Y_AXIS));
			/* langue */
			pnlTmp = UI.createPanel(null);
			pnlTmp.setLayout(new BoxLayout(pnlTmp, BoxLayout.X_AXIS));
			pnlTmp.add(UI.createLabel("Langue : ", UI.FONT_PLAIN_ITALIC));
			String[] tabl = { "Français" };
			pnlTmp.add(new JComboBox(tabl));
			/* nouvelle simulation */
			pnlTmp.add(UI.createPanel(null, new Dimension(8, 25)));
			pnlTmp.add(btnSimul = UI.createButton(Lang.STR_BTN_SHOW, this, new Dimension(180, 25)));
			pnlConfigStd.add(pnlTmp);
			/* logo */
			JLabel lbl = new JLabel("");
			ImageIcon ic = Icons.LOGO_MAIN_470;
			if(ic != null) lbl.setIcon(ic);
			pnlTmp = UI.createPanel(null);
			pnlTmp.setPreferredSize(new Dimension(25, 25));
			pnlConfigStd.add(pnlTmp);
			pnlTmp = UI.createPanel(new FlowLayout());
			pnlTmp.add(lbl);
			pnlConfigStd.add(pnlTmp);
			pnlTmp = UI.createPanel(null);
			pnlTmp.setPreferredSize(new Dimension(25, 25));
			pnlConfigStd.add(pnlTmp);
			/* mémoire */
			lblMemory = new JLabel("");
			pnlTmp = UI.createPanel(new FlowLayout());
			pnlTmp.add(lblMemory);
			pnlConfigStd.add(pnlTmp);
			pnlTmp = UI.createPanel(new FlowLayout());
			pnlTmp.add(btnMemUpdate = UI.createButton("Mise à jour", this, UI.SIZE_BUTTON));
			pnlTmp.add(btnMemGc = UI.createButton("Nettoyer", this, UI.SIZE_BUTTON));
			pnlConfigStd.add(pnlTmp);
			/* pour la bordure */
			pnlConfigStdC = new JPanel(new BorderLayout());
			pnlConfigStdC.setBackground(APPLET_COLOR);
			pnlConfigStdC.setBorder(new LineBorder(APPLET_COLOR, 5, false));
			pnlConfigStdC.add(pnlConfigStd, BorderLayout.NORTH);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "buildMainPanel()", ex);}
	}



	private final void parseMemory()
	{
		try{
			long total = Runtime.getRuntime().totalMemory();
			long used = total - Runtime.getRuntime().freeMemory();
			lblMemory.setText("Mémoire utilisée : " + (used / 1024) + " Ko. (" + ((int)((((double)used / (double)total)) * 100.0)) + "%), " + (Runtime.getRuntime().freeMemory() / 1024) + " Ko. libre, " + (Runtime.getRuntime().maxMemory() / 1024) + " Ko. dispo.");
			pnlConfigStdC.updateUI();
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "parseMemory()", ex);}
	}



	public final void actionPerformed(ActionEvent ae)
	{
		try{
			if(ae.getSource() == btnMemGc){
				System.gc();
				parseMemory();
				return;
			}
			if(ae.getSource() == btnMemUpdate){
				parseMemory();
				return;
			}
			if(ae.getSource() == btnSimul){
				// nouvel environnement
				SEnv env = new SEnv();
				// affiche la fenetre
				env.getWindow().setVisible(true);
				env.getWindow().buildEditor(env);
				return;
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "actionPerformed()", ex);}
	}

}
