/*
 SOS Project Classes.

 Copyright (C) 2006-2008 SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sos.framework.generator;

import sos.framework.maths.Random;
import sos.framework.maths.conversions.solidangles.FromArcsecSq;
import sos.framework.simulation.simul.SSimul;
import sos.framework.simulation.simul.specs.SSpecImage;
import sos.framework.simulation.simul.specs.SSpecs;
import sos.framework.simulation.simul.specs.distrib.DistribFunction;
import sos.framework.simulation.simul.specs.sources.AiryPoint;
import sos.lib.logger.Logger;

public class DistributionGenerator
{
	
	
	
	
	
	public static final void generateDistribution(final SSimul simul)
	{
		try{
			
			Logger.log(" ");
			Logger.beginLog("Distribution des sources...");
			
			if( simul.getSpecs().isUseGenerator() == true ){
	
				// regarde d'il faut distribuer
				boolean blRedistribute = false;
				for(int iImgNb=0; iImgNb<simul.getImagesCount(); iImgNb++){
					if( simul.needGenerateStage(iImgNb, SSimul.STAGE_DISTRIBUTION) ){
						blRedistribute = true;
						break;
					}
				}
			
				if( (blRedistribute == true) || (simul.getSpecs().getSourcesContainer().getSourcesCount() == 0) ){
					simul.getSpecs().getSourcesContainer().clearSources();
					simul.getSpecs().getSourcesContainer().setSourcesOrigin(simul.getSpecs().getDistributionFunction().getName());
						
					int nb = distributeFunction(simul.getSpecs(), simul.getSpecs().getDistributionFunction());
					
					Logger.log("total : " + nb + " objets distribués");
		

					//simul.getListeners().fireSimulationComponentsChanged();
				
				}else Logger.log("distribution inutile");
				
			}else Logger.log("générateur de sources désactivé");
		
			Logger.endLog(true);
			return;
		}catch(Exception ex) {Logger.fireError(DistributionGenerator.class.getName(), "generateDistribution()", ex);}
		Logger.endLog(false);
	}
	
	
	
	

	
	
	
	private static final int distributeFunction(final SSpecs specs, final DistribFunction func)
	{
		try{
			Logger.beginLog("distribution de : " + func.getName() + "...");
			
			// recherche la plus grande surface
			final int iAddPix = 0;
			SSpecImage image = null;
			double dblMaxX = 0.0, dblNewX = 0.0;
			double dblMaxY = 0.0, dblNewY = 0.0;
			double dblMaxSurface = 0.0, dblNewSurface = 0.0;
			for(int i=0; i<specs.getImages().size(); i++){
				image = specs.getImageAt(i);
				dblNewX = image.getPixelResolution() * (image.getImageWidth() + iAddPix);
				dblNewY = image.getPixelResolution() * (image.getImageHeight() + iAddPix);
				dblNewSurface = dblNewX * dblNewY;
				if(dblNewSurface > dblMaxSurface){
					dblMaxSurface = dblNewSurface;
					dblMaxX = dblNewX;
					dblMaxY = dblNewY;
				}
			}
			
			// conversion de la surface en steradian
			Logger.log("surface de l'image (arcsec^2) : " + dblMaxSurface);
			dblMaxSurface = FromArcsecSq.toSterad( dblMaxSurface );
			Logger.log("surface de l'image (sterad) : " + dblMaxSurface);
		
			// parcours tous les blocs et ls distribue
			int nbTotal = 0;
			double numberPerSterad;
			int roundNumberPerImage;
			dblMaxX = dblMaxX / 2.0;
			dblMaxY = dblMaxY / 2.0;
			double[] bloc;
	
			Logger.log("[ plage de flux ; plage de nombre / sterad ]");
			for(int nbBloc=0; nbBloc<func.getBlocsNb(); nbBloc++){
				bloc = func.getBlocAt(nbBloc);
				
				// nombre de sources a distribuer par steradian
				if(bloc[DistribFunction.NB1] == bloc[DistribFunction.NB1]) numberPerSterad = bloc[DistribFunction.NB1];
				else numberPerSterad = Random.randomDouble(bloc[DistribFunction.NB1], bloc[DistribFunction.NB2]);

				// on regarde si la valeur est valide
				if( (Double.isNaN(numberPerSterad) == true) || (numberPerSterad <= 0.0) ) return 0;
				
				// nombre pour cette image
				double numberPerImage = numberPerSterad * dblMaxSurface;
				
				// arrondi
				roundNumberPerImage = 0;
				if(numberPerImage >= 1){
					roundNumberPerImage = (int) Math.round(numberPerImage);
				}else if(numberPerImage >= 0.25){
					if( numberPerImage > Math.random() ) roundNumberPerImage = 1;
				}
				
				double x, y, flux;
				for(int nbSrc=0; nbSrc<roundNumberPerImage; nbSrc++){
				
					// determine la position
					x = Random.randomDouble(-dblMaxX, dblMaxX);
					y = Random.randomDouble(-dblMaxY, dblMaxY);
					
					// determine le flux de la source
					if(bloc[DistribFunction.FL1] == bloc[DistribFunction.FL2]) flux = bloc[DistribFunction.FL1];
					else flux = Random.randomDouble(bloc[DistribFunction.FL1], bloc[DistribFunction.FL2]);
					
					// ajoute la source
					specs.getSourcesContainer().addSource(new AiryPoint(x, y, flux));
				}
				Logger.log("[ " + bloc[DistribFunction.FL1] + ".." + bloc[DistribFunction.FL2] + " ; " + bloc[DistribFunction.NB1] + ".." + bloc[DistribFunction.NB2] + " ] : " + numberPerSterad + " nb/sr, " + numberPerImage + " nb/image (" + roundNumberPerImage + " sources)");
				nbTotal += roundNumberPerImage;
			}
			Logger.log(nbTotal + " objet(s) distribues");
			
			Logger.endLog(true);
			return nbTotal;
		}catch(Exception ex) {Logger.fireError(DistributionGenerator.class.getName(), "distributeFunction()", ex);}
		Logger.endLog(false);
		return 0;
	}
	
	
	
	
	
	
	

}

