/*
 SOS Project Classes.

 Copyright (C) SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package sos.framework.generator;

import sos.framework.maths.SOSParser;
import sos.framework.simulation.simul.SSimul;
import sos.framework.simulation.simul.results.SResultImage;
import sos.framework.simulation.simul.specs.SSpecImage;
import sos.framework.simulation.simul.specs.noises.SNoise;
import sos.lib.SOSReplace;
import sos.lib.logger.Logger;
import sos.lib.mathparser.ParsedExpr;



public class NoisesGenerator
{

	
	
	
	
	
	public static final void generateNoises(final SSimul simul)
	{
		try{
			
			Logger.log(" ");
			Logger.beginLog("Génération des bruits...");

			SNoise noise = null;
			
			for(int iImgNb=0; iImgNb<simul.getImagesCount(); iImgNb++){
				
				// on regarde s'il faut la generer la matrice
				if(!simul.needGenerateStage(iImgNb, SSimul.STAGE_NOISES)){
					Logger.log("Image n°" + iImgNb + " ignorée.");
					Logger.log("");
					continue;
				}else Logger.beginLog("Image n°" + iImgNb + "...");
				
				// on recupere les imges
				SSpecImage imageSpec = simul.getSpecs().getImageAt(iImgNb);
				SResultImage imageRes = simul.getResults().getImageAt(iImgNb);

				// reinitialise les bruits
				Logger.beginLog("Initialisation des matrices des bruits...");
				if( imageRes.initForNoises(imageSpec.getNoises()) ) Logger.log("creation de la matrice.");
				else Logger.log("reutilisation de la matrice.");
				
			
				// met a jour les variables qui correspondent a cette image
				SOSParser.updateNoisesParserVariablesForImage(imageSpec);
				
				// max des sources de l'image
				double dMaxSourcesElec = imageRes.computeMaxTotalElectrons();
			
				// on va parcourir tous les bruits
				for(int iNoiseNumber=0; iNoiseNumber<imageSpec.getNoises().size(); iNoiseNumber++){
					
					// retrouve le bruit
					noise = imageSpec.getNoises().get(iNoiseNumber);

					// parse l'expression
					String strFormule = noise.getFunction();
					strFormule = SOSReplace.replace(strFormule, "\r", "\n");
					strFormule = SOSReplace.replace(strFormule, "\n\n", "\n");
					ParsedExpr p = SOSParser.getNoisesParser().parse(strFormule);
					
					p.setVarValue(SOSParser.VAR_MAX_SOURCES_ELECT, dMaxSourcesElec);
					p.setVarValue(SOSParser.VAR_MAX_TOTAL_ELECT, imageRes.computeMaxTotalElectrons());
					
					Logger.log("");
					Logger.log("Fonction correctement parsee, ajout de : " + noise.getDescription() + "...");
					
					double dblNoiseValue;
					if( noise.getApplyForEachPixel() == false ){
						
						// compile l'expression
						p.compute();
						
						// valeur du bruit
						dblNoiseValue = p.value();
						
						// applique le bruit pour chaque pixel
						for(int x=0; x<imageSpec.getImageWidth(); x++){
							for(int y=0; y<imageSpec.getImageHeight(); y++){
								imageRes.setNoiseElectrons(x, y, iNoiseNumber, dblNoiseValue);
							}
						}
						
					}else{
						
						// on applique le bruit pour chaque pixel
						for(int x=0; x<imageSpec.getImageWidth(); x++){
							for(int y=0; y<imageSpec.getImageHeight(); y++){
								
								// met a jour les variables relatives au pixel courant
								p.setVarValue(SOSParser.VAR_SOURCES_ELECT, imageRes.getSourcesElectrons(x, y));
								p.setVarValue(SOSParser.VAR_TOTAL_ELECT, imageRes.getTotalElectrons(x, y));
								p.setVarValue(SOSParser.VAR_X, x);
								p.setVarValue(SOSParser.VAR_Y, y);
								
								// compile l'expression
								p.compute();
								
								dblNoiseValue = p.value();
								
								imageRes.setNoiseElectrons(x, y, iNoiseNumber, dblNoiseValue);
							}
						}
					}
					
					if(p.getLastErr() != null) System.out.println("ERROR : " + p.getLastErr());
					imageRes.validateNoise(iNoiseNumber);
				}
	
				
				Logger.endLog(true);
			}
			
			Logger.endLog(true);
			return;
		}catch(Exception ex) {Logger.fireError(NoisesGenerator.class.getName(), "generateNoises()", ex);}
		Logger.endLog(false);
	}
	
	
	
	
	


}

