/*
 SOS Project Classes.

 Copyright (C) 2006-2008 SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sos.framework.generator;

import sos.framework.maths.conversions.flux.FromElectPerPix;
import sos.framework.simulation.SEnv;
import sos.framework.simulation.simul.SSimul;
import sos.lib.logger.Logger;

public class SOSGenerator
{
	


	public static final synchronized void generate(final SEnv env) {generate(env, true);}
	public static final synchronized void generate(final SEnv env, final boolean blThread)
	{
		try{
			if( blThread == true ){
				/* on va lancer la generation dans un autre thread */
				new Thread(){
					public void run(){
						startGeneration(env);
					}
				}.start();
			}else{
				/* on va lancer la generation dans un le meme thread */
				startGeneration(env);
			}
		}catch(Exception ex) {Logger.fireError(SOSGenerator.class.getName(), "generate()", ex);}
	}

	
	

	
	private static final void startGeneration(final SEnv env)
	{
		SSimul simul = null;
		try{
			// passe en mode generation
			simul = env.getSimul();
			Logger.initLog();
			simul.setSimulating(true);
			
			simul.setSimulationStage(SSimul.STAGE_INITIALISATION);
			for(int i=0; i<simul.getImagesCount(); i++){
				simul.getResults().getImageAt(i).setToJyPerSteradFactor(FromElectPerPix.factorToJyPerSterad(simul.getSpecs().getImageAt(i)));
				
			}

			simul.setSimulationStage(SSimul.STAGE_DISTRIBUTION);
			DistributionGenerator.generateDistribution(simul);
			simul.fireResultsComputed(SSimul.STAGE_DISTRIBUTION);
			
			simul.setSimulationStage(SSimul.STAGE_GENERATION);
			SourcesGenerator.generateSources(simul);
			simul.fireResultsComputed(SSimul.STAGE_GENERATION);
			
			simul.setSimulationStage(SSimul.STAGE_NOISES);
			NoisesGenerator.generateNoises(simul);
			simul.fireResultsComputed(SSimul.STAGE_NOISES);
			
			simul.setSimulationStage(SSimul.STAGE_NORMALISATION);
			ImageGenerator.generateImage(simul);
			simul.fireResultsComputed(SSimul.STAGE_NORMALISATION);
			
			simul.setSimulationStage(SSimul.STAGE_END);
			Logger.endLog(true);
		}catch(Exception ex){
			Logger.fireError(SOSGenerator.class.getName(), "startGeneration()", ex);
			Logger.endLog(true);
		}
		if(simul != null) simul.setSimulating(false);
	}

	
	

	
	
}

