/*
 SOS Project Classes.

 Copyright (C) 2006-2008 SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sos.framework.io;

import java.io.File;

import javax.swing.JOptionPane;

import sos.framework.simulation.SEnv;
import sos.framework.simulation.simul.results.SResultImage;
import sos.framework.simulation.simul.specs.SSpecImage;
import sos.lib.fitswriter.FitsFile;
import sos.lib.fitswriter.FitsHDU;
import sos.lib.fitswriter.FitsHeader;
import sos.lib.fitswriter.FitsHeaderCard;
import sos.lib.fitswriter.FitsImageData;
import sos.lib.logger.Logger;

public class ExportFITS
{
	

	
	public static final void export(final SEnv env)
	{
		try{
			// donnees a exporter
			boolean blNormalized = env.getPrefs().getExportNormalizedValues();
			boolean blRaw = env.getPrefs().getExportRawValues();
			boolean blAll = env.getPrefs().getExportAllImages();
			File file = new File(env.getPrefs().getExportFile());
			file.delete();
			
			// nouvelle image FITS
			FitsFile fits = new FitsFile();
			if(blAll){
				for(int i=0; i<env.getSimul().getImagesCount(); i++){
					exportFITSImage(fits, env.getSimul().getSpecs().getImageAt(i), env.getSimul().getResults().getImageAt(i), blNormalized, blRaw);
				}
			}else{
				exportFITSImage(fits, env.getSimul().getSpecs().getActualImage(), env.getSimul().getResults().getActualImage(), blNormalized, blRaw);
			}
			
			if( !fits.write(file) ){
				JOptionPane.showMessageDialog(env.getWindow(), "Erreur d'enregistrement : ?", "Exporter l'image au format FITS", JOptionPane.ERROR_MESSAGE);
			}
			
			
			JOptionPane.showMessageDialog(env.getWindow(), "Enregistrement terminé.", "Exporter l'image au format FITS", JOptionPane.INFORMATION_MESSAGE);

		}catch(Exception ex){
			Logger.fireError(ExportFITS.class.getName(), "exportFITS()", ex);
			JOptionPane.showMessageDialog(env.getWindow(), "Erreur d'enregistrement : " + ex.getMessage(), "Exporter l'image au format FITS", JOptionPane.ERROR_MESSAGE);
		}
	}
	
	private static final boolean exportFITSImage(final FitsFile fits, final SSpecImage image, final SResultImage res, final boolean expNorm, final boolean expRaw)
	{
		try{
			
			FitsHeader head = new FitsHeader();
			head.addCard( new FitsHeaderCard("NAXIS1", image.getImageWidth(), null) );
			head.addCard( new FitsHeaderCard("NAXIS2", image.getImageHeight(), null) );
			
			double[][] values = new double[image.getImageWidth()][image.getImageHeight()];
			
			if(expNorm){
				for(int x = 0; x < image.getImageWidth(); x++){
					for(int y = 0; y < image.getImageHeight(); y++){
						values[x][y] = res.getNormalizedValue(x, y);
					}
				}
				head.addCard( new FitsHeaderCard("DATAMIN", 0d, null) );
				head.addCard( new FitsHeaderCard("DATAMAX", 255d, null) );
			}
			if(expRaw){
				for(int x = 0; x < image.getImageWidth(); x++){
					for(int y = 0; y < image.getImageHeight(); y++){
						values[x][y] = res.getTotalElectrons(x, y);
					}
				}
				double max = res.computeMaxTotalElectrons();
				head.addCard( new FitsHeaderCard("DATAMIN", max * ( image.getNormalisationBlack() / 1000.0 ), null) );
				head.addCard( new FitsHeaderCard("DATAMAX", max * ( image.getNormalisationWhite() / 1000.0 ), null) );
			}
			FitsHDU hdu = new FitsHDU( head, new FitsImageData(values) );
			fits.addHDU(hdu);
			return true;
		}catch(Exception ex) {Logger.fireError(ExportFITS.class.getName(), "exportFITSImage()", ex);}
		return false;
	}
	
	
	
	
	
	
	
	
}
