/*
 SOS Project Classes.

 Copyright (C) 2006-2008 SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sos.framework.io;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;

import javax.swing.JOptionPane;

import sos.framework.SOSApplication;
import sos.framework.generator.SOSGenerator;
import sos.framework.simulation.SEnv;
import sos.framework.simulation.simul.specs.SSpecImage;
import sos.lib.logger.Logger;

public class ExportTXT
{
	

	

	public static final void export(final SEnv env)
	{
		try{

			// fichier de destination
			File file = new File( env.getPrefs().getExportFile() );
			file.delete();
	 		BufferedWriter w = new BufferedWriter(new FileWriter(file));

	 		// les donnees
	 		String s = exportTXTSpecs(env, env.getPrefs().getExportAllImages());

	 		// ecrit
	 		w.write(s);
	 		w.flush();
	 		w.close();
	 		
	 		JOptionPane.showMessageDialog(env.getWindow(), "Enregistrement terminé.", "Enregistrer au format TXT", JOptionPane.INFORMATION_MESSAGE);

		}catch(Exception ex){
			Logger.fireError(ExportTXT.class.getName(), "exportTXT()", ex);
			JOptionPane.showMessageDialog(env.getWindow(), "Erreur d'enregistrement : " + ex.getMessage(), "Enregistrer au format TXT", JOptionPane.ERROR_MESSAGE);	
		}
	}

	public static final String exportTXTSpecs(final SEnv env, final boolean blAllImages)
	{
		try{
			String str = " ------------------------------------------------------------\n"
                + "                    SOS Simulator v."+SOSApplication.APPLET_VERSION+"                   \n"
                + " ------------------------------------------------------------\n"
                + "          Copyright (c) 2006-2008 GNU Public Licence         \n"
                + "                <http://sos.project.free.fr>                 \n"
                + "              Denis BURGARELLA, Camille AMBROSI              \n"
                + " ------------------------------------------------------------\n"
                + "                                                             \n"
                + "                    Simulation results                       \n"
				+ "                                                             \n"
				+ "GENERAL INFORMATIONS                                         \n"
				+ getInfo("distribution", env.getSimul().getSpecs().getDistributionFunction().getName())
				+ "                                                             \n"
				;

			String strImage;
			if(blAllImages){
				for(int i=0; i<env.getSimul().getSpecs().getImages().size(); i++){
					if( (strImage = exportTXTImage(env.getSimul().getSpecs().getImageAt(i))) == null) return null;
					str += strImage;
				}
			}else{
				if( (strImage = exportTXTImage(env.getSimul().getSpecs().getActualImage())) == null) return null;
				str += strImage;
			}
			
			return str;
		}catch(Exception ex) {Logger.fireError(SOSGenerator.class.getName(), "exportTXTSpecs()", ex);}
		return null;
	}

	private static final String exportTXTImage(final SSpecImage image)
	{
		try{
			String str = "IMAGE                                                        \n"
				+ getInfo("name", image.getImageName())
				+ getInfo("pixel resolution", image.getPixelResolution() + " arcsec")
				+ getInfo("image size", image.getImageWidth() + " x " + image.getImageHeight() + " px")
				+ getInfo("image resolution", (image.getImageWidth()*image.getPixelResolution()) + " x " + (image.getImageHeight()*image.getPixelResolution()) + " arcsec")
				+ getInfo("filter", (image.getFilter().getLambdaCenter()*1e6) + " µm, delta : " + (image.getFilter().getLambdaDelta()*1e6) + " µm")
				+ getInfo("filter desc.", image.getFilter().getDescription())
				+ getInfo("normalization black", ""+image.getNormalisationBlack())
				+ getInfo("normalization white", ""+image.getNormalisationWhite())
				+ getInfo("generation limit", ""+image.getGenerationObjectsSize() + " px")
				+ "                                                             \n"
				;
			return str;
		}catch(Exception ex) {Logger.fireError(SOSGenerator.class.getName(), "exportTXTImage()", ex);}
		return null;
	}
	private static final String getInfo(final String strTitle, final String strValue)
	{
		String str = " " + strTitle + " ";
		try{
			for(int i=0; i<(58-strTitle.length()-strValue.length()); i++) str += ".";
			str = str + " " + strValue + "\n";
		}catch(Exception ex) {Logger.fireError(SOSGenerator.class.getName(), "getInfo()", ex);}
		return str;
	}

	
	
	
	
	
	
}
