/*
 SOS Project Classes.

 Copyright (C) SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package sos.framework.io;

import java.awt.Component;
import java.io.BufferedReader;
import java.io.File;
import java.net.URLDecoder;
import java.util.Enumeration;
import java.util.StringTokenizer;

import javax.swing.JOptionPane;

import sos.framework.ressources.SOSResources;
import sos.framework.simulation.simul.SSimul;
import sos.framework.simulation.simul.specs.SSpecImage;
import sos.framework.simulation.simul.specs.SSpecs;
import sos.framework.simulation.simul.specs.distrib.DistribFunction;
import sos.framework.simulation.simul.specs.filters.SFilter;
import sos.framework.simulation.simul.specs.noises.SNoise;
import sos.lib.logger.Logger;

public class ImportSOS
{
	
	


	public static final boolean load(final Component parent, final SSimul simul, final boolean blLoadAllFile, final String strFileName)
	{
		try{
			SSpecs specs = simul.getSpecs();	

			// demande un nom de fichier
			BufferedReader name = null;
			if(strFileName == null){
				// demande un fichier a l'utilisateur
				File filePath = SOSInputOutput.askForFileName(parent, "Ouvrir", "Ouvrir un fichier...");
				if(filePath == null) return false;
				name = SOSResources.getLocalFileReader( filePath.getPath() );
			}else{
				name = SOSResources.getJARExFileReader(strFileName);
			}
			if(name == null) return false;
			
			// lit le contenu du fichier
			StringTokenizer infos = loadDistribFileData(name);
			if(infos == null){
				JOptionPane.showMessageDialog(parent, "Le fichier choisi ne semble pas valide.", "Erreur de chargement", JOptionPane.ERROR_MESSAGE);	
				return false;
			}

			String data = infos.nextToken();
			int image = 0;
			if("[SSPECS]".equals(data)){
				
				// gestion de la version
				int version = Integer.parseInt( infos.nextToken() );
				if(version != 1){
					// mauvaise version
					JOptionPane.showMessageDialog(parent, "La version du fichier (v." + version + ") n'est pas compatiable avec cette version du simulateur (v.1).", "Erreur de chargement", JOptionPane.ERROR_MESSAGE);
					return false;
				}
				
				// type de chargement 
				if( blLoadAllFile ){
					// numero de l'image
					image = Integer.parseInt(infos.nextToken());
					if( image < 0 ){
						// pas d'image
						JOptionPane.showMessageDialog(parent, "Le fichier ne contient pas de description d'une simulation.", "Erreur de chargement", JOptionPane.ERROR_MESSAGE);
						return false;
					}
					// description de l'image
					simul.setSimulationDescription(URLDecoder.decode( infos.nextToken(), "UTF8" ));
					if(simul.getSimulationDescription().equals("[NULL]")) simul.setSimulationDescription("");
				}
				
			}else{
				// mauvais fichier
				JOptionPane.showMessageDialog(parent, "Le fichier n'est pas une simulation SOS valide.", "Erreur de chargement", JOptionPane.ERROR_MESSAGE);
				return false;
			}

			while( infos.hasMoreTokens() ){
				data = infos.nextToken();
				if( "[NOISE]".equals(data) && blLoadAllFile ) loadNoise(specs, infos);
				else if( "[FILTER]".equals(data) && blLoadAllFile ) loadFilter(specs, infos);
				else if( "[VARIABLE]".equals(data) && blLoadAllFile ) loadVariable(specs, infos);
				else if( "[IMAGE]".equals(data) && blLoadAllFile ) loadImage(simul, infos);
				else if( "[COLORATION]".equals(data) && blLoadAllFile ) loadColoratedImage(specs, infos);
				else if( "[FUNCTION]".equals(data) && blLoadAllFile ) loadDistributionFunction(specs, infos);
				else if( "[DISTRIBUTION]".equals(data) && blLoadAllFile ) loadDistributionMode(specs, infos);
				else if( "[SOURCES]".equals(data) ) loadSources(specs, infos);
			}

			if( blLoadAllFile ){
				simul.setActualImageNumber(image);
			}else{
				if(specs.getSourcesContainer().getSourcesCount() < 1){
					JOptionPane.showMessageDialog(parent, "Le fichier ne contient aucune source.", "Chargement", JOptionPane.INFORMATION_MESSAGE);
				}else{
					simul.fireSimulationComponentsChanged();
				}
			}
			return true;
		
		}catch(Exception ex){
			Logger.fireError(ImportSOS.class.getName(), "load()", ex);
			Logger.notifyError(parent, "Erreur de chargement : " + ex.getMessage(), "Charger une simulation");
		}
		return false;
	}
	
	
	
	
	
	
	private static final StringTokenizer loadDistribFileData(final BufferedReader reader) throws Exception
	{
		if(reader != null){
			// lit les donnees
			String line = null;
			StringBuffer lines = new StringBuffer("");
			while( (line = reader.readLine()) != null ){
				if( (line.length() > 0) && (line.startsWith("#") == false) ){
					// positions des sources
					if( line.startsWith("[") && line.endsWith("]") ) lines.append(line + "\n");
					// info
					else if( (line.length() > 8) && (line.charAt(8) == ' ') ) lines.append(line.substring(9) + "\n");
				}
			}
			if(lines.length() > 0) return new StringTokenizer(lines.toString(), "\n");
		}
		return null;
	}
	

	
	
	
	
	
	
	
	

	
	private static final void loadNoise(final SSpecs specs, final StringTokenizer infos) throws Exception
	{
		String name = URLDecoder.decode( infos.nextToken(), "UTF8" );
		boolean each = new Boolean( infos.nextToken() ).booleanValue();
		String func = URLDecoder.decode( infos.nextToken(), "UTF8" );
		specs.getNoises().add(new SNoise(name, func, each));
	}
	
	private static final void loadFilter(final SSpecs specs, final StringTokenizer infos) throws Exception
	{
		String desc = URLDecoder.decode(infos.nextToken(), "UTF8");
		double center = Double.parseDouble(infos.nextToken());
		double delta = Double.parseDouble(infos.nextToken());
		specs.getFilters().add(new SFilter(desc, center, delta));
	}
	
	private static final void loadVariable(final SSpecs specs, final StringTokenizer infos) throws Exception
	{
		String name = URLDecoder.decode( infos.nextToken(), "UTF8");
		String desc = URLDecoder.decode( infos.nextToken(), "UTF8");
		specs.updateVariable(name, desc, Double.NaN);
	}
	
	private static final void loadImage(final SSimul simul, final StringTokenizer infos) throws Exception
	{
		SSpecImage image = new SSpecImage(simul);
		image.setImageSizeX(Integer.parseInt(infos.nextToken()));
		image.setImageSizeY(Integer.parseInt(infos.nextToken()));
		image.setPixelResolution(Double.parseDouble(infos.nextToken()));
		image.setGenerationObjectsSize(Integer.parseInt(infos.nextToken()));
		image.setNormalisationWhite(Integer.parseInt(infos.nextToken()));
		image.setNormalisationBlack(Integer.parseInt(infos.nextToken()));

		image.setFilter( (SFilter)(simul.getSpecs().getFilters().get(Integer.parseInt(infos.nextToken()))) );

		for(Enumeration e=simul.getSpecs().getVariableNames(); e.hasMoreElements(); ) image.updateVariable( (String)(e.nextElement()), new Double( infos.nextToken()).doubleValue() );

		int noises = Integer.parseInt(infos.nextToken());
		for(int i=0; i<noises; i++) image.getNoises().add( (SNoise)(simul.getSpecs().getNoises().get(Integer.parseInt(infos.nextToken()))) );
		
		simul.addImage(image);
	}
	
	private static final void loadColoratedImage(final SSpecs specs, final StringTokenizer infos) throws Exception
	{
		specs.setColoratedImageRed( Integer.parseInt( infos.nextToken() ) );
		specs.setColoratedImageGreen( Integer.parseInt( infos.nextToken() ) );
		specs.setColoratedImageBlue( Integer.parseInt( infos.nextToken() ) );
	}
	
	private static final void loadDistributionFunction(final SSpecs specs, final StringTokenizer infos) throws Exception
	{
		String name = URLDecoder.decode(infos.nextToken(), "UTF8");
		String func = URLDecoder.decode(infos.nextToken(), "UTF8");
		specs.getDistributionFunctions().add(new DistribFunction(name, func));
	}
	
	private static final void loadDistributionMode(final SSpecs specs, final StringTokenizer infos) throws Exception
	{
		specs.setDistributionFunction( (DistribFunction)(specs.getDistributionFunctions().get(Integer.parseInt(infos.nextToken()))) );
		specs.setUseGenerator( new Boolean( infos.nextToken() ).booleanValue() );
	}
	
	private static final void loadSources(final SSpecs specs, final StringTokenizer infos) throws Exception
	{
		String origin = URLDecoder.decode(infos.nextToken(), "UTF8");
		int number = Integer.parseInt(infos.nextToken());
		specs.getSourcesContainer().clearSources();
		specs.getSourcesContainer().setSourcesOrigin(origin);
		for(int i=0; i<number; i++) specs.getSourcesContainer().addSource(infos.nextToken());
		// si des sources ont ete distribuees, on bloque la distribution
		if(specs.getSourcesContainer().getSourcesCount() > 0) specs.setUseGenerator(false);
	}
	

	

}
