/*
 SOS Project Classes.

 Copyright (C) 2006-2008 SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sos.framework.maths;


public class Geometry
{

	
	


	/**
	 * Calcule la distance entre deux point.
	 * 
	 * @param x1 Position X du point 1.
	 * @param y1 Position Y du point 1.
	 * @param x2 Position X du point 2.
	 * @param y2 Position Y du point 2.
	 * 
	 * @return La distance euclidienne entre le point 1 et le point 2, ou un
	 *         nombre négatif en cas d'erreur.
	 */
	public static final double euclidianDistance(final double x1, final double y1, final double x2, final double y2)
	{
		try{
			/* calcul classique d'une distance entre deux points */
			return Math.sqrt((x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1));
		}catch (Exception ex) {}
		return 0.0;
	}

	
	/**
	 * Calcule le rayon en fonction de x et y dans une géométrie axisymétrique.
	 * 
	 * @param x
	 * @param y
	 * @param PAx
	 * @param Qx
	 * @return
	 */
	public static final double axisymRay(final double x, final double y, final double PAx, final double Qx)
	{
		try{
			return Math.sqrt(Math.pow(x * Math.cos(PAx) + y * Math.sin(PAx), 2.0) / Math.pow(Qx, 2.0) + Math.pow(y * Math.cos(PAx) - x * Math.sin(PAx), 2.0));
		}catch (Exception ex) {}
		return 0.0;
	}

	
	
	
	
	
	
}
