/*
 SOS Project Classes.

 Copyright (C) 2006-2008 SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sos.framework.maths;

import java.text.DecimalFormat;

/**
 * Classe contenant diverses formules mathematiques utiles.
 * 
 * @author Camille AMBROSI
 *
 */
public class Maths
{

	
	
	private static final DecimalFormat[] formaters = new DecimalFormat[] {
		new DecimalFormat("#.#"),
		new DecimalFormat("0E0"),
		new DecimalFormat("0.0E0"),
		new DecimalFormat("0.00E0"),
		new DecimalFormat("0.000E0"),
		new DecimalFormat("0.0000E0"),
		new DecimalFormat("0.00000E0"),
		new DecimalFormat("0.000000E0")
	};

	public static final String toString(final double d) {return toString(d, 3);}
	public static final String toString(final double d, int iNbOfChars)
	{
		try{
			if(Double.isNaN(d)) return "NaN";
			if(iNbOfChars < -1) iNbOfChars = -1;
			if(iNbOfChars > 6) iNbOfChars = 6;
			return formaters[iNbOfChars+1].format(d).toLowerCase().replace(',', '.');
		}catch (Exception ex) {ex.printStackTrace();}
		return ""+d;
	}

	
	
	
	
	
	
	
	

	
	
	
	
	public static final double log(final double a)
	{
		if( Double.isNaN(a) ) return a;
		return Math.log(a) / Math.log(10);
	}

	
	
	public static final double ln(final double a)
	{
		return Math.log(a);
	}

	
	
	public static final double pow(final double a, final double b)
	{
		return Math.pow(a, b);
	}

	
	
	
	
	
	
	public static final double planck(final double lambda, final double T)
	{
		try{
			double v = Constants.C / lambda;
			return (2.0 * Constants.H * Math.pow(v, 3.0)) / (Math.pow(Constants.C, 2.0) * (Math.exp(((Constants.H * v) / (Constants.K * T))) - 1.0));
		}catch(Exception ex) {}
		return Double.NaN;
	}
	public static final double planck2(final double lambda, final double T)
	{
		try{
			double c1 = 3.7418310e-16; // 2 × π × h × c^2
			double c2 = 1.43879e-2; // h × c / k
			return (c1 * Math.pow(lambda, -5.0)) / (Constants.PI * Math.exp(c2 / (lambda * T)) - 1.0);
		}catch(Exception ex) {}
		return Double.NaN;
	}

	
	
	
	
	
	
	public static double gauss(final double x, final double mu, final double ro)
	{
		try{
			return Math.exp(-0.5 * Math.pow((x - mu) / ro, 2.0)) / (ro * Math.sqrt(2.0 * Constants.PI));
		}catch(Exception ex) {}
		return Double.NaN;
	}

	
	
	public static double poisson(final int x, final double lambda)
	{
		try{
			return Math.exp(-lambda) * Math.pow(lambda, x) / factorial(x);
		}catch(Exception ex) {}
		return Double.NaN;
	}

	public static final double factorial(final int n)
	{
		try{
			if (n < 0) return 0;
			if (n < 2) return 1;
			long fact = 2;
			for(int i = 3; i <= n; i++) fact = fact * i;
			return fact;
		}catch(Exception ex) {}
		return Double.NaN;
	}

	
	
	
}
