/*
 SOS Project Classes.

 Copyright (C) 2006-2008 SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package sos.framework.maths;

/**
 * Classe permettant d'obtenir facilement diverses valeurs aleatoires.
 * 
 * @author Camille AMBROSI
 *
 */
public class Random
{

	
	
	/**
	 * Retourne un nombre aleatoire, compris entre <code>iMin</code> et 
	 * <code>iMax</code>.
	 * 
	 * @param dblMin La borne inferieure
	 * @param dblMax La borne superieure
	 * @return Le nombre aleatoire tire
	 */
	public static final double randomDouble(double dblMin, double dblMax)
	{
		if( dblMin > dblMax ){
			double tmp = dblMin;
			dblMin = dblMax;
			dblMax = tmp;
		}
		return ((Math.random() * (dblMax - dblMin)) + dblMin);
	}

	
	/**
	 * Retourne un entier aleatoire, compris entre <code>iMin</code> et 
	 * <code>iMax</code>.
	 * 
	 * @param iMin La borne inferieure
	 * @param iMax La borne superieure
	 * @return Le nombre aleatoire tire
	 */
	public static final int randomInt(int iMin, int iMax)
	{
		if( iMin > iMax ){
			int tmp = iMin;
			iMin = iMax;
			iMax = tmp;
		}
		return (int)((Math.random() * (iMax - iMin)) + iMin);
	}

	
	/**
	 * Retourne un nombre aleatoire, issu d'une variable aleatoire 
	 * dont la fonction de repartition est poissonienne.
	 * 
	 * @param c Le parametre
	 * @return Un entier aleatoire suivant une distribution de poisson
	 */
	public static int randomPoisson(double c)
	{
		int x = 0;
		double t = 0.0;
		for(;;){
			t = t - Math.log(Math.random()) / c;
			if (t > 1.0)
			break;
			x = x + 1;
		}
		return x;
	}

	
	/**
	 * Retourne un nombre aleatoire, issu d'une variable aleatoire 
	 * dont la fonction de repartition est Gaussienne, en utilisant la
	 * methode de Monte-Carlo.
	 * 
	 * @param ro Le parametre
	 * @return Un entier aleatoire suivant une distribution de gauss
	 */
	public static double randomGaussMC(double ro)
	{
		double rnd = 0.0;
		double max = Maths.gauss(0, 0, ro);
		do{
			rnd = Random.randomDouble(-4.0 * ro, +4.0 * ro);
		}while(Maths.gauss((rnd < 0 ? -rnd : rnd), 0.0, ro) < Random.randomDouble(0, max));
		return rnd;
	}


	
	

}
