/*
 SOS Project Classes.

 Copyright (C) 2006-2008 SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sos.framework.maths.conversions;

public class SUnit
{
	
	public static final int UNIT_PERCENT_ID		    	= 0;
	public static final int UNIT_ELECT_PER_PIXEL_ID 	= 1;
	public static final int UNIT_ELECT_PER_STERAD_ID	= 2;
	public static final int UNIT_JY_PER_PIXEL_ID     	= 3;
	public static final int UNIT_JY_PER_STERAD_ID    	= 4;
	
	
	public static final SUnit UNIT_ELECT_PER_PIXEL 		= new SUnit(UNIT_ELECT_PER_PIXEL_ID, "e-/pix", "Elect/Pixel");
	public static final SUnit UNIT_ELECT_PER_STERAD 	= new SUnit(UNIT_ELECT_PER_STERAD_ID, "e-/sterad", "Elect/Steradian");
	public static final SUnit UNIT_JY_PER_PIXEL    		= new SUnit(UNIT_JY_PER_PIXEL_ID, "jy/pix", "Jy/Pixel");
	public static final SUnit UNIT_JY_PER_STERAD    	= new SUnit(UNIT_JY_PER_STERAD_ID, "jy/sterad", "Jy/Steradian");
	public static final SUnit UNIT_PERCENT		    	= new SUnit(UNIT_PERCENT_ID, "%", "Pourcentage");
	
	
	
	
	public final static SUnit getUnitFromID(final int iUnitID)
	{
		switch(iUnitID){
			case UNIT_ELECT_PER_PIXEL_ID: return UNIT_ELECT_PER_PIXEL;
			case UNIT_ELECT_PER_STERAD_ID: return UNIT_ELECT_PER_STERAD;
			case UNIT_JY_PER_PIXEL_ID: return UNIT_JY_PER_PIXEL;
			case UNIT_JY_PER_STERAD_ID: return UNIT_JY_PER_STERAD;
		}
		return null;
	}
	
	
	
	
	
	
	
	
	
	private int id								= 0;
	
	private String strAbrev						= "";
	
	private String strDesc						= "";
	
	
	
	
	private SUnit(final int id, final String strAbrev, final String strDesc)
	{
		this.id = id;
		this.strAbrev = strAbrev;
		this.strDesc = strDesc;
	}
	
	public final int getID() {return id;}
	
	public final String getAbbrev() {return strAbrev;}
	
	public final String getDescription() {return strDesc;}
	
	public final String toString() {return " " + getAbbrev() + " (" + getDescription() + ")";}

	

	
	
	
}
