/*
 SOS Project Classes.

 Copyright (C) 2006-2008 SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sos.framework.maths.conversions.flux;

import sos.framework.maths.Constants;
import sos.framework.simulation.simul.SSimul;
import sos.framework.simulation.simul.specs.SSpecImage;

public class FromElect
{

	/**
	 * Convertie un flux en Electron en un flux en Jy.
	 * 
	 * @param flux_Elec Le flux exprime en Electron
	 * @param lambda_M La bande d'observation en metre
	 * @param dlambda_M La largeur de la bande d'observation en metre
	 * @param t_Sec La duree de l'observation en seconde
	 * @param diamTel_M Le diametre du telescope en metre
	 * @return Le flux exprime en Jy
	 */
	public static final double toJy(double flux_Elec, final SSpecImage image)
	{
		double lambda = image.getFilter().getLambdaCenter();
		double tdiam = image.getVariableValue(SSimul.VAR_TELESCOPE_DIAMETER_NAME);
		double tpause = image.getVariableValue(SSimul.VAR_OBSERVATION_TIME_NAME);
		double Atel = Constants.PI * tdiam * tdiam / 4.0;
		double dnu = (Constants.C * image.getFilter().getLambdaDelta()) / (lambda * lambda);
		return flux_Elec / 1.0e-26 / tpause / dnu / Atel / lambda * Constants.HC;
	}
	
	public static final double factorToJy(final SSpecImage image)
	{
		double lambda = image.getFilter().getLambdaCenter();
		double tdiam = image.getVariableValue(SSimul.VAR_TELESCOPE_DIAMETER_NAME);
		double tpause = image.getVariableValue(SSimul.VAR_OBSERVATION_TIME_NAME);
		double Atel = Constants.PI * tdiam * tdiam / 4.0;
		double dnu = (Constants.C * image.getFilter().getLambdaDelta()) / (lambda * lambda);
		return 1 / 1.0e-26 / tpause / dnu / Atel / lambda * Constants.HC;
	}
	
	
	

}
