/*
 SOS Project Classes.

 Copyright (C) SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package sos.framework.simulation.simul.specs;

import java.util.Hashtable;

import sos.framework.simulation.simul.SSimul;
import sos.framework.simulation.simul.specs.filters.SFilter;
import sos.framework.simulation.simul.specs.noises.SSpecNoises;
import sos.lib._SModificationListener;
import sos.lib.logger.Logger;

public class SSpecImage implements _SModificationListener
{

	private static final long serialVersionUID 				= 1L;

	private SSimul simul									= null;

	
	
	
	
	public SSpecImage(final SSimul simul)
	{
		try{
			this.simul = simul;
			this.noises = new SSpecNoises(simul, this);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "SImage()", ex);}
	}
	
	
	
	
	

	public final SSpecImage getCloneImage()
	{
		try{
			SSpecImage img = new SSpecImage(simul);
			
			img.iSizeX = iSizeX;
			img.iSizeY = iSizeY;
			img.dblPixelResolution = dblPixelResolution;
			
			img.iGenerationObjectsSize = iGenerationObjectsSize;
		
			img.iNormWhitePerMil = iNormWhitePerMil;
			img.iNormBlackPerMil = iNormBlackPerMil;
		 	
			img.lstVariableValues = (Hashtable)(lstVariableValues.clone());

			// set le filtre (et s'ajoute dans les listeners)
			img.setFilter(filter);
			
			// set les bruits (et s'ajoute dans les listeners)
			for(int i=0; i<this.getNoises().size(); i++) img.getNoises().add(this.getNoises().get(i));
			
			return img;
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "getCloneImage()", ex);}
		return null;
	}
	
	public final void destroyImage()
	{
		try{
			lstVariableValues.clear();
			if(filter != null) filter.removeModificationListener(this);
			for(int i=noises.size()-1; i>-1; i--) noises.remove(i);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "destroyImage()", ex);}
	}
	
	public final boolean checkImage()
	{
		try{
			
			
			
			return true;
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "checkImage()", ex);}
		return false;
	}
	
	
	
	
	
	
	

	
	private int iImageNumber									= -1;
	
	public final int getImageNumber() {return iImageNumber;}
	
	public final void setImageNumber(final int iImageNumber) {this.iImageNumber = iImageNumber;}
	
	
	
	
	
	
	public final String getImageName() {return "Image " + (iImageNumber+1);}
	
	public final String toString() {return getImageName();}
	
	
	

	
	
	
	

// BRUITS //


	private SSpecNoises noises	 								= null;
		
	public final SSpecNoises getNoises() {return noises;}
		
		
		
		

	
	
	
	
	
// FILTRE //
	 	
	
	// filtre
	private SFilter filter										= null;									

	public final SFilter getFilter() {return filter;}
	
	public final void setFilter(final SFilter filter)
	{
		try{
			// quitte si les deux filtres sont nuls
			if( (this.filter == null) && (filter == null) ) return;
			// quitte si les deux filtres sont egaux
			if( (this.filter != null) && (this.filter.equals(filter)) ) return;
			// s'il y avait un filtre utilisé, on se detache
			if(this.filter != null) this.filter.removeModificationListener(this);
			// set le nouveau filtre
			this.filter = filter;
			// si on a mis un nouveau filtre, on s'attache
			if(this.filter != null) this.filter.addModificationListener(this);
			// on nottifie la modification
			simul.fireImageSpecsChanged(iImageNumber, SSimul.STAGE_GENERATION);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "setFilter()", ex);}
	}
	
	public void onModificated(final Object object)
	{
		if( object == filter ){
			// notre filtre vient d'etre modifie
			simul.fireImageSpecsChanged(iImageNumber, SSimul.STAGE_GENERATION);
		}
	}
	

	 	
			
	
	
	
		

	
	
 	
// TAILLE //

	 	
	// taille de l'image
	private int iSizeX											= 0;
	private int iSizeY											= 0;
	
	public final int getImageWidth() {return iSizeX;}
	
	public final void setImageSizeX(final int iSizeX)
	{
		if(this.iSizeX != iSizeX){
 			this.iSizeX = iSizeX;
 			simul.fireImageSpecsChanged(iImageNumber, SSimul.STAGE_DISTRIBUTION);
 		}
	}
	
	public final int getImageHeight() {return iSizeY;}
		
	public final void setImageSizeY(final int iSizeY)
	{
		if(this.iSizeY != iSizeY){
 			this.iSizeY = iSizeY;
 			simul.fireImageSpecsChanged(iImageNumber, SSimul.STAGE_DISTRIBUTION);
 		}
	}
	
	private double dblPixelResolution							= 0.0;
	
	public final double getPixelResolution() {return dblPixelResolution;}
	
	public final void setPixelResolution(final double dblPixelResolution)
	{
		if(this.dblPixelResolution != dblPixelResolution){
 			this.dblPixelResolution = dblPixelResolution;
 			simul.fireImageSpecsChanged(iImageNumber, SSimul.STAGE_DISTRIBUTION);
 		}
	}
	
	
	
	
	
	
// PROPRIETES //
	

 	// taille de la zone de geneartion d'un objet
 	private int iGenerationObjectsSize							= 40;
 	
 	public final int getGenerationObjectsSize() {return iGenerationObjectsSize;}
 	
 	public final void setGenerationObjectsSize(final int iGenerationObjectsSize)
 	{
 		if(this.iGenerationObjectsSize != iGenerationObjectsSize){
 			this.iGenerationObjectsSize = iGenerationObjectsSize;
 			simul.fireImageSpecsChanged(iImageNumber, SSimul.STAGE_GENERATION);
 		}
 	}

 
 	
 	
 	
 	
 	

 	
// NORMALISATION //
  	
 	// valeur du blanc (pour mille)
  	private int iNormWhitePerMil								= 1000;

  	public final int getNormalisationWhite() {return iNormWhitePerMil;}
  	
  	public final void setNormalisationWhite(final int iNormWhitePerMil)
  	{
  		if(this.iNormWhitePerMil != iNormWhitePerMil){
  			this.iNormWhitePerMil = iNormWhitePerMil;
  			simul.fireImageSpecsChanged(iImageNumber, SSimul.STAGE_NORMALISATION);
  		}
  	}
  	
 	// valeur du noir (pour mille)
  	private int iNormBlackPerMil								= 0;

  	public final int getNormalisationBlack() {return iNormBlackPerMil;}
  	
  	public final void setNormalisationBlack(final int iNormBlackPerMil)
  	{
  		if(this.iNormBlackPerMil != iNormBlackPerMil){
  			this.iNormBlackPerMil = iNormBlackPerMil;
  			simul.fireImageSpecsChanged(iImageNumber, SSimul.STAGE_NORMALISATION);
  		}
  	}
  	
  	


 	
 	
 	
 	
 		
// VARIABLES //
	
	
	/* la liste des variables */
	private Hashtable lstVariableValues						= new Hashtable();
	
	public final void updateVariable(final String strVarName, final double dblValue)
	{
		try{
			if( lstVariableValues.containsKey(strVarName) ){
				if(getVariableValue(strVarName) == dblValue) return;
				removeVariable(strVarName);
			}
			lstVariableValues.put(strVarName, new Double(dblValue));
			simul.fireImageSpecsChanged(iImageNumber, SSimul.STAGE_GENERATION);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "updateVariable()", ex);}
	}
	
	public final void removeVariable(final String strVarName)
	{
		try{
			if(lstVariableValues.containsKey(strVarName)){
				lstVariableValues.remove(strVarName);
				simul.fireImageSpecsChanged(iImageNumber, SSimul.STAGE_GENERATION);
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "removeVariable()", ex);}
	}
	
	public final double getVariableValue(final String strVarName)
	{
		try{
			return ((Double)(lstVariableValues.get(strVarName))).doubleValue();
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "getVariableValue()", ex);}
		return Double.NaN;
	}
		
	

	
	
	
	
	



}
