/*
 SOS Project Classes.

 Copyright (C) SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package sos.framework.simulation.simul.specs;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import sos.framework.simulation.simul.SSimul;
import sos.framework.simulation.simul.specs.distrib.DistribFunction;
import sos.framework.simulation.simul.specs.sources.SSourcesContainer;
import sos.lib.logger.Logger;

public class SSpecs
{

	private SSimul simul									= null;

	
	
	
	
	
	
	public SSpecs(final SSimul simul)
	{
		this.simul = simul;
	}
	
	public final void dispose()
	{
		try{
			simul = null;
			lstNoises.clear(); lstNoises = null;
			lstFilters.clear(); lstFilters = null;
			lstVariableNames.clear(); lstVariableNames = null;
			lstDistributionFunctions.clear(); lstDistributionFunctions = null;
			sourceContainer = null;
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "dispose()", ex);}
	}
	

	
	
	
	

	
	
// IMAGES //
		
		
	// la liste des images de la simulation
	private Vector lstImages								= new Vector();
	
	public final Vector getImages() {return lstImages;}
	
	// retourne une image
	public final SSpecImage getImageAt(final int i)
	{
		try{
			return (SSpecImage)(lstImages.get(i));
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "getImageAt()", ex);}
		return null;
	}
	
	public final SSpecImage getActualImage()
	{
		try{
			return (SSpecImage)(lstImages.get(simul.getActualImageNumber()));
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "getImageAt()", ex);}
		return null;
	}


	
	
	
	
// IMAGE COULEUR //
	

	private int imageRed											= -1;
	public final int getColoratedImageRed() {return imageRed;}
	public final void setColoratedImageRed(final int imageRed) {this.imageRed = imageRed;}
	
	private int imageGreen											= -1;
	public final int getColoratedImageGreen() {return imageGreen;}
	public final void setColoratedImageGreen(final int imageGreen) {this.imageGreen = imageGreen;}

	private int imageBlue											= -1;
	public final int getColoratedImageBlue() {return imageBlue;}
	public final void setColoratedImageBlue(final int imageBlue) {this.imageBlue = imageBlue;}
		
		

		
	
	
	
	
	
// VARIABLES //


	private Hashtable lstVariableNames							= new Hashtable();
	
	public void updateVariable(final String strVarName, final String strVarDescription, final double dblValue)
	{
		try{
			if(!lstVariableNames.contains(strVarName)) lstVariableNames.put(strVarName, strVarDescription);
			for(int i=0; i<lstImages.size(); i++) getImageAt(i).updateVariable(strVarName, dblValue);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "updateVariable()", ex);}
	}
	
	public void removeVariable(final String strVarName)
	{
		try{
			if(lstVariableNames.contains(strVarName)) lstVariableNames.remove(strVarName);
			for(int i=0; i<lstImages.size(); i++) getImageAt(i).removeVariable(strVarName);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "removeVariable()", ex);}
	}
	
	public String getVariableDescription(final String strVarName)
	{
		try{
			return (String)(lstVariableNames.get(strVarName));
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "getVariableDescription()", ex);}
		return "";
	}
	
	public Enumeration getVariableNames()
	{
		try{
			return lstVariableNames.keys();
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "getVariableNames()", ex);}
		return null;
	}
	

	
	
	
	
	
// FILTRES //
	
	
	// les filtres des observations
	private Vector lstFilters	 							= new Vector();
	
	// les filtres des observations
	public final Vector getFilters() {return lstFilters;}
	public final void setFilters(final Vector lstFilters) {this.lstFilters = lstFilters;}



	
	
	
// TYPE DE NORMALISATION //
	
  	// type de normalisation
  	private boolean blIndepNorm								= false;

  	public final boolean getIndependentNormalisation() {return blIndepNorm;}
  	
  	public final void setIndependentNormalisationk(final boolean blIndepNorm)
  	{
  		if(this.blIndepNorm != blIndepNorm){
  			this.blIndepNorm = blIndepNorm;
  			for(int i=0; i<lstImages.size(); i++) simul.fireImageSpecsChanged(i, SSimul.STAGE_NORMALISATION);
  		}
  	}

  	
	
  	
  	
  	

   	
	
// BRUITS //

	
	/* les bruits */
	private Vector lstNoises	 							= new Vector();
	
	public final Vector getNoises() {return lstNoises;}


	
	
	
	

	
	
	
	
// DISTRIBUTION DES SOURECS //
	
	private SSourcesContainer sourceContainer				= new SSourcesContainer();
	
	public final SSourcesContainer getSourcesContainer() {return sourceContainer;}
	
	

	private Vector lstDistributionFunctions					= new Vector();
	
	public final Vector getDistributionFunctions() {return lstDistributionFunctions;}
	
	
	
	private boolean blUseGenerator							= true;
	
	public final boolean isUseGenerator() {return blUseGenerator;}
	
	public final void setUseGenerator(final boolean blUseGenerator) {this.blUseGenerator = blUseGenerator;}
	
	

	
	private DistribFunction distribFunction						= null;

	public final void setDistributionFunction(final DistribFunction distribFunction)
	{
		if( (this.distribFunction == null) && (distribFunction == null) ) return;
		if( (this.distribFunction != null) && (this.distribFunction.equals(distribFunction)) ) return;
		this.distribFunction = distribFunction;
		for(int i=0; i<lstImages.size(); i++) simul.fireImageSpecsChanged(i, SSimul.STAGE_DISTRIBUTION);
	}
	
	public final DistribFunction getDistributionFunction() {return distribFunction;}





}

