/*
  SOS Project Classes.

  Copyright (C) SOS Project <http://sos.project.free.fr>
  SOS Project is :
    Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>
    Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package sos.framework.simulation.simul.specs.distrib;

import java.util.StringTokenizer;
import java.util.Vector;

import sos.framework.maths.Maths;
import sos.lib.SOSReplace;
import sos.lib.logger.Logger;
import sos.lib.uicommon.graphics.SFunction;
import sos.lib.uicommon.graphics.SPointsFunction;


public class DistribFunction
{
	
	public static final int NB_IND								= 4;
	public static final int FL1									= 0;
	public static final int FL2									= 1;
	public static final int NB1									= 2;
	public static final int NB2									= 3;
	

	// pour les points
	private String strPoints									= null;
	private Vector blocs										= new Vector();
	
	// le nom de la fonction
	private String strName										= "";
	
	
	
	

	
	public DistribFunction(final String strName, final String strPoints)
	{
		this.strName = strName;
		this.setPoints(strPoints);
	}



	
	public final void setPoints(final String strNewPoints)
	{
		try{
			// efface les anciennes donnees
			blocs.clear();
			
			// nouveau points
			strPoints = strNewPoints;
			if((strPoints == null) || strPoints.length() < 1) return;
			String cleanPoints = strPoints.replace('\t', ' ');
			
			// enleve les doubles espaces
			while(cleanPoints.indexOf("  ") > 0){
				cleanPoints = SOSReplace.replace(cleanPoints, "  ", " ");	
			}
			
			// lit les donnees
			StringTokenizer st = new StringTokenizer(cleanPoints, "\n");
			String strBloc;
			String[] lineStrings;
			double[] lineDoubles = null, previousLine = null, realBloc = null;
			while( st.hasMoreTokens() )
			{
				try{
					strBloc = st.nextToken().trim();
					// regarde si c'est un commentaire
					if(strBloc.startsWith("#") == false){
						lineStrings = strBloc.split(" ");
						if( lineStrings.length >= 2 ){
							lineDoubles = new double[2];
							lineDoubles[0] = Maths.pow( 10.0 , Double.parseDouble( lineStrings[0] ) );
							lineDoubles[1] = Double.parseDouble( lineStrings[1] );
							// construit le vrai bloc
							if( previousLine != null ){
								realBloc = new double[NB_IND];
								realBloc[FL1] = previousLine[0];
								realBloc[FL2] = lineDoubles[0];
								realBloc[NB1] = previousLine[1];
								realBloc[NB2] = lineDoubles[1];
								blocs.add(realBloc);
							}		
							previousLine = lineDoubles;
						}	
					}
				}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "setPoints()*", ex);}
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "setPoints()", ex);}
	}
	
	
	
	
	public final String getName() {return strName;}
	
	public final void setName(final String strName) {this.strName = strName;}

	
	
	
	
	public final String toString() {return getName();}
	
	
	
	

	public final String getPoints() {return strPoints;}

	
	
	
	
	public final int getBlocsNb()
	{
		return blocs.size();
	}
	
	public final double[] getBlocAt(final int i)
	{
		return (double[])(blocs.get(i));
	}
	
	
	

	

	
	public final SFunction generateFunction()
	{
		if(blocs.size() == 0) return new SPointsFunction("", "", null);
		String strPoints = "[" + getBlocAt(0)[FL1] + ";NaN]";
		double[] bloc;
		for(int i=0; i<blocs.size(); i++)
		{
			bloc = getBlocAt(i);
			strPoints  = strPoints + "[" + bloc[FL1] + ";" + bloc[NB1] + "]";
			strPoints  = strPoints + "[" + bloc[FL2] + ";" + bloc[NB2] + "]";
		}
		strPoints = strPoints + "[" + getBlocAt(blocs.size()-1)[FL2] + ";NaN]";
		SPointsFunction f = new SPointsFunction("", "", SPointsFunction.LOG_POINTS_FUNCTION +   strPoints);
		return f;
	}
	

	
	


}
