/*
 SOS Project Classes.

 Copyright (C) 2006-2008 SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sos.framework.simulation.simul.specs.noises;

import java.util.Vector;

import sos.framework.simulation.simul.SSimul;
import sos.framework.simulation.simul.specs.SSpecImage;
import sos.lib._SModificationListener;
import sos.lib.logger.Logger;

public class SSpecNoises implements _SModificationListener
{

	private static final long serialVersionUID 				= 1L;

	
	

	private SSimul simul	 							= null;
	
	private SSpecImage image	 							= null;

	public SSpecNoises(final SSimul simul, final SSpecImage image)
	{
		this.simul = simul;
		this.image = image;
	}
	
	
	

	

	
// BRUITS //

	/* les bruits */
	private Vector lstNoises	 							= new Vector();
	


	public final int size() {return lstNoises.size();}
	public final int indexOf(final SNoise noise) {return lstNoises.indexOf(noise);}
	public final boolean contains(final SNoise noise) {return lstNoises.contains(noise);}
	
	
	public final void add(final SNoise noise) {add(-1, noise);}

	public final void add(final int i, final SNoise noise)
	{
		try{
			// on verifie que ce bruit n'est pas selectionne
			if( lstNoises.contains(noise) == false ){
				// ajoute le bruit a la bonne position
				if(i < 0) lstNoises.add(noise);
				else lstNoises.add(i, noise);
				// on s'attache au bruit
				noise.addModificatioListener(this);
				// on nottifie qu'il y a un nouveau bruit
				simul.fireImageSpecsChanged(image.getImageNumber(), SSimul.STAGE_NOISES);
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "add()", ex);}
	}	

	public final SNoise remove(final int i)
	{
		try{
			// enleve le bruit de notre liste
			SNoise noise = (SNoise)(lstNoises.remove(i));
			// on se detache
			noise.removeModificationListener(this);
			// on nottifie du changement
			simul.fireImageSpecsChanged(image.getImageNumber(), SSimul.STAGE_NOISES);
			// on retourne le bruit enleve
			return noise;
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "remove()", ex);}
		return null;
	}

	public final void remove(final SNoise noise)
	{
		try{
			// enleve le bruit de notre liste
			lstNoises.remove(noise);
			// on se detache
			noise.removeModificationListener(this);
			// on nottifie du changement
			simul.fireImageSpecsChanged(image.getImageNumber(), SSimul.STAGE_NOISES);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "remove()", ex);}
	}

	
	
	public final SNoise get(final int i)
	{
		try{
			return (SNoise)(lstNoises.get(i));
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "get()", ex);}
		return null;
	}
	
	
	
	
	

	

	public void onModificated(final Object object)
	{
		try{
			// un bruit vient d'etre modifie
			if( lstNoises.contains(object) == true ){
				// ce bruit est bien dans notre liste, on notifie la modification
				simul.fireImageSpecsChanged(image.getImageNumber(), SSimul.STAGE_NOISES);
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "onModification()", ex);}
	}
	
	

}
