/*
 SOS Project Classes.

 Copyright (C) 2006-2008 SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sos.framework.simulation.simul.specs.sources;

import sos.framework.maths.Bessel;
import sos.framework.maths.Constants;
import sos.framework.maths.Geometry;
import sos.framework.maths.Maths;
import sos.framework.maths.conversions.angles.FromArcsec;
import sos.framework.simulation.simul.SSimul;
import sos.framework.simulation.simul.specs.SSpecImage;
import sos.lib.logger.Logger;


public class AiryPoint extends SSource
{
	
	
	

	public AiryPoint(final double xCoord, final double yCoord, final double dblTotalFlux)
	{
		super(xCoord, yCoord, dblTotalFlux);
	}




	public double calculateIntensity(final double xArcsec, final double yArcsec, final SSpecImage image)
	{
		try{

			double A = 0.0;
			double B = 0.0;
			double e = 0.0;

			final double lambda = image.getFilter().getLambdaCenter();
			final double Epsi = 0.01; 								// ouverture (0=claire, 0.33=Hubble)

	
			double rArcsec = Geometry.euclidianDistance(0, 0, xArcsec, yArcsec);			// rayon du point considere
			double rRad;
			
			if(rArcsec == 0.0) rRad = 1.0E-9;
			rRad = FromArcsec.toRad(rArcsec);
			

			/* v (en radians) */
			double v = Constants.PI * image.getVariableValue(SSimul.VAR_TELESCOPE_DIAMETER_NAME) * rRad / lambda;
			//double v = 2.0 * Math.PI * 3.5 * r / lambda;

			/* epsilon au carre */
			double EpsiSquare = Epsi * Epsi;

			/* v multiplie par epsilon */
			double vTimesEpsi = v * Epsi;

			A = Maths.pow(1.0 - EpsiSquare, 2.0);
			B = (2 * Bessel.j1(v)) / v - (EpsiSquare * 2.0 * Bessel.j1(vTimesEpsi)) / vTimesEpsi;
			e = B * B / A;

			if(e < 0) e = 0;

			return e;

		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "calculateIntensity()", ex);}
		return 0.0;
	}

}
