/*
 SOS Project Classes.

 Copyright (C) SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package sos.framework.ui;

import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JPanel;

import sos.framework.ressources.Lang;
import sos.lib.logger.Logger;
import sos.lib.uicommon.UI;

public class SOSDialog extends JDialog implements WindowListener, ActionListener
{

	
	private static final long serialVersionUID		= 1L;

	// les dispositions des boutons possibles
	public static final int BUTTONS_OK_CANCEL		= 1;
	public static final int BUTTONS_CLOSE			= 2;
	
	// le bouton qui a ete utilise
	public static final int BUTTON_NONE				= 0;
	public static final int BUTTON_OK				= 1;
	public static final int BUTTON_CANCEL			= 2;
	public static final int BUTTON_CLOSE			= 3;
	
	
	
	// les boutons
	private JButton btnOK							= null;
	private JButton btnCANCEL						= null;
	private JButton btnCLOSE						= null;
	
	// le panneau des boutons
	private JPanel pnlButtons						= null;

	// la valeur du bouton de fermeture
	private int iReturnValue						= 0;
	
	// la disposition actuelle des boutons
	private int iActualButtons						= 0;
	
	// si la fenetre est bloquee
	private boolean isEnabled						= true;

	
	
	
	

		
	public SOSDialog(final Frame parent, final String strName, final boolean blModal, final boolean blResize, final int iButtons)
	{
		super(parent, strName, blModal);
		this.setLocation(parent.getLocationOnScreen().x + 20, parent.getLocationOnScreen().y + 20);
		this.buildContent(blResize, iButtons);
	}
	
	public SOSDialog(final Dialog parent, final String strName, final boolean blModal, final boolean blResize, final int iButtons)
	{
		super(parent, strName, blModal);
		this.setLocation(parent.getLocationOnScreen().x + 20, parent.getLocationOnScreen().y + 20);
		this.buildContent(blResize, iButtons);
	}


	
	
	private final void buildContent(final boolean blResize, final int iButtons)
	{
		try{
			this.getContentPane().setLayout(new BorderLayout());
			this.setResizable(blResize);
			this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
			this.addWindowListener(this);
			// construit le contenu
			this.iActualButtons = iButtons;
			
			GridBagConstraints c = new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 3, 3), 0, 0);
			pnlButtons = UI.createPanel(new GridBagLayout());
			// ajoute une zone vide sur la gauche
			pnlButtons.add(UI.createPanel(null), c);
			c.weightx = 0.1;
			c.gridx++;
			// ajoute les boutons
			if(iActualButtons == BUTTONS_OK_CANCEL){
				pnlButtons.add(btnCANCEL = UI.createButton("   " + Lang.STR_BTN_CANCEL + "   ", this, null), c);
				c.gridx++;
				pnlButtons.add(btnOK = UI.createButton("   " + Lang.STR_BTN_VALIDATE + "   ", this, null), c);
			}else{
				pnlButtons.add(btnCLOSE = UI.createButton("   " + Lang.STR_BTN_CLOSE + "   ", this, null), c);
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "buildContent()", ex);}
	}
	
	

	
	
	

	public final void setContent(final JComponent pnlContent)
	{
		try{
			this.getContentPane().removeAll();
			this.getContentPane().add(pnlContent, BorderLayout.CENTER);
			this.getContentPane().add(pnlButtons, BorderLayout.SOUTH);
			this.pack();
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "setContent()", ex);}
	}
	
	public final void removeContent()
	{
		try{
			this.getContentPane().removeAll();
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "removeContent()", ex);}
	}

	
	
	
	
	
	
	public final void setEnable(boolean newVal)
	{
		try{
			this.isEnabled = newVal;
			if(iActualButtons == BUTTONS_CLOSE){
				btnCLOSE.setEnabled(isEnabled);
			}else{
				btnOK.setEnabled(isEnabled);
				btnCANCEL.setEnabled(isEnabled);
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "setEnable()", ex);}
	}

	
	
	

	public final boolean isEnable() {return isEnabled;}
	
	public final int retournedValue() {return iReturnValue;}


	
	
	

	
	
	private final void closeDialog(final int iReturnValue)
	{
		try{
			this.iReturnValue = iReturnValue;
			this.setVisible(false);
			this.iReturnValue = BUTTON_NONE;
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "closeDialog()", ex);}
	}

	public void actionPerformed(ActionEvent ae)
	{
		try{
			if(ae.getSource() == btnOK) {closeDialog(BUTTON_OK); return;}
			if(ae.getSource() == btnCANCEL) {closeDialog(BUTTON_CANCEL); return;}
			if(ae.getSource() == btnCLOSE) {closeDialog(BUTTON_CLOSE); return;}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "actionPerformed()", ex);}
	}

	public void windowClosing(WindowEvent e)
	{
		try{
			if(iActualButtons == BUTTONS_OK_CANCEL) closeDialog(BUTTON_CANCEL);
			else closeDialog(BUTTON_CLOSE);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "windowClosing()", ex);}
	}


	public void windowOpened(WindowEvent e) {}
	public void windowClosed(WindowEvent e) {}
	public void windowIconified(WindowEvent e) {}
	public void windowDeiconified(WindowEvent e) {}
	public void windowActivated(WindowEvent e) {}
	public void windowDeactivated(WindowEvent e) {}
	

}
