/*
 SOS Project Classes.

 Copyright (C) 2006-2008 SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sos.framework.ui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;

import sos.framework.io.SOSInputOutput;
import sos.framework.ressources.Lang;
import sos.framework.simulation.SEnv;
import sos.framework.ui.dialogs.AboutDialog;
import sos.framework.ui.dialogs.ExportDialog;
import sos.framework.ui.dialogs.InformationDialog;
import sos.framework.ui.editor.SOSEditor;
import sos.framework.ui.mutables.ColoratedImagesMutable;
import sos.framework.ui.mutables.DistributionMutable;
import sos.framework.ui.mutables.GenerationMutable;
import sos.framework.ui.mutables.NoisesMutable;
import sos.framework.ui.mutables.NormalizationMutable;
import sos.framework.ui.mutables.PixelInfosMutable;
import sos.framework.ui.mutables.PropertiesMutable;
import sos.lib._SLogsListener;
import sos.lib.logger.Logger;

public class SOSFrame extends JFrame implements WindowListener, _SLogsListener
{

	private static final long serialVersionUID 			= 1L;
	
	private SEnv env									= null;
	
	private SOSMenu menu								= null;
	private SOSEditor editor							= null;
	private JPanel pnlMainContent						= null;
	private JTextArea txtLogs							= null;

	// la fenetre de generation
	private PropertiesMutable frameProp					= null;
	private DistributionMutable frameObj				= null;
	private GenerationMutable frameGen					= null;
	private NoisesMutable frameNoises					= null;
	private NormalizationMutable frameNorm				= null;
	private InformationDialog frameInfos				= null;
	private ColoratedImagesMutable frameColor			= null;
	private PixelInfosMutable framePixel				= null;
	private ExportDialog frameExport					= null;

	
	
	

	
	
	public SOSFrame()
	{
		super(Lang.STR_SIMUL_FRAME_TITLE);
		buildContain();
	}
	
	private final void buildContain()
	{
		try{
			// zone de texte
			txtLogs = new JTextArea("");
			txtLogs.setEditable(false);
			Logger.addLogsListener(this);
			JScrollPane scrollLogs = new JScrollPane(txtLogs);
			scrollLogs.setBorder(null);
			scrollLogs.setMinimumSize(new Dimension(80, 80));
			scrollLogs.setPreferredSize(scrollLogs.getMinimumSize());
			
			// menu
			menu = new SOSMenu(this);
			this.setJMenuBar(menu);
			
			// contenu
			pnlMainContent = new JPanel(new BorderLayout());
			JSplitPane split = new JSplitPane(JSplitPane.VERTICAL_SPLIT, pnlMainContent, scrollLogs);
			split.setResizeWeight(1);
			split.setOneTouchExpandable(true);
	
			// ajoute le tout
			this.getContentPane().setLayout(new BorderLayout());
			this.getContentPane().add(split, BorderLayout.CENTER);
			this.addWindowListener(this);
			this.pack();
			
			boolean setMax = false;
			int maxWidth = Toolkit.getDefaultToolkit().getScreenSize().width;
			if(maxWidth > 1200) maxWidth = 1200;
			else setMax = true;
			
			int maxHeight = Toolkit.getDefaultToolkit().getScreenSize().width;
			if(maxHeight > 750) maxHeight = 750;
			else setMax = true;
				
			this.setBounds(Toolkit.getDefaultToolkit().getScreenSize().width/2-maxWidth/2, Toolkit.getDefaultToolkit().getScreenSize().height/2-maxHeight/2, maxWidth, maxHeight);
			if(setMax) this.setExtendedState(JFrame.MAXIMIZED_BOTH);
		
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "buildContain()", ex);}
	}


	

	
	public final void buildEditor(final SEnv env)
	{
		try{
			this.env = env;
			new Thread()
			{
				public void run(){
					// nouvel editeur
					editor = new SOSEditor(env);
					// on charge les fenetres
					frameProp = new PropertiesMutable(env);
					frameObj = new DistributionMutable(env);
					frameGen = new GenerationMutable(env);
					frameNorm = new NormalizationMutable(env);
					frameNoises = new NoisesMutable(env);
					frameInfos = new InformationDialog(env);
					frameColor = new ColoratedImagesMutable(env);
					framePixel = new PixelInfosMutable(env);
					frameExport = new ExportDialog(env);
					// on est pret a lancer une simulation
					menu.setMenuState(false);
				}
			}.start();
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "buildContain()", ex);}
	}


	
	
	
	
	
	
	public final void duplicateImage() {env.getSimul().duplicateActualImage();}
	
	public final void deleteImage() {env.getSimul().deleteActualImage(this);}
	
	
	
	
	
	
	
	
	

	
	public final void startSimulation(final String strFileName)
	{
		try{
			// ferme la simulation actuelle au cas ou
			this.endSimulation();
			
			// nouvelle simulation
			if( env.getSimul().startSimulation(this, strFileName) == false){
				return;
			}
			
			// init les prefs
			env.getPrefs().initPrefs();
			
			// affiche les elements graphiques
			pnlMainContent.add(editor, BorderLayout.CENTER);
			togglePropertiesDialog(true);
			toggleGenerationDialog(true);
			toggleNoisesDialog(true);
			toggleColoratedImageDialog(true);
			togglePixelInfosDialog(true);
			menu.setMenuState(true);
			pnlMainContent.updateUI();
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "startSimulation()", ex);}
	}

	public final void endSimulation()
	{
		try{
			// termine la simulation
			this.env.getSimul().endSimulation();
			
			// cache les elements graphiques
			this.menu.setMenuState(false);
			this.windowsHideAll();
			this.pnlMainContent.removeAll();
			this.pnlMainContent.updateUI();

			// nettoie la memoire
			System.gc();
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "endSimulation()", ex);}
	}
	
	public final void closeWindow()
	{
		try{
			// termine la simulation
			this.endSimulation();
			
			// cache la fenetre
			this.setVisible(false);
			this.dispose();
			
			// quitte
			System.exit(0);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "closeWindow()", ex);}
	}


	
	
	


	

	
	public final void windowsShowAll()
	{
		try{
			if(frameProp != null) frameProp.toggleDialog(true);
			if(frameGen != null) frameGen.toggleDialog(true);
			if(frameNoises != null) frameNoises.toggleDialog(true);
			if(frameColor != null) frameColor.toggleDialog(true);
			if(framePixel != null) framePixel.toggleDialog(true);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "windowsShowAll()", ex);}
	}
	
	public final void windowsHideAll()
	{
		try{
			if( (frameProp != null) && (frameProp.isVisible() == true) ) frameProp.toggleDialog();
			if( (frameNorm != null) && (frameNorm.isVisible() == true) ) frameNorm.toggleDialog();
			if( (frameObj != null) && (frameObj.isVisible() == true) ) frameObj.toggleDialog();
			if( (frameGen != null) && (frameGen.isVisible() == true) ) frameGen.toggleDialog();
			if( (frameNoises != null) && (frameNoises.isVisible() == true) ) frameNoises.toggleDialog();
			if( (framePixel != null) && (framePixel.isVisible() == true) ) framePixel.toggleDialog();
			if( (frameColor != null) && (frameColor.isVisible() == true) ) frameColor.toggleDialog();
			if( (frameInfos != null) && (frameInfos.isVisible() == true) ) frameInfos.setVisible(false);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "windowsHideAll()", ex);}
	}
	
	


	
	private final void toggleDialog(final SOSMutableDialog dialog, final boolean blForceVisible)
	{
		if(dialog != null){
			dialog.setLocationOnScreen(this.getEditor().getImageCanvas().getLocationOnScreen().x+20, this.getEditor().getImageCanvas().getLocationOnScreen().y+20);	
			if(blForceVisible) dialog.toggleDialog(blForceVisible);
			else dialog.toggleDialog();
		}
	}
	
	public final void togglePropertiesDialog(boolean blForceVisible) {toggleDialog(frameProp, blForceVisible);}
	public final void toggleDistributionDialog(final boolean blForceVisible) {toggleDialog(frameObj, blForceVisible);}
	public final void toggleNoisesDialog(final boolean blForceVisible) {toggleDialog(frameNoises, blForceVisible);}
	public final void toggleGenerationDialog(final boolean blForceVisible) {toggleDialog(frameGen, blForceVisible);}
	public final void toggleNormalizationDialog(final boolean blForceVisible) {toggleDialog(frameNorm, blForceVisible);}
	public final void togglePixelInfosDialog(final boolean blForceVisible) {toggleDialog(framePixel, blForceVisible);}
	public final void toggleColoratedImageDialog(final boolean blForceVisible) {toggleDialog(frameColor, blForceVisible);}
	
	public final void showAboutDialog() {new AboutDialog(env).setVisible(true);}
	public final void showExportSOSDialog() {if(frameExport != null) frameExport.setVisible(SOSInputOutput.EXPORT_FORMAT_SOS);}
	public final void showExportFITSDialog() {if(frameExport != null) frameExport.setVisible(SOSInputOutput.EXPORT_FORMAT_FITS);}
	public final void showExportTXTDialog() {if(frameExport != null) frameExport.setVisible(SOSInputOutput.EXPORT_FORMAT_TXT);}
	public final void showExportIMGDialog() {if(frameExport != null) frameExport.setVisible(SOSInputOutput.EXPORT_FORMAT_IMG);}

	public final void toggleInformationsDialog(final boolean blForceVisible)
	{
		if(frameInfos.isVisible() == false || blForceVisible) frameInfos.setVisible(true);
		else frameInfos.setVisible(false);
	}
	

	
	
	
	
	
	
	

	
	public final SOSEditor getEditor() {return editor;}
	
	public final void fireLog(final String strMessage) {fireLog(strMessage, true);}

	public final void fireLog(final String strMessage, final boolean blRetour) {}
	
	
	
	
	

	
	
	
	
	
	

	public void windowOpened(WindowEvent e) {}
	public void windowClosing(WindowEvent e) {closeWindow();}
	public void windowClosed(WindowEvent e) {}
	public void windowIconified(WindowEvent e) {}
	public void windowDeiconified(WindowEvent e) {}
	public void windowActivated(WindowEvent e) {}
	public void windowDeactivated(WindowEvent e) {}



	public void onLog(String strLog)
	{
		try{
			txtLogs.setText(txtLogs.getText() + strLog);
			
			SwingUtilities.invokeLater(
				new Runnable(){
					public void run(){
						try{
							txtLogs.updateUI();
							txtLogs.scrollRectToVisible(new Rectangle(1, txtLogs.getHeight()+100, 1, 1));
						}catch(Exception ex) {}
					}
				}
			);
			
		}catch(Exception ex) {}
	}

	
}
