/*
 SOS Project Classes.

 Copyright (C) SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package sos.framework.ui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;

import sos.framework.ressources.Icons;
import sos.lib.logger.Logger;
import sos.lib.uicommon.UI;

public class SOSMenu extends JMenuBar implements ActionListener
{

	private static final long	serialVersionUID			= 1L;

	
	private SOSFrame window										= null;
	
	
	// simulation
	private JMenu				mnuSimulation				= null;
	private JMenuItem			mnuSimulationExemplesEx1	= null;
	private JMenuItem			mnuSimulationExemplesEx2	= null;
	private JMenuItem			mnuSimulationOpen			= null;
	private JMenuItem			mnuSimulationClose			= null;
	private JMenuItem			mnuSimulationSave			= null;
	private JMenuItem			mnuSimulationExportAsFITS	= null;
	private JMenuItem			mnuSimulationExportAsTXT	= null;
	private JMenuItem			mnuSimulationExportAsIMG	= null;
	private JMenuItem			mnuSimulationQuit			= null;
	
	// images
	private JMenu				mnuImages					= null;
	private JMenuItem			mnuImagesCloneImage			= null;
	private JMenuItem			mnuImagesDeleteImage		= null;
	
	// etapes
	private JMenu				mnuStages					= null;
	private JMenuItem			mnuStagesProperties			= null;
	private JMenuItem			mnuStagesNormalization		= null;
	private JMenuItem			mnuStagesDistribution		= null;
	private JMenuItem			mnuStagesGeneration			= null;
	private JMenuItem			mnuStagesNoisification		= null;
	private JMenuItem			mnuStagesInformation		= null;
	private JMenuItem			mnuStagesColorImage			= null;
	
	// fenetre
	private JMenu				mnuWindows					= null;
	private JMenuItem			mnuWindowsShowAll			= null;
	private JMenuItem			mnuWindowsHideAll			= null;
	
	// aide
	private JMenu				mnuAbout					= null;
	private JMenuItem			mnuAboutInformation			= null;




	
	
	
	

	public SOSMenu(final SOSFrame window)
	{
		try{
			this.window = window;
			
			// menu simulation
			mnuSimulation = new JMenu("Simulation");
			mnuSimulationExemplesEx1 = UI.createMenuItem("Image simple", this, Icons.MENU_EXEMPLE);
			mnuSimulation.add(mnuSimulationExemplesEx1);
			mnuSimulationExemplesEx2 = UI.createMenuItem("Image colorée", this, Icons.MENU_EXEMPLE);
			mnuSimulation.add(mnuSimulationExemplesEx2);
			mnuSimulation.addSeparator();
			mnuSimulationOpen = UI.createMenuItem("Ouvrir", this, Icons.MENU_OPEN);
			mnuSimulation.add(mnuSimulationOpen);
			mnuSimulationClose = UI.createMenuItem("Fermer", this, Icons.MENU_CLOSE);
			mnuSimulation.add(mnuSimulationClose);
			mnuSimulation.addSeparator();
			mnuSimulationSave = UI.createMenuItem("Enregistrer sous...", this, Icons.MENU_SAVE);
			mnuSimulation.add(mnuSimulationSave);
			mnuSimulation.addSeparator();
			mnuSimulationExportAsFITS = UI.createMenuItem("Exporter au format FITS", this, Icons.MENU_EXPORT);
			mnuSimulation.add(mnuSimulationExportAsFITS);
			mnuSimulationExportAsTXT = UI.createMenuItem("Exporter au format TXT", this, Icons.MENU_EXPORT);
			mnuSimulation.add(mnuSimulationExportAsTXT);			
			mnuSimulationExportAsIMG = UI.createMenuItem("Exporter au format Image", this, Icons.MENU_EXPORT);
			mnuSimulation.add(mnuSimulationExportAsIMG);
			mnuSimulation.addSeparator();
			mnuSimulationQuit = UI.createMenuItem("Quitter", this, Icons.MENU_EXIT);
			mnuSimulation.add(mnuSimulationQuit);
			this.add(mnuSimulation);
			
			// menu images
			mnuImages = new JMenu("Images");
			mnuImagesCloneImage = UI.createMenuItem("Dupliquer l'image", this, Icons.MENU_DUPLICATE_IMAGE);
			mnuImages.add(mnuImagesCloneImage);
			mnuImagesDeleteImage = UI.createMenuItem("Supprimer l'image", this, Icons.MENU_REMOVE_IMAGE_IMAGE);
			mnuImages.add(mnuImagesDeleteImage);
			this.add(mnuImages);
		
			// menu etapes
			mnuStages = new JMenu("Etapes");
			mnuStagesProperties = UI.createMenuItem("Propriétés...", this, Icons.MENU_PROPERTIES);
			mnuStages.add(mnuStagesProperties);
			mnuStagesDistribution = UI.createMenuItem("Objets...", this, Icons.MENU_DISTRIBUTION);
			mnuStages.add(mnuStagesDistribution);
			mnuStagesGeneration = UI.createMenuItem("Générer...", this, Icons.MENU_GENERATION);
			mnuStages.add(mnuStagesGeneration);
			mnuStagesNoisification = UI.createMenuItem("Bruits...", this, Icons.MENU_NOISES);
			mnuStages.add(mnuStagesNoisification);
			mnuStagesNormalization = UI.createMenuItem("Normalisation...", this, Icons.MENU_NORMALIZE);
			mnuStages.add(mnuStagesNormalization);
			mnuStages.addSeparator();
			//mnuStagesPixelInfos = UI.createMenuItem("Pixel infos...", this, Icons.MENU_INTENS_STATS);
			//mnuStages.add(mnuStagesPixelInfos);
			mnuStagesInformation = UI.createMenuItem("Informations...", this, Icons.MENU_INFOS);
			mnuStages.add(mnuStagesInformation);
			mnuStages.addSeparator();
			mnuStagesColorImage = UI.createMenuItem("Coloriser...", this, Icons.MENU_COLORIZE);
			mnuStages.add(mnuStagesColorImage);
			//mnuStagesSpectre = UI.createMenuItem("Spectre...", this, null);
			//mnuStages.add(mnuStagesSpectre);
			this.add(mnuStages);
			
			// menu fenetres
			mnuWindows = new JMenu("Fenêtres");
			mnuWindowsShowAll = UI.createMenuItem("Toutes visibles", this, Icons.MENU_WINDOWS_SHOW);
			mnuWindows.add(mnuWindowsShowAll);
			mnuWindowsHideAll = UI.createMenuItem("Toutes cachées", this, Icons.MENU_WINDOWS_HIDE);
			mnuWindows.add(mnuWindowsHideAll);
			this.add(mnuWindows);
			
			// menu aide
			mnuAbout = new JMenu(" ? ");
			mnuAboutInformation = UI.createMenuItem("A propos...", this, Icons.MENU_ABOUT);
			mnuAbout.add(mnuAboutInformation);
			this.add(mnuAbout);
			
			
			setMenuState(false);
			mnuSimulation.setEnabled(false);

			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "SOSMenu()", ex);}
	}

	
	
	


	
	
	public final void setMenuState(final boolean active)
	{
		try{
			mnuSimulation.setEnabled(true);
			mnuSimulationExemplesEx1.setEnabled(!active);
			mnuSimulationExemplesEx2.setEnabled(!active);
			mnuSimulationOpen.setEnabled(!active);
			mnuSimulationSave.setEnabled(active);
			mnuSimulationExportAsFITS.setEnabled(active);
			mnuSimulationExportAsTXT.setEnabled(active);
			mnuSimulationExportAsIMG.setEnabled(active);
			mnuSimulationClose.setEnabled(active);
			mnuImages.setEnabled(active);
			mnuStages.setEnabled(active);
			mnuWindows.setEnabled(active);
		}catch(Exception ex){Logger.fireError(this.getClass().getName(), "setMenuState()", ex);}
	}



	
	
	
	
	public final void actionPerformed(ActionEvent ae)
	{
		try{
			
			// menu simulation
			if(ae.getSource() == mnuSimulationExemplesEx1) {window.startSimulation("Simple_image.sos"); return;}
			if(ae.getSource() == mnuSimulationExemplesEx2) {window.startSimulation("Colorated_image.sos"); return;}
			if(ae.getSource() == mnuSimulationOpen) {window.startSimulation(null); return;}
			if(ae.getSource() == mnuSimulationSave) {window.showExportSOSDialog(); return;}
			if(ae.getSource() == mnuSimulationExportAsFITS) {window.showExportFITSDialog(); return;}
			if(ae.getSource() == mnuSimulationExportAsTXT) {window.showExportTXTDialog(); return;}
			if(ae.getSource() == mnuSimulationExportAsIMG) {window.showExportIMGDialog(); return;}
			if(ae.getSource() == mnuSimulationClose) {window.endSimulation(); return;}
			if(ae.getSource() == mnuSimulationQuit) {window.closeWindow(); return;}
			
			// menu images
			if(ae.getSource() == mnuImagesCloneImage) {window.duplicateImage(); return;}
			if(ae.getSource() == mnuImagesDeleteImage) {window.deleteImage(); return;}
			
			// menu etapes
			if(ae.getSource() == mnuStagesProperties) {window.togglePropertiesDialog(true); return;}
			if(ae.getSource() == mnuStagesDistribution) {window.toggleDistributionDialog(true); return;}
			if(ae.getSource() == mnuStagesGeneration) {window.toggleGenerationDialog(true); return;}
			if(ae.getSource() == mnuStagesNoisification) {window.toggleNoisesDialog(true); return;}
			if(ae.getSource() == mnuStagesNormalization) {window.toggleNormalizationDialog(true); return;}
			//if(ae.getSource() == mnuStagesPixelInfos) {env.getSimFrame().togglePixelInfosDialog(true); return;}
			if(ae.getSource() == mnuStagesInformation) {window.toggleInformationsDialog(true); return;}
			if(ae.getSource() == mnuStagesColorImage) {window.toggleColoratedImageDialog(true); return;}

			// menu fenetres
			if(ae.getSource() == mnuWindowsShowAll) {window.windowsShowAll(); return;}
			if(ae.getSource() == mnuWindowsHideAll) {window.windowsHideAll(); return;}

			// menu aide
			if(ae.getSource() == mnuAboutInformation) {window.showAboutDialog(); return;}		
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "actionPerformed()", ex);}
	}

}
