/*
 SOS Project Classes.

 Copyright (C) SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package sos.framework.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.LayoutManager;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.LineBorder;

import sos.framework.ressources.Icons;
import sos.framework.simulation.SEnv;
import sos.framework.simulation.simul._SSpecsListener;
import sos.framework.simulation.simul.specs.SSpecImage;
import sos.framework.ui.editor.controlbar.ControlBar;
import sos.lib.logger.Logger;
import sos.lib.uicommon.UI;



public abstract class SOSMutableDialog implements ActionListener, _SSpecsListener
{

	// mode possible de la dialog
	public static final int SOSDIALOG_MODE_WINDOW			= 1;
	public static final int SOSDIALOG_MODE_LEFT_LATERAL 	= 2;
	public static final int SOSDIALOG_MODE_RIGHT_LATERAL	= 3;
	


	// toutes les infos sur la simulation actuelle
	protected SEnv env										= null;
	
	// la boite de dialogue liee
	protected SOSDialog theDialog							= null;
	
	// le contenu de la fenetre
	private JPanel theContent								= null;
	
	// contenu et titre
	private SOSPanel theContentAndTitle						= null;

	
	
	protected SSpecImage imageSpec							= null;

	
	
	
	

	
	/* les boutons */
	private JPanel pnlButtons								= null;
	private JButton btnToLeft								= null;
	private JButton btnToRight								= null;
	private JButton btnToWin								= null;
	private JButton btnClose								= null;
	
	
	/* le type d'affichage de la boite */
	private int iActualMode									= 0;
	
	/* memorise si la fenetre est visible */
	private boolean isVisible								= false;
	
	/* le nom de la fenetre */
	private String strTitle									= "?";
	private JLabel lblTitle									= null;
	
	
	
	
	
	
	
	
	
	protected SOSMutableDialog(final SEnv env, final String strTitle, final boolean blResize, final int iButtons, final int iMode)
	{
		try{
			this.env = env;
			this.strTitle = strTitle;
			this.iActualMode = iMode;
			this.env.getSimul().addSimulationListener(this);
			
			// construit le dialog lie
			theDialog = new SOSDialog(env.getWindow(), strTitle, false, blResize, iButtons);

		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "SOSDialog()", ex);}
	}

	protected final void setContent(final JPanel content)
	{
		try{
			if(content == null) theContent = new JPanel();
			else theContent = content;
			
			theContent.setBorder(new LineBorder(Color.darkGray, 1));
			// les boutons
			btnToLeft = UI.createButton("Ancrer à gauche", this, UI.SIZE_SMALL_ICON, Icons.ICON_BUTTON_LEFT); btnToLeft.setBounds(0, 0, UI.SIZE_SMALL_ICON.width, UI.SIZE_SMALL_ICON.height);
			btnToRight = UI.createButton("Ancrer à droite", this, UI.SIZE_SMALL_ICON, Icons.ICON_BUTTON_RIGHT); btnToRight.setBounds(UI.SIZE_SMALL_ICON.width, 0, UI.SIZE_SMALL_ICON.width, UI.SIZE_SMALL_ICON.height);
			btnClose = UI.createButton("Fermer", this, UI.SIZE_SMALL_ICON, Icons.ICON_BUTTON_CLOSE); btnClose.setBounds(UI.SIZE_SMALL_ICON.width, 0, UI.SIZE_SMALL_ICON.width, UI.SIZE_SMALL_ICON.height);
			btnToWin = UI.createButton("Libérer", this, UI.SIZE_SMALL_ICON, Icons.ICON_BUTTON_UP); btnToWin.setBounds(0, 0, UI.SIZE_SMALL_ICON.width, UI.SIZE_SMALL_ICON.height);
			pnlButtons = UI.createPanel(null, new Dimension(2*UI.SIZE_SMALL_ICON.width, UI.SIZE_SMALL_ICON.height));
			// titre et boutons
			
			
			JPanel pnlTitle = new DialogTitle();
			pnlTitle.add(lblTitle = UI.createLabel(" " + strTitle, UI.FONT_BOLD_BIG), BorderLayout.CENTER);
			pnlTitle.add(pnlButtons, BorderLayout.EAST);
			// ajoute le tout
			

			theContentAndTitle = new SOSPanel(new BorderLayout());
			theContentAndTitle.setOpaque(true);
			theContentAndTitle.add(pnlTitle, BorderLayout.NORTH);
			theContentAndTitle.add(theContent, BorderLayout.CENTER);
			// change le mode
			int askedMode = iActualMode;
			iActualMode = 0;
			changeDialogMode(askedMode);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "onContentBuilded()", ex);}
	}


	
	
	
	
	
	
	public final void setTitle(final String strNewTitle)
	{
		try{
			this.strTitle = strNewTitle;
			if(lblTitle != null) lblTitle.setText(" " + strNewTitle);
			if(theDialog != null) theDialog.setTitle(strNewTitle);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "setTitle()", ex);}
	}
	
	
	
	
	
	
	
	public final Point getLocationOnScreen()
	{
		try{
			if(iActualMode == SOSDIALOG_MODE_WINDOW) return theDialog.getLocationOnScreen();
			else return theContentAndTitle.getLocationOnScreen();
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "getLocationOnScreen()", ex);}
		return new Point(0, 0);
	}
	
	public final void setLocationOnScreen(final int x, final int y)
	{
		try{
			if(iActualMode == SOSDIALOG_MODE_WINDOW) theDialog.setLocation(x, y);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "setLocationOnScreen()", ex);}
	}
	
	
	
	
	
	
	public final int getActualWindowMode() {return iActualMode;}
	
	public final boolean isVisible()
	{
		if(iActualMode == SOSDIALOG_MODE_WINDOW) return theDialog.isVisible();
		else return isVisible;
	} 

	public final Frame getPanelModeFrame()
	{
		try{
			if(iActualMode != SOSDIALOG_MODE_WINDOW) return (Frame)(env.getWindow());
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "getFrame()", ex);}
		return null;
	}
	
	public final Dialog getWindowModeDialog()
	{
		try{
			if(iActualMode == SOSDIALOG_MODE_WINDOW) return (Dialog)theDialog;
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "getLocationOnScreen()", ex);}
		return null;
	}
	
	
	

	
	
	public final void toggleDialog()
	{
		toggleDialog(false);
	}
	
	public final void toggleDialog(final boolean blForceVisible)
	{
		try{
			if( (this.isVisible() == false) || (blForceVisible == true) ){
				// montre la fenetre
				if(this.isVisible()) return;
				switch(iActualMode){
					case SOSDIALOG_MODE_WINDOW:
						theDialog.setVisible(true);
						break;
					case SOSDIALOG_MODE_LEFT_LATERAL:
						env.getWindow().getEditor().getLeftBar().addDialog(theContentAndTitle);
						break;
					case SOSDIALOG_MODE_RIGHT_LATERAL:
						env.getWindow().getEditor().getRightBar().addDialog(theContentAndTitle);
						break;
				}
				isVisible = true;
				onDialogBecomeVisible();
			}else{
				// cache la fenetre
				if(!this.isVisible()) return;
				switch(iActualMode){
					case SOSDIALOG_MODE_WINDOW:
						theDialog.setVisible(false);
						break;
					case SOSDIALOG_MODE_LEFT_LATERAL:
						env.getWindow().getEditor().getLeftBar().removeDialog(theContentAndTitle);
						break;
					case SOSDIALOG_MODE_RIGHT_LATERAL:
						env.getWindow().getEditor().getRightBar().removeDialog(theContentAndTitle);
						break;
				}
				isVisible = false;
				onDialogBecomeInvisible();
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "showDialog()", ex);}
		
	}

	public final void changeDialogMode(final int newMode)
	{
		try{
			if(newMode != iActualMode){
				// cache la fenetre si besoin
				boolean wasVisible = this.isVisible();
				if(this.isVisible()) toggleDialog();
				
				// change le mode
				if(iActualMode == SOSDIALOG_MODE_WINDOW) theDialog.removeContent();
				iActualMode = newMode;
				pnlButtons.removeAll();
				if(iActualMode == SOSDIALOG_MODE_WINDOW){
					pnlButtons.add(btnToRight);
					pnlButtons.add(btnToLeft);
					theDialog.setContent(theContentAndTitle);
				}else{
					pnlButtons.add(btnToWin);
					pnlButtons.add(btnClose);
				}
				// reaffiche la fenetre si besoin
				if(wasVisible) toggleDialog();
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "changeDialogMode()", ex);}
	}
	
	
	
	
	
	
	
	private final void imageChanged()
	{
		try{
			if(this.isVisible() == false) return;
			
			if( env.getSimul().getActualImageNumber() < 0 ){
				// plus de simulation
				imageSpec = null;
			}else{
				// nouvelle simulation
				if( imageSpec == null ) fillSimulation();
				imageSpec = env.getSimul().getSpecs().getActualImage();
				fillImage();
			}
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "imageChanged()", ex);}
	}


	

	

	protected void fillImage() {}
	
	protected void fillSimulation() {}
	
	
	
	protected void onDialogBecomeInvisible() {}
	
	protected void onDialogBecomeVisible() {imageChanged();}
	
	
	
	
	public void onSimulationImageChanged() {imageChanged();}
	
	public void onSimulationStateChanged(boolean blIsSimulating) {}
	
	public void onSimulationStageChanged(int iSimulationState) {}
	
	
	

	
	
	public void actionPerformed(ActionEvent ae)
	{
		try{
			if(ae.getSource() == btnToLeft) {changeDialogMode(SOSDIALOG_MODE_LEFT_LATERAL); return;}
			if(ae.getSource() == btnToRight) {changeDialogMode(SOSDIALOG_MODE_RIGHT_LATERAL); return;}
			if(ae.getSource() == btnToWin) {changeDialogMode(SOSDIALOG_MODE_WINDOW); return;}
			if(ae.getSource() == btnClose) {toggleDialog(); return;}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "actionPerformed()", ex);}
	}

	
	

	
	
	
	
	
	private class SOSPanel extends JPanel implements MouseListener, MouseWheelListener
	{
		private static final long serialVersionUID			= 1L;

		public SOSPanel(final LayoutManager l)
		{
			super(l);
			this.addMouseListener(this);
			this.addMouseWheelListener(this);
		}
	    public void mousePressed(MouseEvent e) {}
	    public void mouseReleased(MouseEvent e) {}
	    public void mouseEntered(MouseEvent e) {}
	    public void mouseExited(MouseEvent e) {}
		public void mouseClicked(MouseEvent e) {}
		
	    public void mouseWheelMoved(MouseWheelEvent e) {}
	}
	
	
	
	
    public class DialogTitle extends JPanel
    {

		private static final long serialVersionUID			= 1L;
		private GradientPaint gradient 						= null; 
		
		public DialogTitle()
		{
			super(new BorderLayout());
			gradient = new GradientPaint(0, 0, theContent.getBackground(), 0, 15, ControlBar.COLOR_BAR);
			this.setOpaque(false);
		}
		
		public void paint(Graphics g)
		{
			Graphics2D g2d = (Graphics2D) g;
			g2d.setPaint(gradient);
			g2d.fillRect(0, 0, this.getSize().width, this.getSize().height); 
			
			super.paint(g);
			
		}
    }
    
    
	
}
