/*
 SOS Project Classes.

 Copyright (C) SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package sos.framework.ui.dialogs;

import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JTextField;

import sos.framework.ressources.Icons;
import sos.framework.ressources.Lang;
import sos.framework.simulation.simul.specs.distrib.DistribFunction;
import sos.framework.ui.SOSDialog;
import sos.lib.logger.Logger;
import sos.lib.uicommon.SOSNumberedTextPane;
import sos.lib.uicommon.UI;
import sos.lib.uicommon.graphics.SOSGraphCanvas;

public class DistributionDialog extends SOSDialog
{

	private static final long serialVersionUID 			= 1L;

	private DistribFunction function					= null;

	private JTextField txtName							= null;

	
	private SOSNumberedTextPane txtFunction				= null;
	private JPanel pnlFunctionContent					= null;
	private JButton btnTest								= null;
	
	private boolean blShowRich							= false;
	
	
	private SOSGraphCanvas cvDistrib					= null;
	

	
	
	
	
	public DistributionDialog(final Frame parent, final DistribFunction function)
	{
		super(parent, Lang.STR_TITLE_DISTRIB_FUNCS_MANAGER, true, true, SOSDialog.BUTTONS_OK_CANCEL);
		this.builContent(function);
	}
	
	public DistributionDialog(final Dialog parent, final DistribFunction function)
	{
		super(parent, Lang.STR_TITLE_DISTRIB_FUNCS_MANAGER, true, true, SOSDialog.BUTTONS_OK_CANCEL);
		this.builContent(function);
	}

	private final void builContent(final DistribFunction function)
	{
		try{
			this.function = function;
		
			// construit le contenu
			GridBagConstraints c = new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 0, 3), 0, 0);
			JPanel pnlContent = UI.createPanel(new BorderLayout());
			
			// description
			JPanel pnlGridBag = UI.createPanel(new GridBagLayout());
			pnlGridBag.add(UI.createLabel("  " + "Description", UI.FONT_PLAIN_ITALIC), c);
			c.gridx++;
			pnlGridBag.add(txtName = new JTextField(function.getName()), c);
			c.gridx = 0;

			
			pnlContent.add(pnlGridBag, BorderLayout.NORTH);
			

			pnlGridBag = UI.createPanel(new GridBagLayout());
			c.gridy = 0;
			c.insets = new Insets(3, 3, 3, 3);
		
			// fonctions
			txtFunction = new SOSNumberedTextPane();
			txtFunction.setText(function.getPoints());
			txtFunction.getTextArea().setCaretPosition(0);
			
			pnlFunctionContent = new JPanel(new BorderLayout());
			JTextField txtTop = new JTextField("function distribute() {");
			JTextField txtBottom = new JTextField("}");
			

			btnTest = UI.createButton("Tester", this, UI.SIZE_SMALL_ICON, Icons.ICON_BUTTON_EDIT);
			
			pnlFunctionContent.add(txtFunction);
			
			pnlGridBag.add(UI.createInputFunctionPanel(txtTop, pnlFunctionContent, txtBottom, null, btnTest), c);
			//pnlGridBag.add(UI.createScrollPane(txtFunction, null), c);
			
			
			
			cvDistrib = new SOSGraphCanvas(1e-8, 1e4, 1, 0.2);
			cvDistrib.setYPolicy(1e-3, 1e10, 1, 0.2);
			cvDistrib.setXScale(SOSGraphCanvas.SCALE_LOG);
			cvDistrib.setXLabel("Flux {Jy}");
			cvDistrib.setYScale(SOSGraphCanvas.SCALE_LOG);
			cvDistrib.setYLabel("Flux {Nb/Sr}");
			
			
			
			pnlContent.add(pnlGridBag, BorderLayout.CENTER);
			
			setContent(pnlContent);
			
			setSize(650, 600);
			setResizable(true);

		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "NoiseEditor()", ex);}
	}

	
	

	
	
	

	
	
	private final void toggleFunctionView()
	{
		try{
			if(blShowRich == false){
				
				
				cvDistrib.clearFunctions();
				//cvDistrib.addFunction(function);
				
				
				pnlFunctionContent.remove(txtFunction);
				pnlFunctionContent.add(cvDistrib, BorderLayout.CENTER);
			}else{
				pnlFunctionContent.remove(cvDistrib);
				pnlFunctionContent.add(txtFunction, BorderLayout.CENTER);
			}
			pnlFunctionContent.updateUI();
			blShowRich = !blShowRich;
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "toggleFunctionView()", ex);}
	}
	


	public void setVisible(boolean blVisible)
	{
		try{
			
			// montre la fenetre
			super.setVisible(blVisible);
			
			if((blVisible == false) && (this.retournedValue() == SOSDialog.BUTTON_OK)){

				try{ function.setPoints(txtFunction.getText()); }catch(Exception ex) {}
				try{ function.setName(txtName.getText()); }catch(Exception ex) {}
				
			}
	
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "setVisible()", ex);}
	}

	public final void actionPerformed(ActionEvent ae)
	{
		super.actionPerformed(ae);
		try{
			if(ae.getSource() == btnTest) {toggleFunctionView(); return;}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "actionPerformed()", ex);}
	}
	

}
