/*
 SOS Project Classes.

 Copyright (C) 2006-2008 SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sos.framework.ui.dialogs;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.io.File;
import java.util.Iterator;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.imageio.ImageWriteParam;
import javax.imageio.ImageWriter;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.border.LineBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import sos.framework.io.SOSInputOutput;
import sos.framework.ressources.Icons;
import sos.framework.simulation.SEnv;
import sos.framework.ui.SOSDialog;
import sos.lib.logger.Logger;
import sos.lib.uicommon.UI;

public class ExportDialog extends SOSDialog implements ChangeListener
{

	private static final long serialVersionUID 			= 1L;
	

	private SEnv env									= null;

	private JCheckBox chkSpecs							= null;
	private JCheckBox chkSources						= null;
	private JCheckBox chkRaw							= null;
	private JCheckBox chkNormalised						= null;
	
	private JLabel lblDescription						= null;
	
	private JTextField txtDesc							= null;

	private JRadioButton optAllImages					= null;
	private JRadioButton optActualImage					= null;
	
	private JComboBox cmbFormat							= null;
	private JSlider sldQuality							= null;
	
	private Vector lstFormats							= new Vector();
	private Vector lstDescs								= new Vector();
	private Vector lstWriters							= new Vector();

	private int iLastSel								= 0;
	
	private JLabel lblQuality							= null;
	
	private ImageWriteParam iwpJpeg						= null;

	
	
	
	
	
	
	
	public ExportDialog(final SEnv env)
	{
		super(env.getWindow(), "Exporter l'image", true, false, BUTTONS_OK_CANCEL);
		try{
			this.env = env;
			
			// construit le contenu
			JPanel pnlContent = UI.createPanel(new GridBagLayout());
			pnlContent.setBorder(new LineBorder(Color.darkGray, 1));
			GridBagConstraints c = new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 0, 3), 0, 0);
			c.gridwidth = 4;
	
			// logo
			ImageIcon ic = Icons.LOGO_FORMAT_SOS;
			if(ic != null){
				JLabel lbl = UI.createLabel("", null);
				lbl.setIcon(ic);
				pnlContent.add(lbl, c);
				c.gridy++;
			}
			
			// recherche les format disponibles
			addFormat("SOS", "Simulation d'Observation Spatiale", null);
			addFormat("-", "-", null);
			addFormat("FITS", "Flexible Image Transport System", null);
			addFormat("TXT", "Texte Simple", null);
			addFormat("-", "-", null);
			searchForFormat("JPG", "Joint Photographic Experts Group");
			searchForFormat("PNG", "Portable Network Graphics");
			searchForFormat("GIF", "Graphics Interchange Format");
			searchForFormat("BMP", "Windows Bitmap");
			searchForFormat("WBMP", "Wireless Bitmap");
			
			pnlContent.add(UI.createLabel("Propriétés de l'image", UI.FONT_BOLD), c);
			c.gridy++;
			c.insets.left = 12;
			c.gridwidth = 1;
			pnlContent.add(UI.createLabel("Format", UI.FONT_PLAIN_ITALIC), c);
			c.gridx++;
			c.gridwidth = 3;
			pnlContent.add(cmbFormat = UI.createComboBox(lstDescs.toArray(), this, 0), c);
			c.gridx=0;
			c.gridy++;
			c.gridwidth = 1;
			pnlContent.add(UI.createLabel("Qualité", UI.FONT_PLAIN_ITALIC), c);
			c.gridx++;
			c.gridwidth = 3;
			pnlContent.add(sldQuality = new JSlider(5, 100, 100), c);
			sldQuality.addChangeListener(this);
			c.gridx=0;
			c.gridy++;
			c.gridwidth = 1;
			pnlContent.add(UI.createLabel(" ", UI.FONT_PLAIN_ITALIC), c);
			c.gridx++;
			JLabel lbl;
			pnlContent.add(UI.createLabel("Compression", UI.FONT_PLAIN_ITALIC), c);
			c.gridx++;
			pnlContent.add(lblQuality = UI.createLabel("100%", UI.FONT_PLAIN_ITALIC), c);
			lblQuality.setHorizontalAlignment(SwingConstants.CENTER);
			c.gridx++;
			pnlContent.add(lbl = UI.createLabel("Qualité", UI.FONT_PLAIN_ITALIC), c);
			lbl.setHorizontalAlignment(SwingConstants.RIGHT);
			c.insets.left = 3;
			c.gridx=0;
			c.gridy++;
			c.gridwidth = 4;
			
			pnlContent.add(UI.createLabel("Données à enregistrer", UI.FONT_BOLD), c);
			c.insets.left = 12;
			c.gridy++;
			pnlContent.add(chkSpecs = UI.createCheckBox("Spécifications (permet la ré-ouverture)", null, false, UI.FONT_PLAIN_ITALIC), c);
			c.gridy++;
			pnlContent.add(chkSources = UI.createCheckBox("Sources distribuées", null, false, UI.FONT_PLAIN_ITALIC), c);
			c.gridy++;
			pnlContent.add(chkRaw = UI.createCheckBox("Données brutes (electrons)", null, false, UI.FONT_PLAIN_ITALIC), c);
			c.gridy++;
			pnlContent.add(chkNormalised = UI.createCheckBox("Données normalisées (image affichée)", null, false, UI.FONT_PLAIN_ITALIC), c);
			c.gridy++;
			
			// description
			JPanel pnlDesc = UI.createPanel(new BorderLayout());
			pnlDesc.add(lblDescription = UI.createLabel("Description : ", UI.FONT_PLAIN_ITALIC), BorderLayout.WEST);
			pnlDesc.add(txtDesc = UI.createTextField(env.getSimul().getSimulationDescription()), BorderLayout.CENTER);
			pnlContent.add(pnlDesc, c);
			c.insets.left = 3;
			c.gridy++;

			pnlContent.add(UI.createLabel("Images", UI.FONT_BOLD), c);
			c.gridy++;
			c.insets.left = 12;
			ButtonGroup g = new ButtonGroup();
			pnlContent.add(optAllImages = UI.createRadioButton("Toutes les images", null, true, g, UI.FONT_PLAIN_ITALIC), c);
			c.gridy++;
			pnlContent.add(optActualImage = UI.createRadioButton("Image actuelle", null, false, g, UI.FONT_PLAIN_ITALIC), c);
			c.gridy++;
			
			cmbFormat.setSelectedIndex(SOSInputOutput.EXPORT_FORMAT_SOS);
			
			// ajoute le tout
			setContent(pnlContent);
			setSize(410, this.getHeight());

		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "ExportFrame()", ex);}
	}
	
	
	private final void searchForFormat(final String format, final String desc)
	{
		try{
			String formatNames[] = ImageIO.getWriterFormatNames();
			for(int i=0; i<formatNames.length; i++){
				if( formatNames[i].equals(format) ){
					Iterator it = ImageIO.getImageWritersByFormatName(format);
					if(it.hasNext()){
						ImageWriter iw = (ImageWriter)( it.next() );
						if( "JPG".equals( format ) ){
							iwpJpeg = iw.getDefaultWriteParam();
							iwpJpeg.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);
						}
						addFormat(format, desc, iw);
					}
				}
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "searchForFormat()", ex);}
	}
	
	
	private final void addFormat(final String format, final String desc, final ImageWriter writer)
	{
		try{
			lstFormats.add(format);
			lstWriters.add(writer);
			if("-".equals(format)) lstDescs.add(" ");	
			else lstDescs.add(format + " (" + desc + ")");
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "addFormat()", ex);}
	}

	private final void resetValues()
	{
		try{
			chkSpecs.setSelected(false); chkSpecs.setEnabled(false);
			chkSources.setSelected(false); chkSources.setEnabled(false);
			chkRaw.setSelected(false); chkRaw.setEnabled(false);
			chkNormalised.setSelected(false); chkNormalised.setEnabled(false);
			lblDescription.setEnabled(false);
			txtDesc.setEnabled(false); txtDesc.setText("");
			optAllImages.setSelected(true);
			optAllImages.setEnabled(false);
			optActualImage.setEnabled(false);
			sldQuality.setValue(100);
			sldQuality.setEnabled(false);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "resetValues()", ex);}
	}
	
	private final String getSelectedFormat()
	{
		try{
			return (String)( lstFormats.get( cmbFormat.getSelectedIndex() ) );
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "getSelectedFormat()", ex);}
		return null;
	}
	
	private final ImageWriter getSelectedWriter()
	{
		try{
			return (ImageWriter)( lstWriters.get( cmbFormat.getSelectedIndex() ) );
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "getSelectedWriter()", ex);}
		return null;
	}

	
	
	
	

	
	 public final void actionPerformed(ActionEvent ae)
	 {
		super.actionPerformed(ae);
		try{
			if(ae.getSource() == cmbFormat){
				switch( cmbFormat.getSelectedIndex() ){
					case SOSInputOutput.EXPORT_FORMAT_SOS: // SOS
						resetValues();
						chkSpecs.setEnabled(true); chkSpecs.setSelected(env.getPrefs().getExportSpecs());
						chkSources.setEnabled(true); chkSources.setSelected(env.getPrefs().getExportSources());
						lblDescription.setEnabled(true);
						txtDesc.setEnabled(true);
						txtDesc.setText(env.getSimul().getSimulationDescription());
						break;
					case 1: // -
						cmbFormat.setSelectedIndex(iLastSel);
						break;
					case SOSInputOutput.EXPORT_FORMAT_FITS: // FITS
						resetValues();
						chkRaw.setEnabled(true); chkRaw.setSelected(env.getPrefs().getExportRawValues());
						chkNormalised.setEnabled(true); chkNormalised.setSelected(env.getPrefs().getExportNormalizedValues());
						optAllImages.setEnabled(true);
						optActualImage.setEnabled(true);
						optAllImages.setSelected(env.getPrefs().getExportAllImages());
						optActualImage.setSelected(!env.getPrefs().getExportAllImages());
						break;
					case SOSInputOutput.EXPORT_FORMAT_TXT: // TXT
						resetValues();
						optAllImages.setEnabled(true);
						optActualImage.setEnabled(true);
						optAllImages.setSelected(env.getPrefs().getExportAllImages());
						optActualImage.setSelected(!env.getPrefs().getExportAllImages());
						break;
					case 4: // -
						cmbFormat.setSelectedIndex(iLastSel);
						break;
					default: // IMG
						resetValues();
						optActualImage.setSelected(true);
						chkNormalised.setSelected(true);
						String format = getSelectedFormat();
						if( "JPG".equals(format) && (iwpJpeg != null) ){
							// qualité
							sldQuality.setEnabled(true);
							sldQuality.setValue(90);
						}
				}
				iLastSel = cmbFormat.getSelectedIndex();
				return;
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "actionPerformed()", ex);}
	}


	public void stateChanged(ChangeEvent ce)
	{
		try{
			if(ce.getSource() == sldQuality){
				lblQuality.setText(sldQuality.getValue() + "%");
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "stateChanged()", ex);}
	}


	
	
	
	
	
	public void setVisible(final int strFormat)
	{
		cmbFormat.setSelectedIndex(strFormat);
		setVisible(true);
	}
	
	public void setVisible(boolean blVisible)
	{
		try{
			super.setVisible(blVisible);
			if((blVisible == false) && (this.retournedValue() == SOSDialog.BUTTON_OK)){	
				
				// demande le fichier de destination
				File file = SOSInputOutput.askForFileName(this, "Enregistrer", "Fichier de destination");
				if( file != null ){
					String strFile = file.getAbsolutePath();
					String strFormat = getSelectedFormat();
					String strExt = "." + strFormat.toLowerCase();
					if(strFile.toLowerCase().endsWith(strExt) == false) strFile += strExt;
					env.getPrefs().setExternalFile(strFile);
					
					// regarde le format
					switch( cmbFormat.getSelectedIndex() ){
						case SOSInputOutput.EXPORT_FORMAT_SOS: // SOS
							env.getPrefs().setExportSources(chkSources.isSelected());
							env.getPrefs().setExportSpecs(chkSpecs.isSelected());
							env.getSimul().setSimulationDescription(txtDesc.getText());
							SOSInputOutput.exportSOS(env);
							break;
						case 1: return;
						case SOSInputOutput.EXPORT_FORMAT_FITS: // FITS
							env.getPrefs().setExportNormalizedValues(chkNormalised.isSelected());
							env.getPrefs().setExportRawValues(chkRaw.isSelected());
							env.getPrefs().setExportAllImages(optAllImages.isSelected());
							SOSInputOutput.exportFITS(env);
							break;
						case SOSInputOutput.EXPORT_FORMAT_TXT: // TXT
							env.getPrefs().setExportAllImages(optAllImages.isSelected());
							SOSInputOutput.exportTXT(env);
							break;
						case 4: return;
						default: // IMG
							env.getPrefs().setImageFormat(strFormat);
							if( "JPG".equals(strFormat) && (iwpJpeg != null) ){
								iwpJpeg.setCompressionQuality( ((float)(sldQuality.getValue())) / 100.0f );
								SOSInputOutput.exportIMG(env, getSelectedWriter(), iwpJpeg);
							}else{
								SOSInputOutput.exportIMG(env, getSelectedWriter(), null);	
							}

					}
				}
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "setVisible()", ex);}
	}


	

}
