/*
 SOS Project Classes.

 Copyright (C) 2006-2008 SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sos.framework.ui.dialogs;

import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.JPanel;
import javax.swing.JTextField;

import sos.framework.ressources.Lang;
import sos.framework.simulation.simul.specs.filters.SFilter;
import sos.framework.ui.SOSDialog;
import sos.lib.logger.Logger;
import sos.lib.uicommon.UI;

public class FilterDialog extends SOSDialog
{
	private static final long serialVersionUID 			= 1L;
	
	
	private JTextField txtFilterDescription				= null;
	private JTextField txtFilterCenter					= null;
	private JTextField txtFilterDelta					= null;
	private JTextField txtFilterR						= null;
	
	private SFilter filter								= null;


	
	
	
	

	public FilterDialog(final Frame parent, final SFilter filter)
	{
		super(parent, Lang.STR_TITLE_FILTERS_MANAGER, true, false, SOSDialog.BUTTONS_OK_CANCEL);
		buildContent(filter);
	}
	
	public FilterDialog(final Dialog parent, final SFilter filter)
	{
		super(parent, Lang.STR_TITLE_FILTERS_MANAGER, true, false, SOSDialog.BUTTONS_OK_CANCEL);
		buildContent(filter);
	}
	


	private final void buildContent(final SFilter filter)
	{
		try{
			this.filter = filter;

			// construit le contenu
			GridBagConstraints c = new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 0, 3), 0, 0);
			JPanel pnlGridBag = UI.createPanel(new GridBagLayout());

			// nom du filtre
			pnlGridBag.add(UI.createLabel("  " + "Description", UI.FONT_PLAIN_ITALIC), c);
			c.gridx++;
			pnlGridBag.add(txtFilterDescription = new JTextField(filter.getDescription()), c);
			txtFilterDescription.setPreferredSize(new Dimension(200, txtFilterDescription.getPreferredSize().height));
			c.gridx = 0;

			// centre
			c.gridy++;
			pnlGridBag.add(UI.createLabel("  " + "Lambda central", UI.FONT_PLAIN_ITALIC), c);
			c.gridx++;
			pnlGridBag.add(txtFilterCenter = UI.createTextField((""+filter.getLambdaCenter()).toLowerCase(), "m"), c);
			c.gridx = 0;

			// valeurs
			c.gridy++;
			pnlGridBag.add(UI.createLabel("  " + "Largeur", UI.FONT_PLAIN_ITALIC), c);
			c.gridx++;
			pnlGridBag.add(txtFilterDelta = UI.createTextField((""+filter.getLambdaDelta()).toLowerCase(), "m"), c);
			c.gridx = 0;

			// r
			c.gridy++;
			pnlGridBag.add(UI.createLabel("  " + "R (lambda/dlambda)", UI.FONT_PLAIN_ITALIC), c);
			c.gridx++;
			pnlGridBag.add(txtFilterR = new JTextField( (""+(filter.getLambdaCenter()/filter.getLambdaDelta())).toLowerCase() ), c);
			txtFilterR.setEditable(false);
			
			setContent(pnlGridBag);

		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "FilterManager()", ex);}
	}
	

	
	public void setVisible(boolean blVisible)
	{
		try{
			// montre la fenetre
			super.setVisible(blVisible);
			
			if((blVisible == false) && (this.retournedValue() == SOSDialog.BUTTON_OK)){
				
				// nouvelles valeurs
				try{ filter.setDescription(txtFilterDescription.getText()); }catch(Exception ex) {}
				try{ filter.setLambdaCenter( Double.parseDouble(txtFilterCenter.getText()) ); }catch(Exception ex) {}
				try{ filter.setLambdaDelta( Double.parseDouble(txtFilterDelta.getText()) ); }catch(Exception ex) {}
		
			}
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "setVisible()", ex);}
	}

	
	
	
	


}
