/*
 SOS Project Classes.

 Copyright (C) 2006-2008 SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sos.framework.ui.dialogs;

import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextField;

import sos.framework.maths.Constants;
import sos.framework.maths.Maths;
import sos.framework.maths.SOSParser;
import sos.framework.ressources.Icons;
import sos.framework.ressources.Lang;
import sos.framework.simulation.simul.SSimul;
import sos.framework.simulation.simul.specs.SSpecImage;
import sos.framework.simulation.simul.specs.noises.SNoise;
import sos.framework.ui.SOSDialog;
import sos.lib.logger.Logger;
import sos.lib.mathparser.ParsedExpr;
import sos.lib.mathparser.Printer;
import sos.lib.uicommon.SOSNumberedTextPane;
import sos.lib.uicommon.SOSRichTextPane;
import sos.lib.uicommon.UI;

public class NoiseDialog extends SOSDialog
{

	private static final long serialVersionUID 			= 1L;
	
	private SNoise noise								= null;
	private SSpecImage image							= null;

	private JTextField txtDescription					= null;
	private JTextField txtTop							= null;
	private JTextField txtBottom						= null;

	private SOSNumberedTextPane txtFunction				= null;
	private SOSRichTextPane txtRichText					= null;
	private JScrollPane scrollRich						= null;
	
	private JCheckBox chkApplyForEachPixel				= null;

	private JPopupMenu popup							= null;
	private JButton btnAdd								= null;
	private JButton btnTest								= null;
	

	
	
	
	
	public NoiseDialog(final Frame parent, final SNoise noise, final SSpecImage image)
	{
		super(parent, Lang.STR_TITLE_NOISE_MANAGER, true, true, SOSDialog.BUTTONS_OK_CANCEL);
		buildContent(noise, image);
		buildPopupMenu();
	}
	
	public NoiseDialog(final Dialog parent, final SNoise noise, final SSpecImage image)
	{
		super(parent, Lang.STR_TITLE_NOISE_MANAGER, true, true, SOSDialog.BUTTONS_OK_CANCEL);
		buildContent(noise, image);
		buildPopupMenu();
	}
	
	
	
	private final void buildContent(final SNoise noise, final SSpecImage image)
	{
		try{
			this.noise = noise;
			this.image = image;
		
			// construit le contenu
			GridBagConstraints c = new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 0, 3), 0, 0);
			JPanel pnlContent = UI.createPanel(new BorderLayout());
			
			// description du bruit
			JPanel pnlGridBag = UI.createPanel(new GridBagLayout());
			pnlGridBag.add(UI.createLabel("  " + "Description", UI.FONT_PLAIN_ITALIC), c);
			c.gridx++;
			pnlGridBag.add(txtDescription = new JTextField(noise.getDescription()), c);
			c.gridx = 0;
			
			// global ou pas
			c.gridy++;
			pnlGridBag.add(UI.createPanel(null), c);
			c.gridx++;
			pnlGridBag.add(chkApplyForEachPixel = UI.createCheckBox("Appliquer à chaque pixel", this, noise.getApplyForEachPixel(), UI.FONT_PLAIN_ITALIC), c);
			pnlContent.add(pnlGridBag, BorderLayout.NORTH);
			
			// zone de texte
			txtTop = new JTextField("");
			txtBottom = new JTextField("}");
			txtFunction = new SOSNumberedTextPane();
			txtFunction.setText(noise.getFunction());
			btnAdd = UI.createButton("Ajouter", this, UI.SIZE_SMALL_ICON, Icons.ICON_BUTTON_ADD);
			btnTest = UI.createButton("Tester", this, UI.SIZE_SMALL_ICON, Icons.ICON_BUTTON_EDIT);
			c.gridy = 0;
			c.insets = new Insets(0, 3, 0, 3);
			pnlGridBag = UI.createPanel(new GridBagLayout());
			pnlGridBag.add(UI.createInputFunctionPanel(txtTop, txtFunction, txtBottom, btnAdd, btnTest), c);
			pnlContent.add(pnlGridBag, BorderLayout.CENTER);
			
			// apercu
			txtRichText = new SOSRichTextPane();
			txtRichText.setMargin(new Insets(5, 5, 5, 5));
			scrollRich = new JScrollPane(txtRichText);
			scrollRich.setBorder(null);
			c.insets = new Insets(3, 3, 0, 3);
			pnlGridBag = UI.createPanel(new GridBagLayout());
			pnlGridBag.add(scrollRich, c);
			

			JSplitPane split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, pnlContent, pnlGridBag);
			split.setBorder(null);
			split.setOneTouchExpandable(true);
			
			setContent(split);
			setSize(800, 520);
			setResizable(true);
			
			split.setResizeWeight(0.5);
			split.setDividerLocation(0.7);
			
			updateFunctionSignature();
			testNoise();
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "buildContent()", ex);}
	}

	private final void buildPopupMenu()
	{
		try{
			/* construit le menu */
			popup = new JPopupMenu();
		
			popup.add(UI.createLabel(" Ajouter...", UI.FONT_BOLD));
			popup.addSeparator();

			JMenu mnuVars = new JMenu("Variables");
			mnuVars.add(UI.createMenuItem("Durée de l'observation [" + image.getVariableValue(SSimul.VAR_OBSERVATION_TIME_NAME) + " s]", this, null, SOSParser.VAR_OBS_TIME));
			mnuVars.add(UI.createMenuItem("Diamètre du télescope [" + image.getVariableValue(SSimul.VAR_TELESCOPE_DIAMETER_NAME) + " m]", this, null, SOSParser.VAR_TELESC_DIAM));
			mnuVars.add(UI.createMenuItem("Résolution d'un pixel [" + image.getPixelResolution() + " arcsec]", this, null, SOSParser.VAR_PIX_RES));
			mnuVars.add(UI.createMenuItem("Lamdba du filtre [" + image.getFilter().getLambdaCenter() + "]", this, null, SOSParser.VAR_LAMBDA));
			mnuVars.add(UI.createMenuItem("Delta du filtre [" + image.getFilter().getLambdaDelta() + "]", this, null, SOSParser.VAR_DLAMBDA));
			mnuVars.addSeparator();
			mnuVars.add(UI.createMenuItem("Maximum des electrons des sources [-]", this, null, SOSParser.VAR_MAX_SOURCES_ELECT));
			mnuVars.add(UI.createMenuItem("Maximum des electrons totaux [-]", this, null, SOSParser.VAR_MAX_TOTAL_ELECT));
			mnuVars.addSeparator();
			mnuVars.add(UI.createMenuItem("Electrons des sources [-]", this, null, SOSParser.VAR_SOURCES_ELECT));
			mnuVars.add(UI.createMenuItem("Electrons totaux [-]", this, null, SOSParser.VAR_TOTAL_ELECT));
			mnuVars.add(UI.createMenuItem("Pixel actuel x [-]", this, null, SOSParser.VAR_X));
			mnuVars.add(UI.createMenuItem("Pixel actuel y [-]", this, null, SOSParser.VAR_Y));

			popup.add(mnuVars);
			
			JMenu mnuConst = new JMenu("Constantes");
			mnuConst.add(UI.createMenuItem("Boltzmann (K) [" + Constants.K + " J/K]", this, null, "k"));
			mnuConst.add(UI.createMenuItem("Célérité (C) [" + Constants.C + " m/s]", this, null, "c"));
			mnuConst.add(UI.createMenuItem("Charge élémentaire (Q) [" + Constants.Q + " A.s]", this, null, "q"));
			mnuConst.add(UI.createMenuItem("Néper (E) [" + Constants.E + "]", this, null, "e"));
			mnuConst.add(UI.createMenuItem("Pi [" + Constants.PI + "]", this, null, "pi"));
			mnuConst.add(UI.createMenuItem("Planck (H) [" + Constants.H + " J.s]", this, null, "h"));
			popup.add(mnuConst);
			
			JMenu mnuFuncs = new JMenu("Fonctions");
			JMenu mnuFuncsBase = new JMenu("Classiques");
			mnuFuncsBase.add(UI.createMenuItem("Exponentielle [exp]", this, null, "exp( x )"));
			mnuFuncsBase.add(UI.createMenuItem("Factorielle [fact]", this, null, "fact( x )"));
			mnuFuncsBase.add(UI.createMenuItem("Logarithme base 10 [log]", this, null, "log( x )"));
			mnuFuncsBase.add(UI.createMenuItem("Logarithme népérien [ln]", this, null, "ln( x )"));
			mnuFuncsBase.add(UI.createMenuItem("Planck [planck]", this, null, "planck( x , y )"));
			mnuFuncsBase.add(UI.createMenuItem("Puissance [pow]", this, null, "pow( x , y )"));
			mnuFuncsBase.add(UI.createMenuItem("Racine carrée [sqrt]", this, null, "sqrt( x )"));
			mnuFuncs.add(mnuFuncsBase);
			JMenu mnuFuncsArr = new JMenu("Arrondis");
			mnuFuncsArr.add(UI.createMenuItem("Arrondi [round]", this, null, "round( x )"));
			mnuFuncsArr.add(UI.createMenuItem("Partie entière [floor]", this, null, "floor( x )"));
			mnuFuncsArr.add(UI.createMenuItem("Partie entière par excès [ceil]", this, null, "ceil( x )"));
			mnuFuncs.add(mnuFuncsArr);
			JMenu mnuFuncsTrigo = new JMenu("Trigoniétrie");
			mnuFuncsTrigo.add(UI.createMenuItem("Sinus [sin]", this, null, "sin( x )"));
			mnuFuncsTrigo.add(UI.createMenuItem("Cosinus [cos]", this, null, "cos( x )"));
			mnuFuncsTrigo.add(UI.createMenuItem("Tangente [tan]", this, null, "tan( x )"));
			mnuFuncsTrigo.add(UI.createMenuItem("Arcsinus [asin]", this, null, "asin( x )"));
			mnuFuncsTrigo.add(UI.createMenuItem("Arccosinus [acos]", this, null, "acos( x )"));
			mnuFuncsTrigo.add(UI.createMenuItem("Arctangente [atan]", this, null, "atan( x )"));
			mnuFuncs.add(mnuFuncsTrigo);
			JMenu mnuFuncsBool = new JMenu("Logique");
			mnuFuncsBool.add(UI.createMenuItem("Condition [iif]", this, null, "iif( x , y , z )"));
			mnuFuncsBool.add(UI.createMenuItem("Et [&]", this, null, "x & y"));
			mnuFuncsBool.add(UI.createMenuItem("Ou [|]", this, null, "y | y"));
			mnuFuncsBool.add(UI.createMenuItem("Non [!]", this, null, "!x"));
			mnuFuncsBool.add(UI.createMenuItem("Supérieur [>]", this, null, "x > y"));
			mnuFuncsBool.add(UI.createMenuItem("Supérieur ou égal [>=]", this, null, "x >= y"));
			mnuFuncsBool.add(UI.createMenuItem("Inférieur [<]", this, null, "x < y"));
			mnuFuncsBool.add(UI.createMenuItem("Inférieur ou égal [<=]", this, null, "x <= y"));
			mnuFuncsBool.add(UI.createMenuItem("Egalité [==]", this, null, "x == y"));
			mnuFuncs.add(mnuFuncsBool);
			JMenu mnuFuncsBessel = new JMenu("Bessel");
			mnuFuncsBessel.add(UI.createMenuItem("Bessel J0 [j0]", this, null, "j0( x )"));
			mnuFuncsBessel.add(UI.createMenuItem("Bessel J1 [j1]", this, null, "j1( x )"));
			mnuFuncsBessel.add(UI.createMenuItem("Bessel Jn [jn]", this, null, "jn( n , x )"));
			mnuFuncsBessel.add(UI.createMenuItem("Bessel Y0 [y0]", this, null, "y0( x )"));
			mnuFuncsBessel.add(UI.createMenuItem("Bessel Y1 [y1]", this, null, "y1( x )"));
			mnuFuncsBessel.add(UI.createMenuItem("Bessel Yn [yn]", this, null, "yn( n , x )"));
			mnuFuncs.add(mnuFuncsBessel);
			JMenu mnuFuncsAlea = new JMenu("Aléatoire");
			mnuFuncsAlea.add(UI.createMenuItem("Tirrage gaussien [randgauss]", this, null, "randgauss( x )"));
			mnuFuncsAlea.add(UI.createMenuItem("Tirrage uniforme [randuni]", this, null, "randuni()"));
			mnuFuncs.add(mnuFuncsAlea);
			popup.add(mnuFuncs);
			
			JMenu mnuConvs = new JMenu("Conversions");
			mnuConvs.add(UI.createMenuItem("JySterad > ElectPix", this, null, "jy.sterad}elect.pix( x )"));
			mnuConvs.add(UI.createMenuItem("ElectPix > JySterad", this, null, "elect.pix}jy.sterad( x )"));
			popup.add(mnuConvs);
			
			popup.addSeparator();
			popup.add(UI.createMenuItem("Annuler", null, null));
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "buildPopupMenu()", ex);}
	}
	
	
	
	
	
	private final void updateFunctionSignature()
	{
		try{
			if( chkApplyForEachPixel.isSelected() ){
				txtTop.setText("function calculate_noise( x , y ) {");
			}else{
				txtTop.setText("function calculate_noise( void ) {");
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "updateFunctionSignature()", ex);}
	}

	private final void testNoise()
	{
		try{
			
			String txt = computeNoise();
			if(txt == null) return;
			txtRichText.setHTMLText(txt);
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "testNoise()", ex);}
	}
	
	private final String computeNoise()
	{
		try{
			SOSParser.updateNoisesParserVariablesForImage(image);

			ParsedExpr p = SOSParser.getNoisesParser().parse(txtFunction.getText());
			p.compute();
			
			if( p.getLastErr() == null ){
				// pas d'erreur
				txtBottom.setText("} = " + Maths.toString( p.value(), 3 ) );
				return Printer.explainAll(p);
			}else{
				// erreur
				txtBottom.setText("} = " + "Erreur : " + p.getLastErr() );
				Logger.notifyError(this, "L'expression du bruit contient contient une ou plusieurs erreurs.\nLe parseur à retourné : " + p.getLastErr() + "\nVous devez corriger l'erreur pour effectuer l'action demandée.", "Erreur dans l'expression");
			}

		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "computeNoise()", ex);}
		return null;
	}
	
	
	
	
	
	
	

	
	
	
	

	public final void actionPerformed(ActionEvent ae)
	{
		super.actionPerformed(ae);
		try{
			if(ae.getSource() == chkApplyForEachPixel) {updateFunctionSignature(); return;}
			if(ae.getSource() == btnAdd) {popup.show(btnAdd, btnAdd.getWidth()-btnAdd.getWidth(), 0); return;}
			if(ae.getSource() == btnTest) {testNoise(); return;}
			
			if(ae.getSource() instanceof JMenuItem){
				txtFunction.setText(txtFunction.getText().substring(0, txtFunction.getTextArea().getSelectionStart()) + ((JMenuItem)(ae.getSource())).getActionCommand() + txtFunction.getText().substring(txtFunction.getTextArea().getSelectionEnd(), txtFunction.getText().length()));
				txtFunction.requestFocus();
				return;
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "actionPerformed()", ex);}
	}
	
	public void setVisible(boolean blVisible)
	{
		try{
			
			if((blVisible == false) && (this.retournedValue() == SOSDialog.BUTTON_OK)){
				
				// test si le bruit est valide
				if( computeNoise() == null ){
					super.setVisible(true);
					return;
				}
				
				noise.setApplyForEachPixel(chkApplyForEachPixel.isSelected());
				noise.setDescription(txtDescription.getText());
				noise.setFunction(txtFunction.getText());
			}
			
			// montre la fenetre
			super.setVisible(blVisible);
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "setVisible()", ex);}
	}

	
	

}
