/*
 SOS Project Classes.

 Copyright (C) SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package sos.framework.ui.editor;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import sos.framework.simulation.SEnv;
import sos.framework.simulation.prefs._SPrefsListener;
import sos.framework.simulation.simul._SSpecsListener;
import sos.framework.ui.editor.imagecanvas.ImageCanvas;
import sos.lib.logger.Logger;
import sos.lib.uicommon.UI;

public class EditorInfoBar extends JPanel implements _SSpecsListener, _SPrefsListener, ChangeListener
{

	private static final long serialVersionUID		= 1L;

	private static final Dimension SIZE				= new Dimension(41, 41);
	private static final Color COLOR_FORGROUND		= new Color(200, 200, 200);
	private static final Color COLOR_BACKGROUND		= new Color(0, 0, 0);
	private static Font INFO_FONT					= null;
 
	// specifications de la simulation
	private SEnv sEnv								= null;
	
	private JSlider sliderZoom						= null;
	private JLabel lblZoomValue						= null;
 	
 
	
	
	
	
	

	
	
 	public EditorInfoBar(final SEnv sEnv, final ImageCanvas cv)
 	{
 		super(new BorderLayout());
 		try{
 			this.sEnv = sEnv;
 			this.setOpaque(true);
 			this.setBackground(COLOR_BACKGROUND);
 			INFO_FONT = new Font(this.getFont().getName(), Font.PLAIN, 9);
 

 			// textes
	 		lblZoomValue = UI.createLabel("");
	 		lblZoomValue.setOpaque(true);
	 		lblZoomValue.setFont(UI.FONT_PLAIN);

 			// les sliders
 			sliderZoom = UI.createSlider(JSlider.HORIZONTAL, 2, 250, 20, this);
 			JPanel pnlEast = UI.createPanel(new BorderLayout(), new Dimension(260, 40));
 			pnlEast.add(lblZoomValue, BorderLayout.NORTH);
 			pnlEast.add(sliderZoom, BorderLayout.CENTER);
	 		this.add(pnlEast, BorderLayout.EAST);
 			
 			
	 		sEnv.getPrefs().addPrefsListener(this);
	 		sEnv.getSimul().addSimulationListener(this);
	 		
 		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "EditorInfoBar()", ex);}
 	}
 	
 	
 	
 	
 	
 	
 	

	
	
	
	
	
 	public final Dimension getMinimumSize() {return SIZE;}
 	public final Dimension getMaximumSize() {return SIZE;}
 	public final Dimension getPreferredSize() {return SIZE;}
 	
	public final void paint(Graphics g)
 	{
		try{
	 		super.paint(g);
	 		Graphics2D g2d = (Graphics2D)g;
	 
	 		g.setFont(INFO_FONT);
			g.setColor(COLOR_FORGROUND); 
			
			g.drawLine(5, 33, 5, 5);
			g.drawLine(5, 5, 2, 8);
			g.drawLine(5, 5, 8, 8);
			g.drawString("x", 11, 11);
			
			g.drawLine(5, 33, 35, 33);
			g.drawLine(35, 33, 32, 30);
			g.drawLine(35, 33, 32, 36);
			g.drawString("y", 28, 26);

			int y=11;
			int x=50;
			int h=18;
			
			//g.setColor(Color.red);
			int width = getWidth()-x-270;
			
			
			int widthAll = (width-1);
			int widthBla = (int) (widthAll*sEnv.getSimul().getSpecs().getImageAt(sEnv.getSimul().getActualImageNumber()).getNormalisationBlack()/1000.0);
			int widthWhi = (int) (widthAll*(1000-sEnv.getSimul().getSpecs().getImageAt(sEnv.getSimul().getActualImageNumber()).getNormalisationWhite())/1000.0);
			int widthGra = widthAll - widthWhi - widthBla;

			g2d.setColor(Color.black);
			g2d.fillRect(x+1, y+1, widthBla, h-1);
			
			
			g2d.setPaint(new GradientPaint(51+widthBla, 0, Color.black, 51+widthBla+widthGra, 0, Color.white));
			g2d.fillRect(x+1+widthBla, y+1, widthGra, h-1);
			
			
			g2d.setColor(Color.white);
			g2d.fillRect(x+1+widthBla+widthGra, y+1, widthWhi, h-1);
			
			g.setColor(COLOR_FORGROUND);
			
			g2d.drawRect(x, y, width, h);
			
			g2d.drawLine(x+widthBla, y+1, x+widthBla, y+h);
			g2d.drawLine(x+widthBla+widthGra/2, y+1, x+widthBla+widthGra/2, y+h);
			g2d.drawLine(x+widthBla+widthGra+1, y+1, x+widthBla+widthGra+1, y+h);
			
			g.drawString("0", x-2+widthBla, 9);
			g.drawString("127", x+widthBla+widthGra/2-8, 9);
			g.drawString("255", x+widthBla+widthGra-16, 9);
			
			///g.drawString("0", x-2, y+h+10);
			//g.drawString("127", x+(width-1)/2, y+h+10);
			//g.drawString("255", x+width-30, y+h+10);
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "paint()", ex);}
	 }
	

	
	

	
	
	private final void updateZoomLabel(final double newVal)
 	{
		try{
			lblZoomValue.setText(" Zoom : " + newVal + "x");
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "updateZoomLabel()", ex);}
 	}
	

	private final void updatePrefs()
 	{
		try{
			int newVal = (int)( 10.0 * sEnv.getPrefs().getZoomFactor() );
			if(sliderZoom.getValue() != newVal){
				updateZoomLabel(newVal/10.0);
				sliderZoom.removeChangeListener(this);
				sliderZoom.setValue(newVal);
				sliderZoom.addChangeListener(this);
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "updatePrefs()", ex);}
 	}
	
    public void stateChanged(ChangeEvent e)
    {
    	try{
	        JSlider source = (JSlider)e.getSource();
	    
	        // facteur de zoom
	        if( source == sliderZoom ){
	        	double newVal = ( ((double)sliderZoom.getValue()) / 10.0 );
				if(newVal < 0.2) newVal = 0.2;
				updateZoomLabel(newVal);
				sEnv.getPrefs().setZoomFactor(newVal);
	        	return;
	        }    
    	}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "stateChanged()", ex);}
    }

    
    
    
    
    
    
	public void onSimulationStateChanged(boolean blIsSimulating) {}

	public void onSimulationStageChanged(int iSimulationStage)
	{
		this.repaint();
	}

	public void onSimulationImageChanged()
	{
		updatePrefs();
	}

	public void onPreferencesChanged()
	{
		updatePrefs();
	}

	
}

 