/*
 SOS Project Classes.

 Copyright (C) SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package sos.framework.ui.editor;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.Vector;

import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.border.LineBorder;

import sos.framework.ui.SOSFrame;
import sos.lib.logger.Logger;
import sos.lib.uicommon.UI;

public class EditorLateralBar extends JPanel implements MouseListener, MouseMotionListener
{

	private static final long serialVersionUID			= 1L;

	/* les orientations possibles pour la barre */
	public static final int LATERAL_BAR_LEFT			= 1;
	public static final int LATERAL_BAR_RIGHT			= 2;
	
	/* les mode possibles pour la barre */
	public static final int LATERAL_BAR_HIDDEN			= 1;
	public static final int LATERAL_BAR_SHOWN			= 2;
	public static final int LATERAL_BAR_AUTO			= 3;
	
	private static final int LATERAL_BAR_MIN_SIZE		= 150;
	private static final int LATERAL_BAR_MAX_SIZE		= 500;
	private static final int LATERAL_BAR_SIZER_SIZE		= 7;
	
	
	

	/* toutes les infos sur la simulation actuelle */
	private SOSFrame window							= null;
	
	/* l'orientation de cette barre */
	private int iBarOrientation							= 0;
	
	/* la taille actuelle */
	private int iAcualWidth								= 0;

	/* les composants graphiques */
	private JPanel pnlSizer								= null;
	private JPanel pnlDimer								= null;
	private JPanel pnlContent							= null;
	private GridBagConstraints c						= null;
	
	/* informations sur l'etat visible */
	private boolean blIsShown							= false;
 
 	/* memorise si on est entrain de dimensionner */
	private boolean blChangingSize						= false;
	private int iChangingSize							= 0;
 	
	/* liste de dialogues affiches */
	private Vector lstDialogs							= new Vector();
	
	

	
	
 	public EditorLateralBar(final SOSFrame window, final int iBarOrientation)
 	{
 		super(new BorderLayout());
 		try{
 			this.window = window;
 			this.iBarOrientation = iBarOrientation;
 			
 			iAcualWidth = 370;
 			//if(iBarOrientation == LATERAL_BAR_LEFT) iAcualWidth = 370;
 			//else iAcualWidth = 280;
 			
 			this.buildLateralBar();
 		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "EditorLateralBar()", ex);}
 	}

 	
 	
 	
 	public final void buildLateralBar()
 	{
 		try{
 			this.setOpaque(false);
 			//this.setBackground(Color.darkGray);
 			
 			// cree la barre de redimensionnement
 			pnlSizer = UI.createPanel(null, new Dimension(LATERAL_BAR_SIZER_SIZE, LATERAL_BAR_SIZER_SIZE));
 			pnlSizer.setOpaque(false);
 			//pnlSizer.setBackground(Color.lightGray);
 			pnlSizer.addMouseListener(this);
 			pnlSizer.addMouseMotionListener(this);
 			pnlSizer.setCursor(Cursor.getPredefinedCursor(Cursor.W_RESIZE_CURSOR));
 			
 			pnlDimer = new LateralBarSizer();
 			pnlSizer.add(pnlDimer);
 			
 			
 			// ajout la barre
 			if(iBarOrientation == LATERAL_BAR_LEFT) this.add(pnlSizer, BorderLayout.EAST);
 			else this.add(pnlSizer, BorderLayout.WEST);
 			// ajoute le panneau central
 			pnlContent = UI.createPanel(new GridBagLayout());
 			c = new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 0, 2), 0, 0);
 			JPanel pnlCenter = UI.createPanel(new BorderLayout());
 			pnlCenter.add(pnlContent, BorderLayout.NORTH);
 			pnlCenter.add(UI.createPanel(null), BorderLayout.CENTER);
			this.add(pnlCenter, BorderLayout.CENTER);
			// taille initiale
 			updateSize();
 		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "buildLateralBar()", ex);}
 	}
 	
 	
 	
 	
 	
 	public final void addDialog(final JPanel theDialog)
 	{
 		if( blIsShown == false ){
 			blIsShown = true;
 			updateSize();
 		}
 		lstDialogs.add(theDialog);
 		pnlContent.add(theDialog, c);
 		c.gridy++;
 		pnlContent.updateUI();
 		updateSizerHeight();
 	}
 	
 	
 	public final void removeDialog(final JPanel theDialog)
 	{
 		lstDialogs.remove(theDialog);
 		pnlContent.removeAll();
 		c.gridy = 0;
 		if(lstDialogs.size() > 0){
 			for(int i=0; i<lstDialogs.size(); i++){
 				pnlContent.add((JPanel)(lstDialogs.get(i)), c);
 				c.gridy++;
 			}
 		}else{
 			if(blIsShown == true){
 	 			blIsShown = false;
 	 			updateSize();
 	 		}
 		}
 		pnlContent.updateUI();
 		updateSizerHeight();
 	}
 	
 	public final void updateSizerHeight()
 	{
 		if(!blIsShown){
 			
 			
 		}else{
 	 		Runnable callRevalidate = new Runnable()
 	 		{
 	            public void run()
 	            {
 	            	int height = -2;
 	         		for(int i=0; i<lstDialogs.size(); i++) height += ((JPanel)(lstDialogs.get(i))).getHeight() + 2;
 	            	if(height > 0) pnlDimer.setSize(LATERAL_BAR_SIZER_SIZE, height);
 	         		else pnlDimer.setSize(LATERAL_BAR_SIZER_SIZE, 30);
 	            }
 	        };
 	        SwingUtilities.invokeLater(callRevalidate);
 		}
 	}
 	
 	
 	
 	
 	
 	
 	
 	public final void updateSize()
 	{
 		try{
 			Dimension d;
 			if(blIsShown == true){
 	 			if(iAcualWidth < LATERAL_BAR_MIN_SIZE) iAcualWidth = LATERAL_BAR_MIN_SIZE;
 	 			if(iAcualWidth > LATERAL_BAR_MAX_SIZE) iAcualWidth = LATERAL_BAR_MAX_SIZE;
 	 			d = new Dimension(iAcualWidth, iAcualWidth);
 			}else{
 				d = new Dimension(LATERAL_BAR_SIZER_SIZE, LATERAL_BAR_SIZER_SIZE);
 			}
 			this.setMaximumSize(d);
 			this.setMinimumSize(d);
 			this.setPreferredSize(d);
 			this.updateUI();
 		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "updateSize()", ex);}
 	}
 	
 	 	
 	


 	
 	
	
    public void mouseClicked(MouseEvent e)
    {
    	if(blIsShown == false && lstDialogs.size() < 1) return;
    	blIsShown = !blIsShown;
    	updateSize();
    	updateSizerHeight();
    }
    
    public void mouseReleased(MouseEvent e)
    {
    	blChangingSize = false;
    	pnlDimer.setBackground(Color.lightGray);
    	window.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }
    
    public void mouseMoved(MouseEvent e)
    {
		try{
	 		if(blChangingSize == true){
	 			if(iBarOrientation == LATERAL_BAR_LEFT) iAcualWidth = e.getX() + iAcualWidth - LATERAL_BAR_SIZER_SIZE - iChangingSize;
	 			else iAcualWidth = iChangingSize - e.getX() + iAcualWidth;
	 			updateSize();
	 		}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "mouseMoved()", ex);}
    }

    public void mousePressed(MouseEvent e) 
    {    
    	try{
	    	if(blIsShown){
	    		blChangingSize = true;
	    		pnlDimer.setBackground(Color.darkGray);
	    		window.setCursor(Cursor.getPredefinedCursor(Cursor.W_RESIZE_CURSOR));
	 			if(iBarOrientation == LATERAL_BAR_LEFT) iChangingSize = e.getX() - LATERAL_BAR_SIZER_SIZE;
	 			else iChangingSize = e.getX();
	    	}
    	}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "mousePressed()", ex);}
    }

    public void mouseDragged(MouseEvent e) {mouseMoved(e);}
    public void mouseEntered(MouseEvent e) {pnlDimer.setBackground(Color.darkGray);}
    public void mouseExited(MouseEvent e) {if(blChangingSize == false) pnlDimer.setBackground(Color.lightGray);}
    
 
    
    
    
    public class LateralBarSizer extends JPanel
    {

		private static final long serialVersionUID			= 1L;
    	
		public LateralBarSizer()
		{
			super(null);
 			this.setOpaque(true);
 			this.setBackground(Color.lightGray);
 			this.setBounds(0, 2, LATERAL_BAR_SIZER_SIZE, LATERAL_BAR_SIZER_SIZE);
 			this.setBorder(new LineBorder(Color.darkGray, 1));
		}
		
		public void paint(Graphics g)
		{
			super.paint(g);
			g.setColor(Color.black);
			g.drawLine(2, this.getSize().height/2-15, 2, this.getSize().height/2+15);
			g.drawLine(4, this.getSize().height/2-15, 4, this.getSize().height/2+15);
		}
    	
    }

    
}
