/*
 SOS Project Classes.

 Copyright (C) 2006-2008 SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sos.framework.ui.editor;

import java.awt.BorderLayout;
import java.awt.Point;

import javax.swing.JPanel;

import sos.framework.simulation.SEnv;
import sos.framework.ui.editor.controlbar.ControlBar;
import sos.framework.ui.editor.imagecanvas.ImageCanvas;
import sos.lib.logger.Logger;
import sos.lib.uicommon.UI;

public class SOSEditor extends JPanel
{

	private static final long serialVersionUID		= 1L;

	/* la zone d'affichage de l'image */
	private ImageCanvas cv							= null;
	
	/* la barre de controle de la simulation */
	private ControlBar bar							= null;
	
	/* la barre d'infos de la simulation */
	private EditorInfoBar infos						= null;
	
	/* les barres laterales */
	private EditorLateralBar lBarLeft				= null;
	private EditorLateralBar lBarRight				= null;
	
	
	
	
	
	
	/**
	 * Construit l'editeur du simulateur.
	 * 
	 * @param env L'environnement de la simulation
	 */
	public SOSEditor(final SEnv env)
	{
		super(new BorderLayout());
		try{
			this.setOpaque(false);
			
			/* contruit le contenu */
			cv = new ImageCanvas(env);
			bar = new ControlBar(env);
			infos = new EditorInfoBar(env, cv);
			lBarLeft = new EditorLateralBar(env.getWindow(), EditorLateralBar.LATERAL_BAR_LEFT);
			lBarRight = new EditorLateralBar(env.getWindow(), EditorLateralBar.LATERAL_BAR_RIGHT);
			
			/* on ajoute le tout */
			JPanel pnlCenter = UI.createPanel(new BorderLayout());
			cv.add(lBarLeft, BorderLayout.WEST);
			cv.add(lBarRight, BorderLayout.EAST);
			pnlCenter.add(cv, BorderLayout.CENTER);

			this.add(bar, BorderLayout.NORTH);
			this.add(pnlCenter, BorderLayout.CENTER);
			this.add(infos, BorderLayout.SOUTH);

		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "SOSEditor()", ex);}
	}
	

	
	
	
	

	
	
	

	
	
	
	
	
	
	
	public final ImageCanvas getImageCanvas() {return cv;}
	
	public final ControlBar getControlBar() {return bar;}
	
	public final EditorInfoBar getInfosBar() {return infos;}
	
	public final EditorLateralBar getLeftBar() {return lBarLeft;}
	
	public final EditorLateralBar getRightBar() {return lBarRight;}
	
	
	public final Point getContentPanelLocationOnScreen(){
		try{
			return new Point(bar.getLocationOnScreen().x, bar.getLocationOnScreen().y + bar.getHeight());
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "getContentPanelLocationOnScreen()", ex);}
		return new Point(0, 0);
	}


}
