/*
 SOS Project Classes.

 Copyright (C) SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package sos.framework.ui.editor.controlbar;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.Timer;
import javax.swing.border.LineBorder;

import sos.framework.simulation.SEnv;
import sos.framework.simulation.simul.SSimul;
import sos.framework.simulation.simul._SSourcesListener;
import sos.framework.simulation.simul._SSpecsListener;
import sos.lib.logger.Logger;
import sos.lib.uicommon.UI;

public class BarInfosZone extends JPanel implements ActionListener, _SSpecsListener, _SSourcesListener
{

	
	private static final long serialVersionUID			= 1L;
	
	private SSimul simul								= null;
	
	private JLabel lblInfo1								= null;
	private JLabel lblInfo2								= null;
	private JLabel lblInfo3								= null;

	private JProgressBar progressBar					= null;
	private Timer timer									= null;

	private long start									= 0;
	private int done									= 0;

	

	
	
	
 	public BarInfosZone(final SEnv env)
 	{
 		super(null);
 		try{
 			this.simul = env.getSimul();
 			this.setOpaque(true);
 			this.setBorder(new LineBorder(Color.darkGray, 1));
 			this.setMaximumSize(ControlBar.INFOS_DIMENSION);
 			this.setMinimumSize(ControlBar.INFOS_DIMENSION);
 			this.setPreferredSize(ControlBar.INFOS_DIMENSION);
 			/* icone des informations */
 			JLabel lblIconT = new JLabel("  ");
 			lblIconT.setBounds(0, 0, 12, 12);
 			/* labels des informations */
 			lblInfo1 = UI.createLabel("nouvelle simulation", UI.FONT_BOLD_SMALL);
 			lblInfo1.setBounds(4, 0, ControlBar.INFOS_DIMENSION.width-14, 14);
 			lblInfo2 = UI.createLabel("", UI.FONT_PLAIN_VERY_SMALL);
 			lblInfo2.setBounds(4, 12, ControlBar.INFOS_DIMENSION.width-14, 12);
 			lblInfo3 = UI.createLabel("", UI.FONT_PLAIN_VERY_SMALL);
 			lblInfo3.setBounds(4, 18, ControlBar.INFOS_DIMENSION.width-14, 12);
 			/* icone de la progression */
 			JLabel lblIconP = new JLabel("  ");
 			lblIconP.setBounds(0, ControlBar.INFOS_DIMENSION.height-14, 12, 12);
 			/* progression */
			progressBar = new JProgressBar(0, 1);
			progressBar.setStringPainted(false);
			progressBar.setString(null);
			progressBar.setValue(0);
			progressBar.setBounds(0, ControlBar.INFOS_DIMENSION.height-18, ControlBar.INFOS_DIMENSION.width, 18);
	 		/* on ajoute le tout */
			this.add(lblIconT);
	 		this.add(lblInfo1);
	 		this.add(lblInfo2);
	 		this.add(lblInfo3);
	 		this.add(lblIconP);
	 		this.add(progressBar);
	 		/* ecoute les infos de la simulation */
	 		env.getSimul().addSimulationListener(this);
	 		env.getSimul().addSourcesListener(this);
 		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "InfosZone()", ex);}
 	}



 	
 	
 	
 	
 	
	private final void addInfo(final String strInfo)
	{
		try{
			//lblInfo3.setText(lblInfo2.getText());
			lblInfo2.setText(lblInfo1.getText());
			lblInfo1.setText(strInfo);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "addInfo()", ex);}
	}
	
	
	
	
	
	
	
	
	
	
	private final void startProgress()
	{
		try{
			/* initialise la barre de progression */
			progressBar.setIndeterminate(true);
			progressBar.setStringPainted(false);
			/* pour calculer le temps */
			start = System.currentTimeMillis();
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "startProgress()", ex);}
	}
	
	private final void stopProgress()
	{
		try{
			/* arrete le compteur */
			if(timer != null){
				timer.stop();
				timer = null;
			}
			/* regarde ou on s'est arreté dans la simulation */
			if(simul.getResults().getImageAt(simul.getActualImageNumber()).getNormalizedImage() == null){
				addInfo("initialisation de l'image");
				addInfo("attente de génération");
				progressBar.setValue(0);
				progressBar.setIndeterminate(false);
				progressBar.setStringPainted(false);
			}else{
				addInfo("image générée en " + ( (int) ( (System.currentTimeMillis() - start + 500) / 1000 ) ) + " sec");
				progressBar.setValue(progressBar.getMaximum());
				progressBar.setIndeterminate(false);
				progressBar.setStringPainted(true);
			}
			start = 0;
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "stopProgress()", ex);}
	}
	
	
	
	

	
	public final void actionPerformed(ActionEvent ae) {progressBar.setValue(done);}

	
	
	
	
	
	public void onSimulationComponentsChanged() {}
	public void onSimulationImageChanged() {}
	public void onSimulationComponentsGenerated(int iGeneratedNumber) {this.done = iGeneratedNumber;}
	
	public void onSimulationStateChanged(boolean blIsSimulating)
	{
		if(blIsSimulating == true) startProgress();
		else stopProgress();
	}
	
	public void onSimulationStageChanged(int iSimulationState)
	{
		try{
			switch(iSimulationState){
				case SSimul.STAGE_DISTRIBUTION:
					addInfo("distribution des objets...");
					break;
				
				case SSimul.STAGE_GENERATION:
					addInfo("génération des objets...");
					progressBar.setValue(0);
					progressBar.setMaximum(simul.getSpecs().getSourcesContainer().getSourcesCount());
					progressBar.setString(null);
					progressBar.setStringPainted(true);
					progressBar.setIndeterminate(false);
					timer = new Timer(1000, this);
			        timer.start();
					break;
					
				case SSimul.STAGE_NOISES:
					if(timer != null){
						timer.stop();
						timer = null;
					}
					addInfo("ajout des bruits...");
					progressBar.setStringPainted(false);
					progressBar.setIndeterminate(true);
					break;
					
				case SSimul.STAGE_NORMALISATION:
					addInfo("normalisation de l'image...");
					break;

			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "stopProgress()", ex);}
	}
	


	
	
	
	
	
}




