/*
 SOS Project Classes.

 Copyright (C) 2006-2008 SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sos.framework.ui.editor.controlbar;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.LineBorder;

import sos.framework.simulation.SEnv;
import sos.framework.simulation.simul._SSpecsListener;
import sos.framework.simulation.simul.specs.SSpecImage;
import sos.lib.logger.Logger;
import sos.lib.uicommon.SOSComboBox;
import sos.lib.uicommon.UI;

public class BarSelectImageZone extends JPanel implements _SSpecsListener, ActionListener
{

	
	private static final long serialVersionUID			= 1L;
	
	private SEnv env									= null;
	
	private SOSComboBox optImages						= null;
	

	
	
	
 	public BarSelectImageZone(final SEnv env)
 	{
 		super(null);
 		try{
 			this.env = env;
 			this.setOpaque(true);
 			this.setBorder(new LineBorder(Color.darkGray, 1));
 			
 			// fixe la taille
 			this.setMaximumSize(ControlBar.IMAGE_DIMENSION);
 			this.setMinimumSize(ControlBar.IMAGE_DIMENSION);
 			this.setPreferredSize(ControlBar.IMAGE_DIMENSION);
 			
 			// texte
 			JLabel lblIconT = UI.createLabel("Image :", UI.FONT_PLAIN_ITALIC);
 			lblIconT.setBounds(5, 0, ControlBar.STAGE_DIMENSION.width-5, 18);
 			
	 		// choix de l'image
 			optImages = UI.createComboBox(null, this, -1);
 			optImages.setBounds(0, ControlBar.IMAGE_DIMENSION.height-18, ControlBar.IMAGE_DIMENSION.width, 18);
			
 			// ajoute le tout
			this.add(lblIconT);
	 		this.add(optImages);

	 		// ecoute les evenements de la simulation
	 		env.getSimul().addSimulationListener(this);
 		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "BarSelectImageZone()", ex);}
 	}



 	
 	
 	

	private final void updateImages()
	{
		try{
			if( env.getSimul().getActualImageNumber() < 0 ){
				// fin de la simulation
				optImages.setModel(new DefaultComboBoxModel());
				env.getWindow().setTitle("SOS Project Simulator");
				return;
			}

 			// selectionne l'image courante dans la liste
			optImages.removeActionListener(this);
			SSpecImage actuImage = env.getSimul().getSpecs().getActualImage();
			optImages.setModel(new DefaultComboBoxModel(env.getSimul().getSpecs().getImages()));
			optImages.setSelectedIndex(actuImage.getImageNumber());
			optImages.addActionListener(this);

			// TODO meilleur titre
			env.getWindow().setTitle("SOS Project Simulator - " + actuImage.getImageName());
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "updateImages()", ex);}
	}
	
	public final void actionPerformed(ActionEvent ae)
	{
		try{
			if(ae.getSource() == optImages){
				// selection d'une image
				env.getSimul().setActualImageNumber(optImages.getSelectedIndex());
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "actionPerformed()", ex);}
	}
	

	
	
	public void onSimulationImageChanged()
	{
		updateImages();
	}
	
	public void onSimulationStateChanged(boolean blIsSimulating) {}
	public void onSimulationStageChanged(int iSimulationState) {}
	



}




