/*
 SOS Project Classes.

 Copyright (C) SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package sos.framework.ui.editor.controlbar;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.SwingConstants;

import sos.lib.logger.Logger;
import sos.lib.uicommon.UI;

public class BarStageZone extends JButton implements MouseListener
{

	private static final long serialVersionUID			= 1L;

	public static final int STATE_DONE					= 0;
	public static final int STATE_CURR					= 1;
	public static final int STATE_WAIT					= 2;
	
	private boolean blOver								= false;
	private boolean blCurrent							= false;
	
	private ImageIcon icActive							= null;
	private ImageIcon icInactive						= null;
	
	private JLabel lblState								= null;

	
	

	
	
	
 	public BarStageZone(final ActionListener lst, final String strTitle, final ImageIcon icActive, final ImageIcon icInactive)
 	{
 		super("");
 		try{
 			// memorise les images
 			this.icActive = icActive;
 			this.icInactive = icInactive;
 			
 			// proprietes
			this.setToolTipText(strTitle);
			this.setPreferredSize(ControlBar.STAGE_DIMENSION);
			this.setMaximumSize(ControlBar.STAGE_DIMENSION);
			this.setMinimumSize(ControlBar.STAGE_DIMENSION);
			this.setOpaque(true);
			this.setBackground(Color.lightGray);
			this.setIcon(icInactive);
			this.addActionListener(lst);
 			this.addMouseListener(this);
 			this.setLayout(null);
 			this.setBorder(null);
 			this.setMargin(new Insets(0, 0, 0, 0));
 			
 			// titre
 			JLabel lbl = UI.createLabel(strTitle, UI.FONT_BOLD_ITALIC);
 			lbl.setHorizontalAlignment(SwingConstants.RIGHT);
 			lbl.setBounds(2, 0, ControlBar.STAGE_DIMENSION.width-6, 20);
 			this.add(lbl);
 			
 			// icon d'etat
 			lblState = UI.createLabel("");
 			lblState.setBounds(ControlBar.STAGE_DIMENSION.width-17, ControlBar.STAGE_DIMENSION.height-17, 14, 14);
 			this.add(lblState);
 			
 		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "BarStageZone()", ex);}
 	}

 	
 	
 	public final void setStageState(final int iState)
 	{
 		try{
	 		switch(iState){
	 			case STATE_DONE:
	 				blCurrent = false;
	 				this.setIcon(icActive);
	 				break;
	 			case STATE_CURR:
	 				blCurrent = true;
	 				this.setIcon(icActive);
	 				break;
	 			case STATE_WAIT:
	 				blCurrent = false;
	 				this.setIcon(icInactive);
	 				break;
	 		}
	 		this.repaint();
 		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "setStageState()", ex);}
 	}
 	
 	public final void setSmallIcon(final ImageIcon icStateIcon)
 	{
 		try{
 			lblState.setIcon(icStateIcon);
 		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "setIconState()", ex);}
 	}
 	
 	
 	
 	
 	
 	
 	public final void paintComponent(Graphics g)
 	{
 		super.paintComponent(g);
 		try{
	 		if(blCurrent || blOver){
	 			g.setColor(Color.black);
	 			g.drawRect(0, 0, ControlBar.STAGE_DIMENSION.width-1, ControlBar.STAGE_DIMENSION.height-1);
	 	 		g.drawRect(0, 0, ControlBar.STAGE_DIMENSION.width-2, ControlBar.STAGE_DIMENSION.height-2);
	 		}
 		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "paintComponent()", ex);}
 	}
 	
    public final void mouseEntered(MouseEvent e)
    {
    	blOver = true;
    	this.repaint();
    }
    
    public final void mouseExited(MouseEvent e)
    {
    	blOver = false;
    	this.repaint();
    }
    
    
    
 	public final void mouseClicked(MouseEvent e) {}
    public final void mousePressed(MouseEvent e) {}
    public final void mouseReleased(MouseEvent e) {}


}
