/*
 SOS Project Classes.

 Copyright (C) SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package sos.framework.ui.editor.controlbar;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JPanel;

import sos.framework.generator.SOSGenerator;
import sos.framework.ressources.Icons;
import sos.framework.simulation.SEnv;
import sos.framework.simulation.simul.SSimul;
import sos.framework.simulation.simul._SImageListener;
import sos.framework.simulation.simul._SSpecsListener;
import sos.lib.logger.Logger;

public class ControlBar extends JPanel implements ActionListener, _SSpecsListener, _SImageListener
 {

	private static final long serialVersionUID			= 1L;
	
	public static final Dimension GAP_DIMENSION			= new Dimension(4, 4);
	public static final Dimension SPACE_DIMENSION		= new Dimension(14, 14);
	public static final Dimension STAGE_DIMENSION		= new Dimension(90, 42);
	public static final Dimension START_DIMENSION		= new Dimension(42, 42);
	public static final Dimension INFOS_DIMENSION		= new Dimension(170, 42);
	public static final Dimension IMAGE_DIMENSION		= new Dimension(100, 42);
	public static final Dimension BAR_DIMENSION			= new Dimension(60, 50);
	
	public static final Color COLOR_BAR					= new Color(200, 221, 242);

	private SEnv env									= null;

	private BarStageZone[] stages						= new BarStageZone[SSimul.STAGES_NUMBER];
	private BarStartZone btnStart						= null;
	
	private int iCurrentStage							= 0;
	

	

/*
	private JPopupMenu mnuPopup = null;
	private JMenuItem mnuItemPixel = null;
	private JMenuItem mnuItemIntens = null;
	private JMenuItem mnuItemInfos = null;
*/
	
	
	

	
 	public ControlBar(final SEnv env)
 	{
 		super(null);
 		try{
 			this.env = env;
 			env.getSimul().addSimulationListener(this);
 			env.getSimul().addSpecsListener(this);
 			
 			this.setOpaque(true);
 			this.setBackground(COLOR_BAR);
 			this.setMaximumSize(BAR_DIMENSION);
 			this.setMinimumSize(BAR_DIMENSION);
 			this.setPreferredSize(BAR_DIMENSION);
 			this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
 			
	 		// on construit les elements graphiques
 			this.add(Box.createRigidArea(GAP_DIMENSION));
 			this.add(btnStart = new BarStartZone(this, "Lancer la simulation", Icons.ICON_START_SIMULATION));
			this.add(Box.createRigidArea(GAP_DIMENSION));
 			this.add(stages[SSimul.STAGE_INITIALISATION] = new BarStageZone(this, "Parametres", Icons.ICON_PROPERTIES_0, Icons.ICON_PROPERTIES_1));
 			this.add(Box.createRigidArea(GAP_DIMENSION));
			this.add(stages[SSimul.STAGE_DISTRIBUTION] = new BarStageZone(this, "Objets", Icons.ICON_DISTRIBUTION_0, Icons.ICON_DISTRIBUTION_1));
			this.add(Box.createRigidArea(GAP_DIMENSION));
			this.add(stages[SSimul.STAGE_GENERATION] = new BarStageZone(this, "Generation", Icons.ICON_GENERATION_0, Icons.ICON_GENERATION_1));
			this.add(Box.createRigidArea(GAP_DIMENSION));
			this.add(stages[SSimul.STAGE_NOISES] = new BarStageZone(this, "Bruits", Icons.ICON_NOISIFICATION_0, Icons.ICON_NOISIFICATION_1));
			this.add(Box.createRigidArea(GAP_DIMENSION));
			this.add(stages[SSimul.STAGE_NORMALISATION] = new BarStageZone(this, "Image", Icons.ICON_NORMALIZATION_0, Icons.ICON_NORMALIZATION_1));
			this.add(Box.createRigidArea(GAP_DIMENSION));
			this.add(stages[SSimul.STAGE_END] = new BarStageZone(this, "Etudes", Icons.ICON_STUDIES_0, Icons.ICON_STUDIES_1));
 			this.add(Box.createRigidArea(SPACE_DIMENSION));
 			this.add(new BarSelectImageZone(env));
			this.add(Box.createRigidArea(GAP_DIMENSION));
			//this.add(new BarInfosZone(env.getSimSpecs()));
			//this.add(Box.createRigidArea(GAP_DIMENSION));

			// this.buildPopupMenu();
			//this.setSimulationStage(SSpecs.STAGE_INITIALISATION);
 		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "ControlBar()", ex);}
 	}

 	
 	/*
	private final void buildPopupMenu()
	{
		try{
			mnuPopup = new JPopupMenu();
			mnuPopup.add(UI.createLabel(" Outils", UI.FONT_PLAIN_ITALIC));
			mnuPopup.addSeparator();
			mnuPopup.add(mnuItemPixel = UI.createMenuItem("Sonde Pixel", this, null, null));
			mnuPopup.add(mnuItemIntens = UI.createMenuItem("Intensité", this, null, null));
			mnuPopup.addSeparator();
			mnuPopup.add(mnuItemInfos = UI.createMenuItem("Informations", this, null, null));
		}catch(Exception ex) {SLogs.fireError(this.getClass().getName(), "buildPopupMenu()", ex);}
	}
	*/
	
	
	
 	
 	

	

	private final void updateStatesIcons()
	{
		try{
			if(env.getSimul().getActualImageNumber() < 0){
				// fin de la simulation
				return;
			}

			// met a jour les icones
			for(int i=0; i<SSimul.STAGES_NUMBER; i++){
				
				// image principale
				if( i < iCurrentStage  ) stages[i].setStageState(BarStageZone.STATE_DONE);
				if( i == iCurrentStage ) stages[i].setStageState(BarStageZone.STATE_CURR);
				if( i > iCurrentStage  ) stages[i].setStageState(BarStageZone.STATE_WAIT);
				
				// petites icones
				if( (i > SSimul.STAGE_INITIALISATION) && (i < SSimul.STAGE_END) ){
					if(env.getSimul().needGenerateStage(env.getSimul().getActualImageNumber(), i)){
						if( (i == SSimul.STAGE_DISTRIBUTION) && (env.getSimul().getSpecs().isUseGenerator() == false) ){
							stages[i].setSmallIcon(Icons.ICON_STAGE_LOCKED);
						}else{
							stages[i].setSmallIcon(Icons.ICON_STAGE_NEEDED);
						}
					}else{
						stages[i].setSmallIcon(null);
					}
				}
			}
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "updateStatesIcons()", ex);}
	}
	
	


	

	public final void actionPerformed(ActionEvent ae)
	{
		try{
			if(ae.getSource() == stages[SSimul.STAGE_INITIALISATION]) 	{env.getWindow().togglePropertiesDialog(false); 		return;}
			if(ae.getSource() == stages[SSimul.STAGE_DISTRIBUTION]) 	{env.getWindow().toggleDistributionDialog(false); 	return;}
			if(ae.getSource() == stages[SSimul.STAGE_GENERATION]) 		{env.getWindow().toggleGenerationDialog(false); 		return;}
			if(ae.getSource() == stages[SSimul.STAGE_NOISES]) 			{env.getWindow().toggleNoisesDialog(false); 			return;}
			if(ae.getSource() == stages[SSimul.STAGE_NORMALISATION]) 	{env.getWindow().toggleNormalizationDialog(false); 	return;}
			if(ae.getSource() == stages[SSimul.STAGE_END]) 				{env.getWindow().togglePixelInfosDialog(false); 		return;}
			if(ae.getSource() == btnStart) 								{SOSGenerator.generate(env);							return;}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "actionPerformed()", ex);}
	}
	
	
	
	
	
	
	
	
	public void onSpecsChanged()
	{
		updateStatesIcons();
	}
	
	public void onSimulationStateChanged(boolean blSimulating)
	{
		btnStart.setEnabled(!blSimulating);
		if(blSimulating == false){
			iCurrentStage = SSimul.STAGE_END;
			updateStatesIcons();
		}
	}
	
	public void onSimulationStageChanged(int iSimulationState)
	{
		iCurrentStage = iSimulationState;
		updateStatesIcons();
	}

	public void onSimulationImageChanged()
	{
		updateStatesIcons();
	}
		
}



