/*
 SOS Project Classes.

 Copyright (C) SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package sos.framework.ui.editor.imagecanvas;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.image.BufferedImage;
import java.util.Vector;

import javax.swing.JPanel;

import sos.framework.simulation.SEnv;
import sos.framework.simulation.prefs.SPrefs;
import sos.framework.simulation.prefs._SPrefsListener;
import sos.framework.simulation.simul._SSpecsListener;
import sos.framework.simulation.simul.results.SResultImage;
import sos.lib.logger.Logger;

public class ImageCanvas extends JPanel implements MouseListener, MouseMotionListener, MouseWheelListener, ComponentListener, _SSpecsListener, _SPrefsListener
{

	private static final long serialVersionUID			= 1L;
	
	public static final Insets IMAGE_INSETS				= new Insets(40, 40, 40, 40);
	public static final int IMAGE_INSETS_GAP_X			= 30;
	public static final int IMAGE_INSETS_GAP_Y			= 30;
	
	public static final double FOCUS_CENTER_X			= 0.50;
	public static final double FOCUS_CENTER_Y			= 0.33;
	
 	public static final Color COLOR_CANVAS_BG			= new Color(103, 103, 103);
 	public static final Color COLOR_IMAGE_BG			= Color.white;
 	public static final Color COLOR_FOCUS				= Color.red;
 	
 	
 	
 	// l'environnement de simulation
	private SEnv env									= null; 
	
	
	
	// l'image qui represente la zone totale
	private BufferedImage bufferedImage	 				= null;

 	// le haut-gauche de l'image totale
 	private Rectangle allImageAfterZoom					= new Rectangle(); 	
 	
 	// position de l'image actuelle
 	private Point actualImagePositionPx					= new Point(); 	
 	
 	// le haut-gauche de l'image actuelle
 	private Point actualImageDeltaPx					= new Point();
 	
 	// position du curseur
 	private Point cursorPositionUnit					= new Point();


 	
 	// memorise si on est entrain de deplacer le focus
 	private boolean blMovingFocus						= false;
 	private Point movingPosition						= new Point();

 	// gestion du zoom
 	private int iZoomNumber								= 0;
 	private double dblZoomFactor						= 1.0;
 	
 	private int maxGenerationSize						= 0;
 	
 	
 	
 	
 	

 	
 	
 	public ImageCanvas(final SEnv env)
 	{
 		super(new BorderLayout());
 		try{
 			this.env = env;
	 		this.setOpaque(false);

     		// evenements a ecouter
     		this.addMouseListener(this);
     		this.addMouseMotionListener(this);
     		this.addMouseWheelListener(this);
     		this.addComponentListener(this);
     		
     		// autres evenements     		
     		this.env.getSimul().addSimulationListener(this);
     		this.env.getPrefs().addPrefsListener(this);

 		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "ImageCanvas()", ex);}
 	}
 	
 	
 	
 	
 	


	// la liste des objets qui veulent etre avertis des evenements
 	private Vector lstListeners = new Vector();
 
 	public final void addImageCanvasListener(final _ImageCanvasListener lst) {lstListeners.add(lst);}
 	public final void removeImageCanvasListener(final _ImageCanvasListener lst) {lstListeners.remove(lst);}
 	
 	private final void fireCanvasMouseMoved(final int cx, final int cy) {for(int i=0; i<lstListeners.size(); i++) try{ ((_ImageCanvasListener)(lstListeners.get(i))).onImageCanvasMouseMoved(cx, cy); }catch(Exception ex) {}}
 	
	

 	
 	
	
	
	
	
 	
 	
 	
 	
 	

 	private final void drawAllImage()
 	{
		try{
			SResultImage res;
			
			// recherche la taille totale de l'image
			int width = 0;
			int height = 0;
			if( env.getPrefs().getOrientation() == SPrefs.DISPLAY_MODE_HORIZONTAL ){
				for(int i=0; i<env.getSimul().getImagesCount(); i++){
					res = env.getSimul().getResults().getImageAt(i);
					width = width + res.getImageWidth();
					if(res.getImageHeight() > height) height = res.getImageHeight();
				}
				width = width + IMAGE_INSETS_GAP_X * ( env.getSimul().getImagesCount() - 1 );
			}else{
				for(int i=0; i<env.getSimul().getImagesCount(); i++){
					res = env.getSimul().getResults().getImageAt(i);
					if(res.getImageWidth() > width) width = res.getImageWidth();
					height = height + res.getImageHeight();
				}
				height = height + IMAGE_INSETS_GAP_Y * ( env.getSimul().getImagesCount() - 1 );
			}
			width = width + IMAGE_INSETS.left + IMAGE_INSETS.right;
			height = height + IMAGE_INSETS.top + IMAGE_INSETS.bottom;
			
			
			// cree une nouvelle image
			if( ( bufferedImage == null ) || (bufferedImage.getWidth() != width) || (bufferedImage.getHeight() != height) ){
				// regarde si c'est une nouvelle simulation
				if( bufferedImage == null ){
					actualImagePositionPx.x = (int)Math.round( (env.getSimul().getResults().getActualImage().getImageWidth()  * FOCUS_CENTER_X) * dblZoomFactor );
		 			actualImagePositionPx.y = (int)Math.round( (env.getSimul().getResults().getActualImage().getImageHeight() * FOCUS_CENTER_Y) * dblZoomFactor );
				}
				// recree une image aux bonnes dimentions
				bufferedImage = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
			}
			Graphics2D g = (Graphics2D)bufferedImage.getGraphics();


			
			// fond de l'image
			g.setColor(COLOR_IMAGE_BG);
			g.fillRect(0, 0, width, height);
			
			
			
			// dessine les images
			SResultImage res2;
			int x = IMAGE_INSETS.left;
			int y = IMAGE_INSETS.top;
			
			if( env.getPrefs().getOrientation() == SPrefs.DISPLAY_MODE_HORIZONTAL ){
				
				for(int i=0; i<env.getSimul().getImagesCount(); i++){
			
					// dessine l'image
					res = env.getSimul().getResults().getImageAt(i);
					g.drawImage(res.getNormalizedImage(), x, y, null);
						
					// dessine les positions
					if( i > 0 ){
			
						res2 = env.getSimul().getResults().getImageAt(i-1);
						double surfaceX = res.getImageWidth() * res.getPixelResolution();
						double surfaceY = res.getImageHeight() * res.getPixelResolution();
						double surfaceX2 = res2.getImageWidth() * res2.getPixelResolution();
						double surfaceY2 = res2.getImageHeight() * res2.getPixelResolution();
						
						if( ( surfaceX < surfaceX2 && surfaceY <= surfaceY2 ) || (surfaceY < surfaceY2 && surfaceX <= surfaceX2 ) ){
							
							double sizeX = surfaceX / res2.getPixelResolution();
							double sizeY = surfaceY / res2.getPixelResolution();

							g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
							g.drawRect( x - IMAGE_INSETS_GAP_X - (int)(res2.getImageWidth()/2.0 + sizeX/2.0),
										IMAGE_INSETS.top + (int)(res2.getImageHeight()/2.0 - sizeY/2.0),
										(int)sizeX,
										(int)sizeY
										);
						
							g.drawLine(	x - IMAGE_INSETS_GAP_X - (int)(res2.getImageWidth()/2.0 - sizeX/2.0), 
										IMAGE_INSETS.top + (int)(res2.getImageHeight()/2.0 - sizeY/2.0),
										IMAGE_INSETS.left + res2.getImageWidth() + IMAGE_INSETS_GAP_X-1,
										IMAGE_INSETS.top
										);
							
							g.drawLine(	x - IMAGE_INSETS_GAP_X - (int)(res2.getImageWidth()/2.0 - sizeX/2.0), 
										IMAGE_INSETS.top + (int)(res2.getImageHeight()/2.0 + sizeY/2.0),
										IMAGE_INSETS.left + res2.getImageWidth() + IMAGE_INSETS_GAP_X-1, 
										IMAGE_INSETS.top  + res2.getImageHeight()-1
										);
							g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
						}
						
					}
				
					x += ( res.getImageWidth() + IMAGE_INSETS_GAP_X );
				}
				
			}else{
				
				for(int i=0; i<env.getSimul().getImagesCount(); i++){
					res = env.getSimul().getResults().getImageAt(i);
					g.fillRect(x, y, res.getImageWidth(), res.getImageHeight());
					y += ( res.getImageHeight() + IMAGE_INSETS_GAP_Y );
				}
				
			}

			// recalcule la position de l'image actuelle dans toute l'image
			updateActualImageDelta();
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "drawAllImage()", ex);}
 	}

 	private final void updateActualImageDelta()
 	{
		try{
			SResultImage res;
			actualImageDeltaPx.x = IMAGE_INSETS.left;
			actualImageDeltaPx.y = IMAGE_INSETS.top;
			if( env.getPrefs().getOrientation() == SPrefs.DISPLAY_MODE_HORIZONTAL ){
				for(int i=0; i<env.getSimul().getActualImageNumber(); i++){
					res = env.getSimul().getResults().getImageAt(i);
					actualImageDeltaPx.x += res.getImageWidth();
				}
				actualImageDeltaPx.x += ( IMAGE_INSETS_GAP_X * env.getSimul().getActualImageNumber() );
			}else{
				for(int i=0; i<env.getSimul().getActualImageNumber(); i++){
					res = env.getSimul().getResults().getImageAt(i);
					actualImageDeltaPx.y += res.getImageHeight();
				}
				actualImageDeltaPx.y += ( IMAGE_INSETS_GAP_Y * env.getSimul().getActualImageNumber() );
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "updateActualImageDelta()", ex);}
 	}
 	
 	

 	
 	
 	
 	
 	

 	private final void actualImageChanged()
 	{
 		try{
 			if( env.getSimul().getActualImageNumber() < 0 ){
 				// fin de la simulation
 				bufferedImage = null;
 			}else{
 				// redessine l'image totale
 				drawAllImage();
 				checkImageBounds();
 				updateZoomAndPositionAndRepaint();
 			}
 		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "actualImageChanged()", ex);}
 	}

 	private final void updateZoomAndPositionAndRepaint()
 	{
 		try{
	 		if( (this.getWidth() > 0) && (this.getWidth() > 0) ){
	 			// calcule la taille de l'image en fonction du zoom
	 			allImageAfterZoom.x = (int)Math.round( (this.getWidth()  * FOCUS_CENTER_X) - ( actualImagePositionPx.x + actualImageDeltaPx.x + 0.5 ) * dblZoomFactor );
	 			allImageAfterZoom.y = (int)Math.round( (this.getHeight() * FOCUS_CENTER_Y) - ( actualImagePositionPx.y + actualImageDeltaPx.y + 0.5 ) * dblZoomFactor );
	 			allImageAfterZoom.width  = allImageAfterZoom.x + (int)( bufferedImage.getWidth()  * dblZoomFactor );
	 			allImageAfterZoom.height = allImageAfterZoom.y + (int)( bufferedImage.getHeight() * dblZoomFactor );
	 			// redessine
	 			this.repaint();
 			}
 		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "updateZoomAndPosition()", ex);}
 	}

    private void updateCursorPosition(final MouseEvent e)
    {
		try{
			// calcule la position du curseur
			cursorPositionUnit.x = (int)Math.round( (e.getX() - (this.getWidth()  * FOCUS_CENTER_X) ) / dblZoomFactor + actualImagePositionPx.x );
			cursorPositionUnit.y = (int)Math.round( (e.getY() - (this.getHeight() * FOCUS_CENTER_Y) ) / dblZoomFactor + actualImagePositionPx.y );
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "updateCursorPosition()", ex);}
    }

    private void checkImageBounds()
    {
		try{
			if(actualImagePositionPx.x > env.getSimul().getResults().getActualImage().getImageWidth()-1) actualImagePositionPx.x = env.getSimul().getResults().getActualImage().getImageWidth()-1;
 			if(actualImagePositionPx.x < 0) actualImagePositionPx.x = 0;
 			if(actualImagePositionPx.y > env.getSimul().getResults().getActualImage().getImageHeight()-1) actualImagePositionPx.y = env.getSimul().getResults().getActualImage().getImageHeight()-1;
 			if(actualImagePositionPx.y < 0) actualImagePositionPx.y = 0;
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "checkImageBounds()", ex);}
    }

 	
 	
 	
 	
 	
 	
 	
	
	
	
	


	public final void setGenerationSize(final int maxGenerationSize)
	{
		try{
			int oldSize = this.maxGenerationSize;
			this.maxGenerationSize = maxGenerationSize;

			if(this.isVisible()){
				if(oldSize == 0) oldSize = maxGenerationSize;
				double sz = oldSize * dblZoomFactor;
				this.repaint((int)Math.round( this.getWidth()*FOCUS_CENTER_X - sz/2.0 )-5, (int)Math.round( this.getHeight()*FOCUS_CENTER_Y - sz/2.0 )-5, (int)Math.round( sz )+15, (int)Math.round( sz )+15 );
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "setGenerationSize()", ex);}
	}

 	
 	
	
	
	
	
 	private final void drawBufferedImage(final Graphics g)
 	{
		try{
			if(bufferedImage != null){
				g.drawImage(
						bufferedImage, 
						allImageAfterZoom.x,
						allImageAfterZoom.y,
						allImageAfterZoom.width,
						allImageAfterZoom.height,
						0,
						0,
						bufferedImage.getWidth(),
						bufferedImage.getHeight(),
						null
					);
				
				g.setColor(Color.black);
				g.drawRect(
						allImageAfterZoom.x,
						allImageAfterZoom.y,
						allImageAfterZoom.width-allImageAfterZoom.x,
						allImageAfterZoom.height-allImageAfterZoom.y
					);
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "drawBufferedImage()", ex);}
 	}

 	private final void drawDynamicGraphics(final Graphics g)
 	{
		try{
			g.setColor(COLOR_FOCUS);
			
	 		// le focus
			g.drawLine( (int)Math.round(this.getWidth()  * FOCUS_CENTER_X ),
						(int)Math.round(this.getHeight() * FOCUS_CENTER_Y - 5 ),
						(int)Math.round(this.getWidth()  * FOCUS_CENTER_X ),
						(int)Math.round(this.getHeight() * FOCUS_CENTER_Y + 5 )
					);
			g.drawLine( (int)Math.round(this.getWidth()  * FOCUS_CENTER_X - 5 ),
						(int)Math.round(this.getHeight() * FOCUS_CENTER_Y ),
						(int)Math.round(this.getWidth()  * FOCUS_CENTER_X + 5 ),
						(int)Math.round(this.getHeight() * FOCUS_CENTER_Y )
					);
			
			// curseur
			/*g.drawRect( (int)Math.round(this.getWidth()  * FOCUS_CENTER_X + (cursorPositionUnit.x - 0.5) * dblZoomFactor ), 
						(int)Math.round(this.getHeight() * FOCUS_CENTER_Y + (cursorPositionUnit.y - 0.5) * dblZoomFactor ), 
						(int)Math.round(dblZoomFactor)-1, 
						(int)Math.round(dblZoomFactor)-1
					);
			*/

			// zone de generation
			if( maxGenerationSize > 0 ){
				g.setColor(COLOR_FOCUS);
				double sz = maxGenerationSize * dblZoomFactor;
				g.drawRect( (int)Math.round( this.getWidth()*FOCUS_CENTER_X - sz/2.0 ), (int)Math.round( this.getHeight()*FOCUS_CENTER_Y - sz/2.0 ), (int)Math.round( sz ), (int)Math.round( sz ) );
			}

		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "drawDynamicGraphics()", ex);}
 	}

	public final void paintComponent(Graphics g)
	{
		try{
			//System.out.println("paintComponent " + g.getClip().getBounds().x + "-" + g.getClipBounds().y + " ("+g.getClipBounds().width+" - "  + g.getClipBounds().height + ")");
	
			
			g.setColor(COLOR_CANVAS_BG);
			g.fillRect(g.getClipBounds().x, g.getClipBounds().y, g.getClipBounds().width, g.getClipBounds().height);

			drawBufferedImage(g);
			drawDynamicGraphics(g);

		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "paintComponent()", ex);}
	}

	
	
	
	
	
    public void mouseMoved(MouseEvent e)
    {
		try{
			// recalcule la position du curseur sur l'image
			updateCursorPosition(e);
			
	 		// regarde si on est entrain de deplacer le point de fuite
	 		if(blMovingFocus == true){
	 			
	 			actualImagePositionPx.x = movingPosition.x - e.getX();
	 			actualImagePositionPx.y = movingPosition.y - e.getY();
	 			checkImageBounds();
	 			
	 			// redessine l'image
	 			updateZoomAndPositionAndRepaint();

	 		}else{
	 			// nottifie que l'on a bouge le curseur
	 			fireCanvasMouseMoved(cursorPositionUnit.x, env.getSimul().getResults().getActualImage().getImageHeight()-1-cursorPositionUnit.y);
	 		}
	 		
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "mouseMoved()", ex);}
    }

    public void mousePressed(MouseEvent e)
    {
    	this.blMovingFocus = true;
    	this.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
    	this.updateCursorPosition(e);
    	movingPosition.x = actualImagePositionPx.x + e.getX();
    	movingPosition.y = actualImagePositionPx.y + e.getY();
    }
    
    public void mouseReleased(MouseEvent e)
    {
    	blMovingFocus = false;
    	this.setCursor(Cursor.getDefaultCursor());
    }
    
    public void mouseDragged(MouseEvent e)
    {
    	mouseMoved(e);
    }

    public void mouseClicked(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
   
    public void componentHidden(ComponentEvent e) {}
    public void componentMoved(ComponentEvent e) {}
    public void componentShown(ComponentEvent e) {}
    
	public void componentResized(ComponentEvent e)
	{
		// on replace l'image dans la fenetre
		updateZoomAndPositionAndRepaint();
	}
   
    
	
	
    public void mouseWheelMoved(MouseWheelEvent e)
    {
    	// on met a jour le zoom
    	iZoomNumber += e.getWheelRotation();
    	if(iZoomNumber < 0) iZoomNumber = 0;
        dblZoomFactor = 1 + iZoomNumber * iZoomNumber;
        
        // memorise
    	env.getPrefs().setZoomFactor(dblZoomFactor);
    	
    	// met a jour
    	updateZoomAndPositionAndRepaint();	
    }
    
    public void onPreferencesChanged()
    {
    	if( dblZoomFactor != env.getPrefs().getZoomFactor() ){
    		// le zoom a change
    		dblZoomFactor = env.getPrefs().getZoomFactor();
    		updateZoomAndPositionAndRepaint();	
    	}
    }
    
	public void onSimulationStateChanged(boolean blIsSimulating)
	{
		if(blIsSimulating == false){
			// a la fin de la simulation, on regenere l'image
			drawAllImage();
			this.repaint();	
		}
	}

	public void onSimulationImageChanged()
	{
		actualImageChanged();
	}

	public void onSimulationStageChanged(int iSimulationState) {}


	
 }
 


