/*
 SOS Project Classes.

 Copyright (C) SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package sos.framework.ui.mutables;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.image.BufferedImage;
import java.util.Vector;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JPanel;

import sos.framework.generator.ImageGenerator;
import sos.framework.simulation.SEnv;
import sos.framework.ui.SOSDialog;
import sos.framework.ui.SOSMutableDialog;
import sos.framework.ui.editor.imagecanvas.ImageCanvas;
import sos.lib.logger.Logger;
import sos.lib.uicommon.SOSComboBox;
import sos.lib.uicommon.UI;

public class ColoratedImagesMutable extends SOSMutableDialog
{

	private static final long serialVersionUID 			= 1L;


	
	
	private ColoratedImagePanel pnlImage				= null;
	
	private SOSComboBox comboRed						= null;
	private SOSComboBox comboGreen						= null;
	private SOSComboBox comboBlue						= null;

	private BufferedImage imageColor					= null;
	
	private Vector lstImages							= new Vector();
	
	
	
	
	
	
	public ColoratedImagesMutable(final SEnv env)
	{
		super(env, "Image colorée", true, SOSDialog.BUTTON_CLOSE, SOSDIALOG_MODE_RIGHT_LATERAL);
		this.buildContent();
	}
	
	
	private final void buildContent()
	{
		try{
			// construit le contenu
			GridBagConstraints c = new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 3, 0, 0), 0, 0);
			JPanel pnlContent = UI.createPanel(new GridBagLayout());
			
			
			pnlContent.add(UI.createLabel("  " + "Rouge", UI.FONT_PLAIN_ITALIC), c);
			c.gridx++;
			pnlContent.add(UI.createLabel("  " + "Vert", UI.FONT_PLAIN_ITALIC), c);
			c.gridx++;
			c.insets = new Insets(0, 3, 0, 3);
			pnlContent.add(UI.createLabel("  " + "Bleu", UI.FONT_PLAIN_ITALIC), c);
	
			c.insets = new Insets(0, 3, 3, 0);
			c.gridx = 0;
			c.gridy++;
			pnlContent.add(comboRed = UI.createComboBox(null, this, -1), c);
			c.gridx++;
			pnlContent.add(comboGreen = UI.createComboBox(null, this, -1), c);
			c.gridx++;
			c.insets = new Insets(0, 3, 3, 3);
			pnlContent.add(comboBlue = UI.createComboBox(null, this, -1), c);
			

			// ajoute le tout
			pnlImage = new ColoratedImagePanel();
			pnlImage.setOpaque(true);
			pnlImage.setBackground(ImageCanvas.COLOR_CANVAS_BG);
			
			JPanel theContent = UI.createPanel(new BorderLayout());
			theContent.add(pnlContent, BorderLayout.NORTH);
			theContent.add(pnlImage, BorderLayout.CENTER);

			super.setContent(theContent);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "buildContent()", ex);}
	}

	
	
	
	

	
	
	
	
	
	
	private final void saveContent()
	{
		try{
			env.getSimul().getSpecs().setColoratedImageRed(comboRed.getSelectedIndex()-1);
			env.getSimul().getSpecs().setColoratedImageGreen(comboGreen.getSelectedIndex()-1);
			env.getSimul().getSpecs().setColoratedImageBlue(comboBlue.getSelectedIndex()-1);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "saveContent()", ex);}
	}
	
	protected final void createImage()
	{
		try{
			ImageGenerator.generateColoratedImage(env.getSimul());
			imageColor = env.getSimul().getResults().getColorImage();
			pnlImage.repaint();
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "createImage()", ex);}
	}
	
	private final void updateCombos()
	{
		try{
		
			// rafraichit la liste des images
			lstImages.clear();
			lstImages.add("Aucune");
			for(int i=0; i<env.getSimul().getImagesCount(); i++){
				lstImages.add(env.getSimul().getSpecs().getImages().get(i));
			}
			
			comboRed.removeActionListener(this);
			comboRed.setModel(new DefaultComboBoxModel(lstImages));
			comboRed.setSelectedIndex(env.getSimul().getSpecs().getColoratedImageRed() + 1);
			comboRed.addActionListener(this);

			comboGreen.removeActionListener(this);
			comboGreen.setModel(new DefaultComboBoxModel(lstImages));
			comboGreen.setSelectedIndex(env.getSimul().getSpecs().getColoratedImageGreen() + 1);
			comboGreen.addActionListener(this);
			
			comboBlue.removeActionListener(this);
			comboBlue.setModel(new DefaultComboBoxModel(lstImages));
			comboBlue.setSelectedIndex(env.getSimul().getSpecs().getColoratedImageBlue() + 1);
			comboBlue.addActionListener(this);
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "updateCombos()", ex);}
	}


	
	
	
	
	
	
	
	protected final void fillSimulation()
	{
		try{
			// nouvelle simulation
			updateCombos();
			createImage();
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "fillSimulation()", ex);}
	}
	
	protected final void fillImage()
	{
		try{
			// met a jour la liste des images au cas ou il y ait une nouvelle image
			if( (env.getSimul().getImagesCount() + 1) != lstImages.size() ){
				updateCombos();
				createImage();
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "fillImage()", ex);}
	}
	
	public void actionPerformed(ActionEvent ae)
	{
		try{
			super.actionPerformed(ae);
			
			if(ae.getSource() instanceof JComboBox){
				// sauve la selection et met a jour l'image
				saveContent();
				createImage();
				return;
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "actionPerformed()", ex);}
	}

	public void onSimulationStateChanged(boolean blIsSimulating)
	{
		comboRed.setEnabled(!blIsSimulating);
		comboGreen.setEnabled(!blIsSimulating);
		comboBlue.setEnabled(!blIsSimulating);
		// met a jour l'image si une simulation est terminee
		if(blIsSimulating == false) createImage();
	}
	


	
	
	
	
	
	
	
	public class ColoratedImagePanel extends JPanel
	{
		
		private static final long serialVersionUID			= 1L;
		
		private final Dimension dim							= new Dimension(2, 200);
		
		public ColoratedImagePanel() {super(null);}	

		public final Dimension getPreferredSize() {return dim;};
		public final Dimension getMaximumSize() {return dim;};
		public final Dimension getMinimumSize() {return dim;};
		
		
		public void paint(final Graphics g)
		{
			super.paint(g);
			
			if( imageColor != null ){
				
				int x=0, y=0, w=0, h=0;
				
				int thisHeight = this.getHeight() - 10;
				int thisWidth = this.getWidth() - 10;
				
				double ratio = imageColor.getWidth() / imageColor.getHeight();
				int height = (int)( thisWidth / ratio );
				
				if( height < thisHeight ){
					w = thisWidth;
					h = height;
					y = (thisHeight-h) / 2;
				}else{
					w = (int)( thisHeight * ratio );
					h = thisHeight;
					x = (thisWidth-w) / 2;
				}
				g.drawImage(imageColor, x+5, y+5, w, h, null);
				
				//g.drawImage(image, g.getClipBounds().x, g.getClipBounds().y, g.getClipBounds().x+g.getClipBounds().width, g.getClipBounds().y+g.getClipBounds().height, g.getClipBounds().x, g.getClipBounds().y, g.getClipBounds().x+g.getClipBounds().width, g.getClipBounds().y+g.getClipBounds().height, null);

			}
		}
	}
	
	
	
	
	
	
	


}
