/*
 SOS Project Classes.

 Copyright (C) SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package sos.framework.ui.mutables;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import sos.framework.generator.DistributionGenerator;
import sos.framework.generator.SOSGenerator;
import sos.framework.io.SOSInputOutput;
import sos.framework.maths.Maths;
import sos.framework.maths.conversions.solidangles.FromArcsecSq;
import sos.framework.ressources.Icons;
import sos.framework.ressources.Lang;
import sos.framework.simulation.SEnv;
import sos.framework.simulation.simul.SSimul;
import sos.framework.simulation.simul.specs.distrib.DistribFunction;
import sos.framework.simulation.simul.specs.sources.SSource;
import sos.framework.ui.SOSDialog;
import sos.framework.ui.SOSMutableDialog;
import sos.framework.ui.dialogs.DistributionDialog;
import sos.lib.logger.Logger;
import sos.lib.uicommon.SOSComboBox;
import sos.lib.uicommon.UI;
import sos.lib.uicommon.graphics.SOSGraphCanvas;
import sos.lib.uicommon.graphics.SPointsFunction;

public class DistributionMutable extends SOSMutableDialog
{

	private static final long serialVersionUID 			= 1L;
	
	private static final int NUMBER_OF_VALUES			= 500;
	
	public static final double DISTRIBUTION_START		= 1e-8;
	public static final double DISTRIBUTION_END			= 1e4;
	public static final double DISTRIBUTION_INC			= 1e-1;



	/* fonction de distribution */
	private SOSComboBox optFuncDist						= null;
	private SOSGraphCanvas cvDistrib					= null;

	/* les boutons */
	private JButton btnUpdate							= null;
	private JButton btnRedist							= null;
	private JButton btnSave								= null;

	private JButton btnEditFunc							= null;
	private JButton btnAddFunc							= null;
	private JButton btnRemoveFunc						= null;
	private JButton btnLoadFromFile						= null;

	
	private JCheckBox chkRedist							= null;
	
	private JLabel lblDesc								= null;
	private JLabel lblNumber							= null;
	
	private SPointsFunction funcDistributed					= null;
	
	
	

	public DistributionMutable(final SEnv env)
	{
		super(env, Lang.STR_TITLE_MANAGE_OBJECTS, true, SOSDialog.BUTTON_CLOSE, SOSDIALOG_MODE_WINDOW);
		this.buildContent();
	}
	

	private final void buildContent()
	{
		try{
			// construit le contenu
			GridBagConstraints c = new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 0, 3), 0, 0);
			JPanel theContent = UI.createPanel(new BorderLayout());
			JPanel pnlGrid = UI.createPanel(new GridBagLayout());
			

			// mode de distribution
			c.gridwidth = 2;
			pnlGrid.add(UI.createLabel("Générateur de source", UI.FONT_BOLD), c);
			

			c.gridy++;
			c.gridwidth = 1;
			btnEditFunc = UI.createButton("Editer", this, UI.SIZE_SMALL_ICON, Icons.ICON_BUTTON_EDIT);
			btnAddFunc = UI.createButton("Ajouter", this, UI.SIZE_SMALL_ICON, Icons.ICON_BUTTON_ADD);
			btnRemoveFunc = UI.createButton("Supprimer", this, UI.SIZE_SMALL_ICON, Icons.ICON_BUTTON_REMOVE);
			pnlGrid.add(UI.createButtonsPanel(new JButton[] {btnAddFunc, btnRemoveFunc, btnEditFunc}), c);
			
			c.gridx++;
			optFuncDist = UI.createComboBox(null, this, -1);
			
			pnlGrid.add(optFuncDist, c);
			c.gridx = 0;
			c.gridwidth = 2;
			
			c.gridy++;
			c.insets.left = 12;
			pnlGrid.add(chkRedist = UI.createCheckBox("Utiliser le générateur automatiquement", null, true, UI.FONT_PLAIN_ITALIC), c);
			c.insets.left = 3;
			
			
			c.gridy++;
			c.insets.bottom = 3;
			btnLoadFromFile = UI.createButton("Importer", this, null);
			btnRedist = UI.createButton("Redistribuer", this, null);
			btnSave = UI.createButton("Enregistrer", this, null);
			btnUpdate = UI.createButton("Appliquer", this, null);
			pnlGrid.add(UI.createButtonsPanel(new JButton[] {btnLoadFromFile, btnRedist, btnSave, btnUpdate}), c);
		
			

			// distribution actuelle
			JPanel pnlSrc = UI.createPanel(new BorderLayout());
			pnlSrc.add(UI.createLabel("Sources distribuées", UI.FONT_BOLD), BorderLayout.WEST);
			//pnlSrc.add(btnLoadFromFile = UI.createButton("Charger depuis un fichier", this, UI.SIZE_SMALL_ICON, Icons.ICON_BUTTON_EDIT), BorderLayout.EAST);
			
			c.gridy++;			
			pnlGrid.add(pnlSrc, c);
	
	
			
			
			c.gridy++;
			pnlGrid.add(lblDesc = UI.createLabel("-", UI.FONT_PLAIN_ITALIC), c);
			c.gridy++;
			pnlGrid.add(lblNumber = UI.createLabel("-", UI.FONT_PLAIN_ITALIC), c);
			c.gridy++;
			
			
			
			
			theContent.add(pnlGrid, BorderLayout.NORTH);


			
			
			cvDistrib = new SOSGraphCanvas(1e-8, 1e4, 1, 0.2);
			cvDistrib.setYPolicy(1e-3, 1e10, 1, 0.2);
			cvDistrib.setXScale(SOSGraphCanvas.SCALE_LOG);
			cvDistrib.setXLabel("Flux {Jy}");
			cvDistrib.setYScale(SOSGraphCanvas.SCALE_LOG);
			cvDistrib.setYLabel("Flux {Nb/Sr}");
			
			Dimension d = new Dimension(360, 180);
			cvDistrib.setPreferredSize(d);
			cvDistrib.setMinimumSize(d);
			

			c.gridy = 0;
			c.insets.left = 3;
			pnlGrid = UI.createPanel(new GridBagLayout());
			pnlGrid.add(cvDistrib, c);
			theContent.add(pnlGrid, BorderLayout.CENTER);
			
			/*
			pnlGrid = UI.createPanel(new GridBagLayout());
			c.insets.bottom = 3;
			btnSave = UI.createButton("Enregistrer", this, null);
			btnUpdate = UI.createButton("Redistribuer", this, null);
			pnlGrid.add(UI.createButtonsPanel(new JButton[] {btnSave, btnUpdate}), c);
			theContent.add(pnlGrid, BorderLayout.SOUTH);
			
			*/
			
			funcDistributed = new SPointsFunction("", "", SPointsFunction.LOG_POINTS_FUNCTION + "[0;NaN]");

			super.setContent(theContent);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "buildContent()", ex);}
	}


	
	
	
	
	

	protected final void fillSimulation()
	{
		try{
			
			optFuncDist.setModel(new DefaultComboBoxModel(env.getSimul().getSpecs().getDistributionFunctions()));
			
			// fonction du generateur
			optFuncDist.setSelectedItem(env.getSimul().getSpecs().getDistributionFunction());
	
			// utilisation du generateur
			chkRedist.setSelected(env.getSimul().getSpecs().isUseGenerator());
			
			// infos
			updateDistributionInfos();
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "fillSimulation()", ex);}
	}
	
	protected final void fillImage()
	{
		try{
			// met a jour le graph
			updateDistributedGraph();
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "fillSimulation()", ex);}
	}
	
	
	
	private final void updateDistributionInfos()
	{
		try{
	
			// met a jour les labels
			lblNumber.setText("  " + "Nombre : " + env.getSimul().getSpecs().getSourcesContainer().getSourcesCount() + " sources");
			lblDesc.setText("  " + "Mode : " + env.getSimul().getSpecs().getSourcesContainer().getSourcesOrigin());

			// met a jour le graph
			updateDistributedGraph();
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "fillContent()", ex);}
	}

	
	
	
	
	private final void updateDistributedGraph()
	{
		try{
			if(imageSpec == null) return;
			
			double[] valuesY = new double[NUMBER_OF_VALUES];
			double min = Maths.log( DISTRIBUTION_START );
			double max = Maths.log( DISTRIBUTION_END );
			double res = (max-min) / (NUMBER_OF_VALUES-1);
			
			// calcule la surface de l'image actuelle
			double dblX = imageSpec.getPixelResolution() * imageSpec.getImageWidth();
			double dblY = imageSpec.getPixelResolution() * imageSpec.getImageHeight();
			double dblSurface = FromArcsecSq.toSterad( dblX * dblY );
			dblX = dblX / 2.0;
			dblY = dblY / 2.0;
			 
			// boucle sur les sources de cette image
			SSource src;
			for(int i=0; i<env.getSimul().getSpecs().getSourcesContainer().getSourcesCount(); i++){
				src = env.getSimul().getSpecs().getSourcesContainer().getSourceAt(i);
				
				// regarde si la source est dans cette image
				if( (src.getPosX() > -dblX) && (src.getPosX() < dblX) && (src.getPosY() > -dblY) && (src.getPosY() < dblY) ){
					double flux = src.getTotalFlux();
					double logflux = Maths.log( flux );
					valuesY[ (int)Math.round( (logflux-min) / res ) ]++;
				}

			}
			
			// annule les donnees vides et convertie en nb/sterad
			for(int i=0; i<valuesY.length; i++){
				
				valuesY[i] = valuesY[i] / dblSurface;
				if(valuesY[i] < 0.0001) valuesY[i]=0.0001;// valuesY[i] = Double.NaN;
			}
	
			// cre la fonction
			String points = "";
			for(int i=0; i<valuesY.length; i++){
				points += "[" +  Maths.pow( 10.0 , (i*res+min) ) + ";" + valuesY[i] + "]";
			}
	
			// affiche la fonction
			funcDistributed.setPoints(SPointsFunction.LOG_POINTS_FUNCTION + points);
			cvDistrib.updateGraphic();

		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "fillContent()", ex);}
	}

	
	
	
	
	
	

	private final void toggleDistributionMode()
	{
		try{
			// fonction
			cvDistrib.clearFunctions();
			DistribFunction f = getSelectedFunction();
			cvDistrib.addFunction(f.generateFunction());
			cvDistrib.addFunction(funcDistributed);
			
			if(optFuncDist.getSelectedIndex() == 0){
				btnRemoveFunc.setEnabled(false);
				btnEditFunc.setEnabled(false);
			}else{
				btnRemoveFunc.setEnabled(true);
				btnEditFunc.setEnabled(true);
			}
			
			// met a jour le graphic
			cvDistrib.updateGraphic();
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "toggleDistributionMode()", ex);}
	}


	private final void loadSourcesFromFile()
	{
		try{
			
			// utiliser un fichier
			if( this.getActualWindowMode() == SOSMutableDialog.SOSDIALOG_MODE_WINDOW ){
				if( SOSInputOutput.importSRC(this.getWindowModeDialog(), env.getSimul()) == false ) return;
			}else{
				if( SOSInputOutput.importSRC(this.getPanelModeFrame(), env.getSimul()) == false ) return;	
			}
				
			
			
			
			
			// on regarde si l'origine correspond a une fonction
			/*optFuncDist.setSelectedIndex(0);
			String origin = env.getSimSpecs().getSourcesContainer().getSourcesOrigin();
			if(origin.length() > 0){
				for(int i=0; i<env.getSimSpecs().getDistributionFunctions().size(); i++){
					if(origin.equals( ((SFunction)(env.getSimSpecs().getDistributionFunctions().get(i))).getName() )){
						env.getSimSpecs().setDistributionFunction( (SFunction)(env.getSimSpecs().getDistributionFunctions().get(i)) );
					}
				}
			}
			
			// on ecrase l'origine par le nom du fichier
			env.getSimSpecs().getSourcesContainer().setSourcesOrigin(new File( env.getSimPrefs().getExportFile() ).getName());
		*/
			// on averti les fenetre qu'il faut regenerer les sources
			notifySourcesChanged();

		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "loadSourcesFromFile()", ex);}
	}

	
	private final void showFunctionEditor(final DistribFunction function)
	{
		try{
			// s'assure de bien avoir un bruit
			DistribFunction localFunction = null;
			if(function == null) localFunction = new DistribFunction("Nouvelle fonction", "# Aucun objet distribué\n");
			else localFunction = function;
			
			// affiche la fenetre
			DistributionDialog fm;
			if( this.getActualWindowMode() == SOSDIALOG_MODE_WINDOW ) fm = new DistributionDialog(this.getWindowModeDialog(), localFunction);
			else fm = new DistributionDialog(this.getPanelModeFrame(), localFunction);	
			
			fm.setVisible(true);
			
			if(fm.retournedValue() == SOSDialog.BUTTON_OK){
				
				// on ajoute le filtre si c'est un nouveau
				if(function == null) env.getSimul().getSpecs().getDistributionFunctions().add(localFunction);
				optFuncDist.updateUI();
				optFuncDist.setSelectedIndex(optFuncDist.getItemCount()-1);

			}
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "showFunctionEditor()", ex);}
	}
	
	private final void redistributeSources()
	{
		try{
			
			// enregistre les pref
			saveSimulation();
			
			// efface lehs sources
			env.getSimul().getSpecs().getSourcesContainer().clearSources();
			
			// TODO redistribue la fonctions
			DistributionGenerator.generateDistribution(env.getSimul());
			
			// annonce que l'on doit redistribuer
			notifySourcesChanged();
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "redistributeSources()", ex);}
	}
	
	private final void saveSimulation()
	{
		try{
			
			env.getSimul().getSpecs().setDistributionFunction( (DistribFunction)(optFuncDist.getSelectedItem()) );
			env.getSimul().getSpecs().setUseGenerator( chkRedist.isSelected() );
			
			if(chkRedist.isSelected()){
				
				for(int i=0; i<env.getSimul().getImagesCount(); i++){
					if( env.getSimul().needGenerateStage(i, SSimul.STAGE_DISTRIBUTION) ){
						env.getSimul().fireImageSpecsChanged(i, SSimul.STAGE_DISTRIBUTION);
					}
				}
				
			}
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "saveSimulation()", ex);}
	}
	
	private final void updateSimulation()
	{
		try{
			this.saveSimulation();
			SOSGenerator.generate(env);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "updateSimulation()", ex);}
	}
	

	
	
	
	private final void notifySourcesChanged()
	{
		try{
			// on averti les fenetre qu'il faut regenerer les sources
			env.getSimul().fireResultsComputed(SSimul.STAGE_DISTRIBUTION);
			for(int i=0; i<env.getSimul().getImagesCount(); i++){
				env.getSimul().fireImageSpecsChanged(i, SSimul.STAGE_GENERATION);
			}
			
			// on met a jour les infos de la fenetre
			updateDistributionInfos();

		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "notifySourcesChanged()", ex);}
	}
	
	
	
	
	
	
	
	
	private final DistribFunction getSelectedFunction()
	{
		try{
			if ((optFuncDist.getSelectedItem() != null) && (optFuncDist.getSelectedItem() instanceof DistribFunction) ) return (DistribFunction)(optFuncDist.getSelectedItem());
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "getSelectedFunction()", ex);}
		return null;
	}
	
	private final void removeSelectedFunction()
	{
		try{
			DistribFunction f = getSelectedFunction();
			int mem = optFuncDist.getSelectedIndex();
			if( (mem > 0) && (f != null) ){
				int ret = JOptionPane.showConfirmDialog(env.getWindow(), "Etes-vous sur de vouloir supprimer définitivement cette fonction ?", "Confirmation ?", JOptionPane.OK_CANCEL_OPTION, JOptionPane.INFORMATION_MESSAGE); 
				if( ret == JOptionPane.OK_OPTION ){
					env.getSimul().getSpecs().getDistributionFunctions().remove(f);
					optFuncDist.setSelectedIndex(mem-1);
				}
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "removeSelectedFilter()", ex);}
	}

	
	
	
	
	
	
	
	

	
	
	
	
	

	
	public final void actionPerformed(ActionEvent ae)
	{
		super.actionPerformed(ae);
		try{
			if(ae.getSource() == optFuncDist) {toggleDistributionMode(); return;}
			
			if(ae.getSource() == btnRedist) {redistributeSources(); return;}
			if(ae.getSource() == btnSave) {saveSimulation(); return;}
			if(ae.getSource() == btnUpdate) {updateSimulation(); return;}
			
			if(ae.getSource() == btnRemoveFunc) {removeSelectedFunction(); return;}
			
			if(ae.getSource() == btnEditFunc) {showFunctionEditor(getSelectedFunction()); return;}
			if(ae.getSource() == btnAddFunc) {showFunctionEditor(null); return;}
			
			if(ae.getSource() == btnLoadFromFile) {loadSourcesFromFile(); return;}
			
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "actionPerformed()", ex);}
	}
	
	
	
	public void onSimulationStateChanged(boolean blIsSimulating)
	{
		btnRedist.setEnabled(!blIsSimulating);
		btnSave.setEnabled(!blIsSimulating);
		btnUpdate.setEnabled(!blIsSimulating);
		if(blIsSimulating == false) updateDistributionInfos();
	}

	
	

}
