/*
 SOS Project Classes.

 Copyright (C) SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package sos.framework.ui.mutables;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import sos.framework.generator.SOSGenerator;
import sos.framework.ressources.Lang;
import sos.framework.simulation.SEnv;
import sos.framework.ui.SOSDialog;
import sos.framework.ui.SOSMutableDialog;
import sos.lib.logger.Logger;
import sos.lib.uicommon.UI;

public class GenerationMutable extends SOSMutableDialog implements MouseListener, ChangeListener
{

	private static final long serialVersionUID 			= 1L;


	/* les elements graphiques */
	private JButton btnUpdate							= null;
	private JButton btnSave								= null;
	private JSlider sliderSize							= null;
	private JLabel lblValue								= null;
	
	

	
	
	
	
	public GenerationMutable(final SEnv sEnv)
	{
		super(sEnv, Lang.STR_TITLE_GENERATE_OBJECTS, false, SOSDialog.BUTTON_CLOSE, SOSDIALOG_MODE_LEFT_LATERAL);
		this.buildContent();
	}

	private final void buildContent()
	{
		try{
			// construit le contenu
			GridBagConstraints c = new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 3, 0, 3), 0, 0);
			JPanel theContent = UI.createPanel(new GridBagLayout());

			// rayon de calcul
			theContent.add(UI.createLabel("Rayon de calcul", UI.FONT_BOLD), c);
			c.gridx = 1;
			theContent.add(lblValue = UI.createLabel("-", UI.FONT_PLAIN_SMALL), c);
			lblValue.setHorizontalAlignment(SwingConstants.RIGHT);
			c.gridy++;
			c.gridx = 0;
			c.gridwidth = 2;
			theContent.add(sliderSize = UI.createSlider(JSlider.HORIZONTAL, 10, 200, 10, this), c);
			sliderSize.addMouseListener(this);
			sliderSize.setPreferredSize(new Dimension(320, sliderSize.getPreferredSize().height));

			// les boutons
			c.gridy++;
			c.gridx = 0;
			c.gridwidth = 2;
			c.insets = new Insets(3, 3, 3, 3);
			btnSave = UI.createButton("Enregistrer", this, null);
			btnUpdate = UI.createButton("Appliquer", this, null);
			theContent.add(UI.createButtonsPanel(new JButton[] {btnSave, btnUpdate}), c);
			
			super.setContent(theContent);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "buildContent()", ex);}
	}

	
	
	
	
	protected final void fillImage()
	{
		try{
			sliderSize.setValue(imageSpec.getGenerationObjectsSize());
			lblValue.setText(""+imageSpec.getGenerationObjectsSize());
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "fillContent()", ex);}
	}


	
	
	
	
	private final void saveGeneration()
	{
		try{
			try{
				int opt = sliderSize.getValue();
				if(opt < 10) opt = 10;
				imageSpec.setGenerationObjectsSize(opt);
			}catch(Exception ex) {}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "startGeneration()", ex);}
	}

	private final void updateGeneration()
	{
		try{
			saveGeneration();
			SOSGenerator.generate(env);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "updateGeneration()", ex);}
	}


	


	
	public final void actionPerformed(ActionEvent ae)
	{
		super.actionPerformed(ae);
		try{
			if(ae.getSource() == btnUpdate) {updateGeneration(); return;}
			if(ae.getSource() == btnSave) {saveGeneration(); return;}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "actionPerformed()", ex);}
	}

    public void stateChanged(ChangeEvent e)
    {
    	try{
    		lblValue.setText(""+sliderSize.getValue());
	        if(sliderSize.getValueIsAdjusting()) env.getWindow().getEditor().getImageCanvas().setGenerationSize(sliderSize.getValue());
	        else env.getWindow().getEditor().getImageCanvas().setGenerationSize(0);
    	}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "stateChanged()", ex);}
    }

    
    
    
	public void onSimulationStateChanged(boolean blIsSimulating)
	{
		try{
			btnUpdate.setEnabled(!blIsSimulating);
			btnSave.setEnabled(!blIsSimulating);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "onSimulationStateChanged()", ex);}
	}

	
	
    public void mousePressed(MouseEvent e) {env.getWindow().getEditor().getImageCanvas().setGenerationSize(sliderSize.getValue());}
    public void mouseReleased(MouseEvent e) {env.getWindow().getEditor().getImageCanvas().setGenerationSize(0);}
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
    public void mouseClicked(MouseEvent e) {}
    
}
