/*
 SOS Project Classes.

 Copyright (C) SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package sos.framework.ui.mutables;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Vector;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;

import sos.framework.generator.SOSGenerator;
import sos.framework.ressources.Icons;
import sos.framework.ressources.Lang;
import sos.framework.simulation.SEnv;
import sos.framework.simulation.simul.specs.noises.SNoise;
import sos.framework.ui.SOSDialog;
import sos.framework.ui.SOSMutableDialog;
import sos.framework.ui.dialogs.NoiseDialog;
import sos.lib.logger.Logger;
import sos.lib.uicommon.UI;

public class NoisesMutable extends SOSMutableDialog implements MouseListener
{
	
	private static final long serialVersionUID 			= 1L;
	
	private JButton btnEditNoise						= null;
	private JButton btnAddNoise							= null;
	private JButton btnRemoveNoise						= null;
	private JButton btnMoveUpNoise						= null;
	private JButton btnMoveDownNoise					= null;
	private JButton btnUpdate							= null;

	private JCheckBox chkActive							= null;
	
	private Vector lstNoisesModel						= null;
	private JList lstNoises 							= null;
	

	
	
	
	
	
	
	public NoisesMutable(final SEnv env)
	{
		super(env, Lang.STR_TITLE_NOISES, true, SOSDialog.BUTTON_CLOSE, SOSDIALOG_MODE_LEFT_LATERAL);
		this.buildContent();
	}
	
	private final void buildContent()
	{
		
		try{
			JPanel theContent = UI.createPanel(new BorderLayout());
			// construit le contenu
			GridBagConstraints c = new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 3, 3), 0, 0);
			JPanel pnlGridBag;

			// titre
			pnlGridBag = UI.createPanel(new GridBagLayout());
			pnlGridBag.add(UI.createLabel("Bruits", UI.FONT_BOLD), c);
			c.gridx++;
			pnlGridBag.add(UI.createLabel("(évalués de haut en bas)", UI.FONT_PLAIN_ITALIC), c);
			theContent.add(pnlGridBag, BorderLayout.NORTH);
			c.gridx = 0;

			// liste
			lstNoisesModel = new Vector();
			Dimension d = new Dimension(326, 100);
			lstNoises = new JList();
			lstNoises.setModel(new DefaultComboBoxModel(lstNoisesModel));
			lstNoises.setPreferredSize(d);
			lstNoises.setMinimumSize(d);
			lstNoises.addMouseListener(this);
			lstNoises.setCellRenderer(new NoiseListCellRenderer());
			c.gridy = 0;
			c.insets = new Insets(0, 3, 0, 3);
			pnlGridBag = UI.createPanel(new GridBagLayout());
			pnlGridBag.add(lstNoises, c);
			theContent.add(pnlGridBag, BorderLayout.CENTER);
			
			// boutons
			pnlGridBag = UI.createPanel(new GridBagLayout());
			c.gridy = 0;
			c.gridx = 0;
			c.insets = new Insets(3, 3, 3, 3);
			chkActive = UI.createCheckBox("Bruit actif" + "  ", this, false, UI.FONT_PLAIN);
			btnAddNoise = UI.createButton("Ajouter", this, UI.SIZE_SMALL_ICON, Icons.ICON_BUTTON_ADD);
			btnRemoveNoise = UI.createButton("Supprimer", this, UI.SIZE_SMALL_ICON, Icons.ICON_BUTTON_REMOVE);
			btnEditNoise = UI.createButton("Editer", this, UI.SIZE_SMALL_ICON, Icons.ICON_BUTTON_EDIT);
			btnMoveUpNoise = UI.createButton("Monter", this, UI.SIZE_SMALL_ICON, Icons.ICON_BUTTON_NOISE_UP);
			btnMoveDownNoise = UI.createButton("Descendre", this, UI.SIZE_SMALL_ICON, Icons.ICON_BUTTON_NOISE_DOWN);
			btnUpdate = UI.createButton("Appliquer", this, null);
			pnlGridBag.add(UI.createButtonsPanel(new JComponent[] {chkActive, btnMoveUpNoise, btnMoveDownNoise, btnAddNoise, btnRemoveNoise, btnEditNoise, btnUpdate}), c);
			
			theContent.add(pnlGridBag, BorderLayout.SOUTH);
			
			//mnuNewNoise = UI.createMenuItem("Nouveau bruit...", this, null);
			//updatePopupMenu();
			
			super.setContent(theContent);	
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "buildContent()", ex);}
	}

	
	
	
	
	
	/*
	private final void updatePopupMenu()
	{
		try{
			popup = new JPopupMenu();
			popup.add(UI.createLabel(" Ajouter un bruit...", UI.FONT_BOLD));
			for(int i=0; i<env.getSimSpecs().getNoises().size(); i++){
				SNoise noise = (SNoise)(env.getSimSpecs().getNoises().get(i));
				popup.add(UI.createMenuItem(noise.getDescription(), this, null, ""));
			}
			popup.addSeparator();
			popup.add(mnuNewNoise);
		}catch(Exception ex) {SLogs.fireError(this.getClass().getName(), "updatePopupMenu()", ex);}
	}
	*/


	
	
	

	
	protected final void fillImage()
	{
		try{

			updateNoisesListModel();
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "fillContent()", ex);}
	}
	
	
	
	
	
	
	private final void updateNoisesListModel()
	{
		try{
			// memorise le bruit actuel
			SNoise selNoise = getSelectedNoise();
			
			// vide la liste
			lstNoisesModel.clear();
			SNoise noise;
			
			// ajoute les bruits de notre image
			for(int i=0; i<imageSpec.getNoises().size(); i++){
				lstNoisesModel.add(imageSpec.getNoises().get(i));
			}
			// ajoute les bruits pas de notre image
			for(int i=0; i<env.getSimul().getSpecs().getNoises().size(); i++){
				noise = (SNoise)(env.getSimul().getSpecs().getNoises().get(i));
				if(imageSpec.getNoises().contains(noise) == false) lstNoisesModel.add(noise);
			}
			
			// reselectionne l'ancien bruit
			lstNoises.updateUI();
			if( (selNoise != null) && (lstNoisesModel.contains(selNoise)) ) lstNoises.setSelectedValue(selNoise, true);
			else if( lstNoisesModel.size() > 0 ) lstNoises.setSelectedIndex(0);
			updateButtons();

		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "updateNoisesListModel()", ex);}
	}

	private final void updateButtons()
	{
		try{
			SNoise noise = getSelectedNoise();
			if(noise == null){
				btnEditNoise.setEnabled(false);
				btnRemoveNoise.setEnabled(false);
				btnMoveUpNoise.setEnabled(false);
				btnMoveDownNoise.setEnabled(false);
				chkActive.setSelected(false);
				chkActive.setEnabled(false);
			}else{
				btnEditNoise.setEnabled(true);
				btnRemoveNoise.setEnabled(!noise.isLocked());
				btnMoveUpNoise.setEnabled(imageSpec.getNoises().contains(noise));
				btnMoveDownNoise.setEnabled(imageSpec.getNoises().contains(noise));
				chkActive.setEnabled(true);
				chkActive.setSelected(imageSpec.getNoises().contains(noise));
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "updateButtons()", ex);}
	}
	
	private final SNoise getSelectedNoise()
	{
		try{
			if(lstNoises.getSelectedValue() != null) return (SNoise)(lstNoises.getSelectedValue());
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "getSelectedNoise()", ex);}
		return null;
	}

	private final void showNoiseEditor(final SNoise noise)
	{
		try{
			// s'assure de bien avoir un bruit
			SNoise localNoise = null;
			if(noise == null) localNoise = new SNoise("Nouvelle description", "\n//\n// Ajouter la fonction ici\n//\n\nreturn 0\n", true);
			else localNoise = (SNoise)noise;
			
			// affiche la fenetre
			NoiseDialog fm;
			if( this.getActualWindowMode() == SOSDIALOG_MODE_WINDOW ) fm = new NoiseDialog(this.getWindowModeDialog(),  localNoise, imageSpec);
			else fm = new NoiseDialog(this.getPanelModeFrame(),  localNoise, imageSpec);	
			
			fm.setVisible(true);
			
			if(fm.retournedValue() == SOSDialog.BUTTON_OK){
				// on ajoute le bruit si c'est un nouveau
				if(noise == null) env.getSimul().getSpecs().getNoises().add(localNoise);
				updateNoisesListModel();
			}
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "showNoiseManager()", ex);}
	}
	
	
	
	
	


	

	
	
	
	
	
	

	
	private final void removeNoise()
	{
		try{
			SNoise noise = getSelectedNoise();
			if( (noise != null) && (noise.isLocked() == false) ){
				int ret = JOptionPane.showConfirmDialog(env.getWindow(), "Etes-vous sur de vouloir supprimer définitivement ce bruit ?", "Confirmation ?", JOptionPane.OK_CANCEL_OPTION, JOptionPane.INFORMATION_MESSAGE); 
				if( ret == JOptionPane.OK_OPTION ){
					env.getSimul().getSpecs().getNoises().remove(noise);
					updateNoisesListModel();
				}
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "removeNoise()", ex);}
	}
	
	private final void moveUpNoise()
	{
		try{
			int i = lstNoises.getSelectedIndex();
			if( i > 0 ){
				if( imageSpec.getNoises().contains(getSelectedNoise()) ){
					SNoise noise = imageSpec.getNoises().remove(i);
					imageSpec.getNoises().add(i-1, noise);
					updateNoisesListModel();
				}
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "moveUpNoise()", ex);}
	}
	
	private final void moveDownNoise()
	{
		try{
			int i = lstNoises.getSelectedIndex();
			if( (i > -1) && (i < imageSpec.getNoises().size()-1) ){
				if( imageSpec.getNoises().contains(getSelectedNoise()) ){
					SNoise noise = imageSpec.getNoises().remove(i);
					imageSpec.getNoises().add(i+1, noise);
					updateNoisesListModel();
				}
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "moveDownNoise()", ex);}
	}
	
	private final void toggleNoise()
	{
		try{
			SNoise noise = getSelectedNoise();
			if(noise != null){
				if( (chkActive.isSelected() == true) && (imageSpec.getNoises().contains(noise) == false) ){
					imageSpec.getNoises().add(noise);
					updateNoisesListModel();
					return;
				}
				if( (chkActive.isSelected() == false) && (imageSpec.getNoises().contains(noise) == true) ){
					imageSpec.getNoises().remove(noise);
					updateNoisesListModel();
					return;
				}
			}else updateButtons();
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "toggleNoise()", ex);}
	}
	




	

	
	
	
	

	public final void actionPerformed(ActionEvent ae)
	{
		super.actionPerformed(ae);
		try{
			
			if(ae.getSource() == btnMoveUpNoise) {moveUpNoise(); return;}
			if(ae.getSource() == btnMoveDownNoise) {moveDownNoise(); return;}
			if(ae.getSource() == btnRemoveNoise) {removeNoise(); return;}
			if(ae.getSource() == btnUpdate) {SOSGenerator.generate(env); return;}
			if(ae.getSource() == btnEditNoise) {showNoiseEditor(getSelectedNoise()); return;}
			if(ae.getSource() == chkActive) {toggleNoise(); return;}
			
			//if(ae.getSource() == mnuNewNoise) {showNoiseEditor(null); return;}
			
			if(ae.getSource() == btnAddNoise) {
				showNoiseEditor(null); 
				//popup.show(btnAddNoise, 0, 0);
				return;
				}
			
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "actionPerformed()", ex);}
	}
	
	
	

	
	
	public void onSimulationStateChanged(boolean blIsSimulating)
	{
		btnUpdate.setEnabled(!blIsSimulating);
	}
	
	
	

	
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}
	
	public void mouseClicked(MouseEvent e)
	{
		try{
			if(e.getClickCount() == 1) {updateButtons(); return;}
			if(e.getClickCount() == 2) {showNoiseEditor(getSelectedNoise());}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "mouseClicked()", ex);}
	}


	
	
	
	class NoiseListCellRenderer extends JLabel implements ListCellRenderer
	{
		
		private static final long serialVersionUID			= 1L;
	
		public NoiseListCellRenderer()
		{
			this.setOpaque(true);
		}
		
		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
		{
			if(value instanceof SNoise){
				SNoise noise = (SNoise)value;
				this.setFont((noise.getApplyForEachPixel()?UI.FONT_PLAIN:UI.FONT_BOLD));
				if( imageSpec.getNoises().contains(noise) ){
					this.setForeground(Color.black);
					this.setText(" " + (imageSpec.getNoises().indexOf(noise)+1)+". "+value.toString());
				}else{
					this.setForeground(Color.darkGray);
					this.setText(" " + value.toString() + "");
				}
				this.setBackground(cellHasFocus ? Color.lightGray : Color.white);
			}else{
				this.setText("?");
			}
			return this;
		}

	}	
	
	
}
