/*
 SOS Project Classes.

 Copyright (C) SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package sos.framework.ui.mutables;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import sos.framework.generator.SOSGenerator;
import sos.framework.maths.Maths;
import sos.framework.ressources.Lang;
import sos.framework.simulation.SEnv;
import sos.framework.simulation.simul.results.SResultImage;
import sos.framework.ui.SOSDialog;
import sos.framework.ui.SOSMutableDialog;
import sos.lib.logger.Logger;
import sos.lib.uicommon.UI;
import sos.lib.uicommon.graphics.SPointsFunction;
import sos.lib.uicommon.graphics.SOSGraphCanvas;

public class NormalizationMutable extends SOSMutableDialog implements ChangeListener
{

	private static final long serialVersionUID 			= 1L;
	
	// precision du graphe
	private static final int NUMBER_OF_VALUES			= 300;


	
	// graphes
	private SOSGraphCanvas cvDistrib					= null;

	// sliders des limites
	private JSlider sliderNormBlack						= null;
	private JSlider sliderNormWhite						= null;

	// labels
	private JLabel lblNormWhite							= null;
	private JLabel lblNormBlack							= null;
	
	// normalisation independante
	private JCheckBox chkIndepNorm						= null;

	
	
	
	double res = 0;
	
	
	

	
	
	public NormalizationMutable(final SEnv env)
	{
		super(env, "Création de l'image", true, SOSDialog.BUTTON_CLOSE, SOSDIALOG_MODE_WINDOW);
		this.buildContent();
	}
	

	private final void buildContent()
	{
		try{
			// construit le contenu
			JPanel theContent = UI.createPanel(new BorderLayout());
			GridBagConstraints c = new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 3, 0, 3), 0, 0);
			JPanel pnlGridBag;

			// graphe
			cvDistrib = new SOSGraphCanvas(0, 100, 10, 1);
			cvDistrib.setXScale(SOSGraphCanvas.SCALE_LINEAR);
			cvDistrib.setXLabel("Flux {elect/pix}");
			cvDistrib.setYScale(SOSGraphCanvas.SCALE_LINEAR);
			cvDistrib.setYLabel("Nombre de pixel");
			Dimension d = new Dimension(400, 200);
			cvDistrib.setPreferredSize(d);
			cvDistrib.setMinimumSize(d);
			cvDistrib.setHistogram(true);
			c.fill = GridBagConstraints.BOTH;
			pnlGridBag = UI.createPanel(new GridBagLayout());
			pnlGridBag.add(cvDistrib, c);
			theContent.add(pnlGridBag, BorderLayout.CENTER);
		
			pnlGridBag = UI.createPanel(new GridBagLayout());
			c.fill = GridBagConstraints.HORIZONTAL;

			// valeur du noir
			pnlGridBag.add(UI.createLabel(Lang.STR_OPT_IMAGE_NORM_BLACK, UI.FONT_BOLD), c);
			c.gridx++;
			pnlGridBag.add(lblNormBlack = UI.createLabel("-", UI.FONT_PLAIN), c);
			lblNormBlack.setHorizontalAlignment(SwingConstants.RIGHT);
			c.gridy++;
			c.gridx = 0;
			c.gridwidth = 2;
			sliderNormBlack = UI.createSlider(JSlider.HORIZONTAL, 0, 999, 0, null);
			sliderNormBlack.setEnabled(false);
			sliderNormBlack.setPreferredSize(new Dimension(320, sliderNormBlack.getPreferredSize().height));
			pnlGridBag.add(sliderNormBlack, c);
			c.gridwidth = 1;

			// valeur du blanc
			c.gridy++;
			pnlGridBag.add(UI.createLabel(Lang.STR_OPT_IMAGE_NORM_WHITE, UI.FONT_BOLD), c);
			c.gridx++;
			pnlGridBag.add(lblNormWhite = UI.createLabel("-", UI.FONT_PLAIN), c);
			lblNormWhite.setHorizontalAlignment(SwingConstants.RIGHT);
			c.gridy++;
			c.gridx = 0;
			c.gridwidth = 2;
			sliderNormWhite = UI.createSlider(JSlider.HORIZONTAL, 1, 1000, 1000, null);
			sliderNormWhite.setEnabled(false);
			sliderNormWhite.setPreferredSize(new Dimension(320, sliderNormWhite.getPreferredSize().height));
			pnlGridBag.add(sliderNormWhite, c);
			c.gridwidth = 1;
			
			// type de normalisation
			c.gridy++;
			c.insets = new Insets(3, 3, 3, 3);
			chkIndepNorm = UI.createCheckBox("Normaliser les images indépendement", this, false, UI.FONT_PLAIN);
			pnlGridBag.add(chkIndepNorm, c);
			theContent.add(pnlGridBag, BorderLayout.SOUTH);
			
			super.setContent(theContent);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "buildContent()", ex);}
	}

	
	
	
	
	
	protected final void fillImage()
	{
		try{
			
			sliderNormBlack.removeChangeListener(this);
			sliderNormWhite.removeChangeListener(this);

			lblNormBlack.setText( "" + ( sliderNormBlack.getValue() / 10.0 ) + "%" );
			sliderNormBlack.setValue( imageSpec.getNormalisationBlack() );
			
			lblNormWhite.setText( "" + ( sliderNormWhite.getValue() / 10.0 ) + "%" );
			sliderNormWhite.setValue( imageSpec.getNormalisationWhite() );
			
			chkIndepNorm.setSelected(env.getSimul().getSpecs().getIndependentNormalisation());

			sliderNormBlack.addChangeListener(this);
			sliderNormWhite.addChangeListener(this);
		
			updateNormInfos();
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "fillContent()", ex);}
	}


	

	
	
	
	
	
	private final void updateSpectre()
	{
		if(!this.isVisible()) return;
		
		// recherche le max
		double dblMaxIntensity = 0.0;
		double dblIntensity;
		for(int i=0; i<env.getSimul().getImagesCount(); i++){
			dblIntensity = env.getSimul().getResults().getImageAt(i).computeMaxTotalElectrons();
			if(dblIntensity > dblMaxIntensity) dblMaxIntensity = dblIntensity;
		}
		
		cvDistrib.clearFunctions();
		res = dblMaxIntensity / (NUMBER_OF_VALUES-1);
		if(res <= 0) return;
		cvDistrib.setXPolicy(0, dblMaxIntensity, dblMaxIntensity/5, dblMaxIntensity/50);
		
		
		double[] values = new double[NUMBER_OF_VALUES];
		for(int im=0; im<env.getSimul().getImagesCount(); im++){
			
			// vide les donnees
			for(int i=0; i<values.length; i++) values[i] = 0;
			
			// infos sur l'image
			SResultImage img = env.getSimul().getResults().getImageAt(im);
			int x = img.getImageWidth();
			int y = img.getImageHeight();
			double min = dblMaxIntensity * ( imageSpec.getNormalisationBlack() / 1000.0 );
			double max = dblMaxIntensity * ( imageSpec.getNormalisationWhite() / 1000.0 );
			double val;
			
			for(int i=0; i<x; i++){
				for(int j=0; j<y; j++){
					val = img.getTotalElectrons(i, j);
					if(val > max) val = max;
					if(val < min) val = min;
					values[ (int)Math.round( val / res ) ]++;
				}
			}
			
			// annule les donnees vides
			for(int i=0; i<values.length; i++) if(values[i] == 0) values[i] = Double.NaN;
	
			String points = "";
			for(int i=0; i<values.length; i++){
				points += "[" + (i*res) + ";" + values[i] + "]";
			}
	
			SPointsFunction f = new SPointsFunction("","", SPointsFunction.SIMPLE_POINTS_FUNCTION + points);
			cvDistrib.addFunction(f);
		}
		updateNormInfos();
	}
	
	
	
	
	private final void updateNormInfos()
	{
		try{
			cvDistrib.clearFrontYAxis();
			
			
			double valB = ( (NUMBER_OF_VALUES-1) * res ) * ( sliderNormBlack.getValue() / 1000.0 ) ;
	    	lblNormBlack.setText( "" + ( sliderNormBlack.getValue() / 10.0 ) + "% (" + Maths.toString( valB, 2 ) + " e/pix)" );
	    	
			double valW = ( (NUMBER_OF_VALUES-1) * res ) * ( sliderNormWhite.getValue() / 1000.0 ) ;
	    	lblNormWhite.setText( "" + ( sliderNormWhite.getValue() / 10.0 ) + "% (" + Maths.toString( valW, 2 ) + " e/pix)" );
	    	
	    	cvDistrib.addFrontYAxis( valB );
	    	cvDistrib.addFrontYAxis( valW );
	    	
	    	
	    	
	    	cvDistrib.updateGraphic();
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "updateNormInfos()", ex);}
	}
	


	
    public void stateChanged(ChangeEvent e)
    {
    	try{
    
	        if( e.getSource() == sliderNormBlack ){
	        	if(res > 0){
	        		if( sliderNormBlack.getValue() > (sliderNormWhite.getValue()-2) ) return;
	        		updateNormInfos();
		        	if( sliderNormBlack.getValueIsAdjusting() == false ){
		        		for(int iImgNb=0; iImgNb<env.getSimul().getImagesCount(); iImgNb++) env.getSimul().getSpecs().getImageAt(iImgNb).setNormalisationBlack( sliderNormBlack.getValue() );
		        		SOSGenerator.generate(env);
		        	}
	        	}
	        	return;
	        }
	        
	        if( e.getSource() == sliderNormWhite ){
	        	if(res > 0){
	        		if( sliderNormWhite.getValue() < (sliderNormBlack.getValue()+2) ) return;
	        		updateNormInfos();
		        	if( sliderNormWhite.getValueIsAdjusting() == false ){
		        		for(int iImgNb=0; iImgNb<env.getSimul().getImagesCount(); iImgNb++) env.getSimul().getSpecs().getImageAt(iImgNb).setNormalisationWhite( sliderNormWhite.getValue() );
		        		SOSGenerator.generate(env);
		        	}
	        	}
	        	return;
	        }
	        
    	}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "stateChanged()", ex);}
    }

    
	public void actionPerformed(ActionEvent ae)
	{
		super.actionPerformed(ae);
		try{
	        if( ae.getSource() == chkIndepNorm ){
	        	updateNormInfos();
		        env.getSimul().getSpecs().setIndependentNormalisationk(chkIndepNorm.isSelected());
		        SOSGenerator.generate(env);
	        	return;
	        }
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "actionPerformed()", ex);}
	}

    
    
	public void onDialogBecomeVisible()
	{
		super.onDialogBecomeVisible();
		this.updateSpectre();
	}

	public void onSimulationStateChanged(boolean blIsSimulating)
	{
		try{
			sliderNormBlack.setEnabled(!blIsSimulating);
			sliderNormWhite.setEnabled(!blIsSimulating);
			chkIndepNorm.setEnabled(!blIsSimulating);
			if(!blIsSimulating) updateSpectre();
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "onSimulationStateChanged()", ex);}
	}


	
	

}
