/*
 SOS Project Classes.

 Copyright (C) SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package sos.framework.ui.mutables;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import sos.framework.maths.Maths;
import sos.framework.maths.conversions.SUnit;
import sos.framework.simulation.SEnv;
import sos.framework.simulation.simul.results.SResultImage;
import sos.framework.simulation.simul.results.SResultNoise;
import sos.framework.ui.SOSDialog;
import sos.framework.ui.SOSMutableDialog;
import sos.framework.ui.editor.imagecanvas._ImageCanvasListener;
import sos.lib.logger.Logger;
import sos.lib.uicommon.UI;

public class PixelInfosMutable extends SOSMutableDialog implements _ImageCanvasListener, MouseWheelListener
{

	private static final long serialVersionUID 			= 1L;
	
	private static final Object[] AVAILABLE_UNITS		= new Object[] {SUnit.UNIT_JY_PER_STERAD, SUnit.UNIT_ELECT_PER_PIXEL, SUnit.UNIT_PERCENT};


	private JPanel pnlNoises							= null;
	//private JPanel pnlColor								= null;
	private GridBagConstraints cNoises					= null;
	
	private JLabel lblPosAndColor						= null;
	private JLabel lblSources							= null;
	private JLabel lblTotal								= null;
	
	private JLabel[] lblNoises							= new JLabel[0];
	private String[] strNoises							= null;
	
	private JComboBox cmbUnits							= null;

	private int iLastX									= -1;
	private int iLastY									= -1;

	private SResultImage imageResult					= null;
	
	
	
	
	
	
	
	
	public PixelInfosMutable(final SEnv env)
	{
		super(env, "Sonde", true, SOSDialog.BUTTON_CLOSE, SOSDIALOG_MODE_RIGHT_LATERAL);
		this.buildContent();
		this.env.getWindow().getEditor().getImageCanvas().addImageCanvasListener(this);
	}
	

	private final void buildContent()
	{
		try{
			// construit le contenu
			cNoises = new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 3, 0, 3), 0, 0);
			JLabel lbl;

			// unites
			JPanel pnlUnit = new JPanel(new BorderLayout());
			pnlUnit.add(UI.createLabel("  " + "Unité :  ", UI.FONT_PLAIN_ITALIC), BorderLayout.WEST);
			pnlUnit.add(cmbUnits = UI.createComboBox(AVAILABLE_UNITS, this, 0), BorderLayout.CENTER);
			cmbUnits.addMouseWheelListener(this);
			
			// donnees
			JPanel pnlPointer = UI.createPanel(new GridBagLayout());
			pnlPointer.setOpaque(true);
			pnlPointer.setBackground(Color.black);
			
			pnlPointer.add(lbl = UI.createLabel("Pointeur", UI.FONT_BOLD), cNoises);
			lbl.setForeground(Color.white);
			cNoises.gridy++;
			
			pnlPointer.add(lblPosAndColor = UI.createLabel("-", UI.FONT_PLAIN_MONOSPACED), cNoises);
			lblPosAndColor.setForeground(Color.white);
			cNoises.gridy++;
			
			pnlPointer.add(lblTotal = UI.createLabel("-", UI.FONT_PLAIN_MONOSPACED), cNoises);
			lblTotal.setForeground(Color.white);
			cNoises.gridy++;

			pnlPointer.add(lbl = UI.createLabel("Composants", UI.FONT_BOLD), cNoises);
			lbl.setForeground(Color.white);
			cNoises.gridy++;
			
			pnlPointer.add(lblSources = UI.createLabel("-", UI.FONT_PLAIN_MONOSPACED), cNoises);
			lblSources.setForeground(Color.white);
			cNoises.gridy++;
			
			// liste des bruits
			cNoises.gridy = 0;
			pnlNoises = UI.createPanel(new GridBagLayout());
			pnlNoises.setOpaque(true);
			pnlNoises.setBackground(Color.black);

			// l'ensemble des elements precedents
			JPanel pnlTotal = new JPanel(new BorderLayout());
			pnlTotal.add(pnlUnit, BorderLayout.NORTH);
			pnlTotal.add(pnlPointer, BorderLayout.CENTER);
			pnlTotal.add(pnlNoises, BorderLayout.SOUTH);

			// pour completer le bas de la fenetre
			//JPanel pnlFill = UI.createPanel(new BorderLayout());
			//pnlFill.setOpaque(true);
			//pnlFill.setBackground(Color.blue);
			//pnlFill.setMinimumSize(new Dimension(20, 20));
			//pnlFill.setPreferredSize(new Dimension(20, 20));
			
			// couleurs
			//pnlColor = new JPanel(null);
			//pnlColor.setOpaque(true);
			//pnlColor.setBackground(Color.red);
			//pnlColor.setMinimumSize(new Dimension(20, 20));
			//pnlColor.setPreferredSize(new Dimension(20, 20));
			//pnlFill.add(pnlColor, BorderLayout.NORTH);
			//pnlFill.add(UI.createPanel(null), BorderLayout.CENTER);
			JPanel pnlFill = UI.createPanel(null);
			pnlFill.setOpaque(true);
			pnlFill.setBackground(Color.black);

			
			// ajoute le tout
			JPanel theContent = UI.createPanel(new BorderLayout());
			theContent.add(pnlTotal, BorderLayout.NORTH);
			theContent.add(pnlFill, BorderLayout.CENTER);
			
			super.setContent(theContent);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "buildContent()", ex);}
	}


	
	
	

	
	

	protected final void fillImage()
	{
		try{
			// recupere l'image actuelle
			this.imageResult = env.getSimul().getResults().getActualImage();
			
			// on regarde si le nombre de bruits a change
			if( lblNoises.length != imageResult.getGeneratedNoisesCount() ){
				
				// enleve la liste actuelle
				pnlNoises.removeAll();
				
				// recree une liste
				if( imageResult.getGeneratedNoisesCount() > 0 ){

					SResultNoise noise;
					lblNoises = new JLabel[imageResult.getGeneratedNoisesCount()];
					strNoises = new String[imageResult.getGeneratedNoisesCount()];
					for(int i=0; i<imageResult.getGeneratedNoisesCount(); i++){
						noise = imageResult.getGeneratedNoiseAt(i);
						strNoises[i] = noise.getNoiseName();
						lblNoises[i] = UI.createLabel("", UI.FONT_PLAIN_MONOSPACED);
						lblNoises[i].setForeground(Color.white);
						pnlNoises.add(lblNoises[i], cNoises);
						cNoises.gridy++;
					}
					
					
				}else{
					// pas de bruits dans cette simulation
					lblNoises = new JLabel[0];
					strNoises = null;
				}
				
				// met a jour l'affichage
				env.getWindow().getEditor().getLeftBar().updateSizerHeight();
				env.getWindow().getEditor().getRightBar().updateSizerHeight();
				
				if( this.getActualWindowMode() == SOSDIALOG_MODE_WINDOW ){
					int i = imageResult.getGeneratedNoisesCount();
					theDialog.setSize(theDialog.getWidth(), 260 + (int)((i==0||i==1?1:(i*1.2))*lblSources.getHeight()) );
				}
				
				// met a jour les donnees
				updateValues(iLastX, iLastY);
			}

		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "fillContent()", ex);}
	}

	
	
	
	
	
	
	
	
	
	private final void updateValues(final int x, final int y)
	{
		try{
			if( this.isVisible() == false ) return;
			
			iLastX = x;
			iLastY = y;
			
 			// position du curseur
			String strPos = "x : " + x;
			while( strPos.length() < 9 ) strPos = strPos + " ";

 			strPos = strPos + " " + "y : " + y;
 			while( strPos.length() < 18 ) strPos = strPos + " ";
 			strPos = "  " + strPos;
 			
 			// on regarde si on est dans l'image
 			if( (env.getSimul().isSimulating() == false) && (imageResult != null) && (x > -1) && (x < imageResult.getImageWidth()) && (y > -1) && (y < imageResult.getImageHeight() )){
 				
	 			// unite choise
	 			SUnit unit = (SUnit)(cmbUnits.getSelectedItem());
	 			
 				// couleur
	 			int c = imageResult.getNormalizedValue(x, y);
	 			if(c < 10) strPos = strPos + "  " + "c :   " + c + " / 255";
	 			else if(c < 100) strPos = strPos + "  " + "c :  " + c + " / 255";
	 			else strPos = strPos + "  " + "c : " + c + " / 255";
	 			
	 			// total
	 			lblTotal.setText(  "  " + "Total   : " + convertFromElectPerPix(imageResult.getTotalElectrons(x, y), unit));
	 			
	 			// les objets
	 			lblSources.setText("  " + "Sources : " + convertFromElectPerPix(imageResult.getSourcesElectrons(x, y), unit));
	 			
	 			// les bruits
	 			if( lblNoises.length == imageResult.getGeneratedNoisesCount() ){
 					for(int i=0; i<lblNoises.length; i++){
						lblNoises[i].setText("  " + strNoises[i] + " : " + convertFromElectPerPix(imageResult.getNoiseElectrons(x, y, i), unit));
					}
	 			}

 			}else{
 	
 				// on est pas dans l'image
 				strPos = strPos + "  " + "c :   - / 255";
 				lblTotal.setText(  "  " + "Total   : -");
 				lblSources.setText("  " + "Sources : -");
 				for(int i=0; i<lblNoises.length; i++){
 					lblNoises[i].setText("  " + strNoises[i] + " : -");
 				}
	 			
 			}

	 		lblPosAndColor.setText(strPos);
	 		
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "updateValues()", ex);}
	}
	
	private final String convertFromElectPerPix(final double valueFrom, final SUnit unitDest)
	{
		try{
			String out = "";
			double ret = 0.0;
			
			if( unitDest.getID() == SUnit.UNIT_PERCENT_ID ){
				
				ret = ( valueFrom / imageResult.getTotalElectrons(iLastX, iLastY) ) * 100;
				out = out + ( Math.round( ret * 100.0 ) / 100.0 ) + " %";
				while( out.length() < 7 ) out = out + " ";
				
				ret = ( valueFrom * imageResult.getNormalisationFactor() );
				out = out + " ( " + ( Math.round( ret * 100.0 ) / 100.0 ) + " / 255 )";

			}else{
				switch(unitDest.getID())
				{
					case SUnit.UNIT_ELECT_PER_PIXEL_ID:
						ret = valueFrom;
						break;
					case SUnit.UNIT_JY_PER_STERAD_ID: 
						ret = valueFrom * imageResult.getToJyPerSteradFactor();
						break;
				}
				out = Maths.toString( ret );
				out = out.toLowerCase() + " " + unitDest.getAbbrev();
			}
			return out;
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "convertFromElectPerPix()", ex);}
		return "?";
	}
	
	
	

	
	


	public void actionPerformed(ActionEvent ae)
	{
		try{
			super.actionPerformed(ae);
			if(ae.getSource() == cmbUnits){
				updateValues(iLastX, iLastY);
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "actionPerformed()", ex);}
	}
	
    public void mouseWheelMoved(MouseWheelEvent e)
    {
		try{
			if(e.getSource() == cmbUnits){
				int iNewIndex = cmbUnits.getSelectedIndex() + e.getWheelRotation();
				if(iNewIndex < 0) iNewIndex = 0;
				if(iNewIndex >= cmbUnits.getItemCount()) iNewIndex = cmbUnits.getItemCount()-1;
				cmbUnits.setSelectedIndex(iNewIndex);
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "mouseWheelMoved()", ex);}
    }
	
	
	public void onSimulationStateChanged(boolean blIsSimulating)
	{
		if(blIsSimulating) updateValues(-1, -1);
		else fillImage();
	}

	public void onImageCanvasMouseClicked(final int x, final int y) {}
	
	public void onImageCanvasMouseMoved(final int x, final int y)
	{
		updateValues(x, y);
	}

	
	
	
	
	


	

}
