/*
 SOS Project Classes.

 Copyright (C) SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package sos.framework.ui.mutables;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import sos.framework.generator.SOSGenerator;
import sos.framework.ressources.Icons;
import sos.framework.ressources.Lang;
import sos.framework.simulation.SEnv;
import sos.framework.simulation.simul.SSimul;
import sos.framework.simulation.simul.specs.filters.SFilter;
import sos.framework.ui.SOSDialog;
import sos.framework.ui.SOSMutableDialog;
import sos.framework.ui.dialogs.FilterDialog;
import sos.lib.logger.Logger;
import sos.lib.uicommon.SOSComboBox;
import sos.lib.uicommon.UI;

public class PropertiesMutable extends SOSMutableDialog
{

	private static final long serialVersionUID 			= 1L;


	/* les elements graphiques */
	private JButton btnSave								= null;
	private JButton btnUpdate							= null;
	
	private JTextField txtSizeX							= null;
	private JTextField txtSizeY							= null;
	private JTextField txtPixelSize						= null;
	private JTextField txtObsTime						= null;
	private JTextField txtTelecDiam						= null;
	
	private SOSComboBox optMonoFilter					= null;
	
	private JButton btnEditFilter						= null;
	private JButton btnAddFilter						= null;
	private JButton btnRemoveFilter						= null;
	

	
	
	
	
	
	public PropertiesMutable(final SEnv env)
	{
		super(env, Lang.STR_TITLE_PROPERTIES, false, SOSDialog.BUTTONS_CLOSE, SOSDIALOG_MODE_LEFT_LATERAL);
		this.buildContent();
	}
	

	private final void buildContent()
	{
		try{
			// construit le contenu
			GridBagConstraints c = new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 3, 0, 3), 0, 0);
			JPanel theContent = UI.createPanel(new GridBagLayout());
			c.gridwidth = 3;
			
			// taille
			theContent.add(UI.createLabel(Lang.STR_OPT_IMAGE_TITLE_ZONE, UI.FONT_BOLD), c);
			c.gridwidth = 1;
			c.gridy++;
			theContent.add(UI.createLabel("  " + Lang.STR_OPT_IMAGE_ZONE_SIZE, UI.FONT_PLAIN_ITALIC), c);
			c.gridx++;
			theContent.add(txtSizeX = UI.createTextField("-", "px"), c);
			c.gridx++;
			theContent.add(txtSizeY = UI.createTextField("-", "px"), c);
			c.gridx = 0;
			
			// resolution
			c.gridy++;
			theContent.add(UI.createLabel("  " + Lang.STR_OPT_IMAGE_ZONE_RES, UI.FONT_PLAIN_ITALIC), c);
			c.gridx++;
			c.gridwidth = 2;
			theContent.add(txtPixelSize = UI.createTextField("-", "arcsec/px"), c);
			c.gridwidth = 1;
			c.gridx = 0;
	
			// filtre
			c.gridy++;
			c.gridwidth = 3;
			theContent.add(UI.createLabel("Filtre d'observation", UI.FONT_BOLD), c);
			c.gridwidth = 1;
			c.gridy++;
			btnEditFilter = UI.createButton("Editer", this, UI.SIZE_SMALL_ICON, Icons.ICON_BUTTON_EDIT);
			btnAddFilter = UI.createButton("Ajouter", this, UI.SIZE_SMALL_ICON, Icons.ICON_BUTTON_ADD);
			btnRemoveFilter = UI.createButton("Supprimer", this, UI.SIZE_SMALL_ICON, Icons.ICON_BUTTON_REMOVE);
			c.insets = new Insets(3, 3, 0, 0);
			theContent.add(UI.createButtonsPanel(new JButton[] {btnAddFilter, btnRemoveFilter, btnEditFilter}), c);
			c.insets = new Insets(3, 3, 0, 3);
			c.gridx++;
			c.gridwidth = 2;
			optMonoFilter = UI.createComboBox(null, this, -1);
			theContent.add(optMonoFilter, c);
			c.gridwidth = 1;
			c.gridx = 0;
			
			// variables
			c.gridy++;
			c.gridwidth = 3;
			theContent.add(UI.createLabel("Variables", UI.FONT_BOLD), c);
			c.gridwidth = 1;
			
			c.gridy++;
			theContent.add(UI.createLabel("  " + "Temps de pause", UI.FONT_PLAIN_ITALIC), c);
			c.gridx++;
			c.gridwidth = 2;
			theContent.add(txtObsTime = UI.createTextField("-", "s"), c);
			c.gridwidth = 1;
			c.gridx = 0;
			
			c.gridy++;
			theContent.add(UI.createLabel("  " + "Diamètre du télescope", UI.FONT_PLAIN_ITALIC), c);
			c.gridx++;
			c.gridwidth = 2;
			theContent.add(txtTelecDiam = UI.createTextField("-", "m"), c);
			c.gridwidth = 1;
			c.gridx = 0;
			
			// les boutons
			c.gridy++;
			c.gridwidth = 3;
			c.insets = new Insets(3, 3, 3, 3);
			btnSave = UI.createButton("Enregistrer", this, null);
			btnUpdate = UI.createButton("Appliquer", this, null);
			theContent.add(UI.createButtonsPanel(new JButton[] {btnSave, btnUpdate}), c);
			
			super.setContent(theContent);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "buildContent()", ex);}
	}


	
	
	
	
	
	protected final void fillSimulation()
	{
		try{
			
			optMonoFilter.setModel(new DefaultComboBoxModel(env.getSimul().getSpecs().getFilters()));
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "fillSimulation()", ex);}
	}

	protected final void fillImage()
	{
		try{
			this.setTitle(Lang.STR_TITLE_PROPERTIES + " " + (imageSpec.getImageNumber()+1) );
			
			txtSizeX.setText(""+imageSpec.getImageWidth());
			txtSizeY.setText(""+imageSpec.getImageHeight());
			txtPixelSize.setText(""+imageSpec.getPixelResolution());

			txtObsTime.setText(""+imageSpec.getVariableValue(SSimul.VAR_OBSERVATION_TIME_NAME));
			txtTelecDiam.setText(""+imageSpec.getVariableValue(SSimul.VAR_TELESCOPE_DIAMETER_NAME));
			
			optMonoFilter.setSelectedItem(imageSpec.getFilter());
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "fillContent()", ex);}
	}

	
	
	

	
	private final void toggleSelectedFilter()
	{
		try{
			SFilter f = getSelectedFilter();
			if( (optMonoFilter.getSelectedIndex() > 0) && (f != null) && (f.isLocked() == false) ) btnRemoveFilter.setEnabled(true);
			else btnRemoveFilter.setEnabled(false);
			btnEditFilter.setEnabled(optMonoFilter.getSelectedIndex() > 0);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "toggleSelectedFilter()", ex);}
	}

	
	
	private void saveSimulation()
	{
		try{
			try{ imageSpec.setFilter(getSelectedFilter()); }catch(Exception ex) {}
			
			try{ imageSpec.setImageSizeX(Integer.parseInt(txtSizeX.getText())); }catch(Exception ex) {}
			try{ imageSpec.setImageSizeY(Integer.parseInt(txtSizeY.getText())); }catch(Exception ex) {}
			try{ imageSpec.setPixelResolution(Double.parseDouble(txtPixelSize.getText())); }catch(Exception ex) {}
			
			try{ imageSpec.updateVariable(SSimul.VAR_OBSERVATION_TIME_NAME, Double.parseDouble(txtObsTime.getText())); }catch(Exception ex) {}
			try{ imageSpec.updateVariable(SSimul.VAR_TELESCOPE_DIAMETER_NAME, Double.parseDouble(txtTelecDiam.getText())); }catch(Exception ex) {}
			
			toggleSelectedFilter();
	
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "saveSimulation()", ex);}
	}

	private void updateSimulation()
	{
		try{
			this.saveSimulation();
			SOSGenerator.generate(env);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "updateSimulation()", ex);}
	}

	
	
	
	private final SFilter getSelectedFilter()
	{
		try{
			if(optMonoFilter.getSelectedItem() != null) return (SFilter)(optMonoFilter.getSelectedItem());
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "getSelectedFilter()", ex);}
		return null;
	}

	private final void removeSelectedFilter()
	{
		try{
			SFilter f = getSelectedFilter();
			int mem = optMonoFilter.getSelectedIndex();
			if( (mem > 0) && (f != null) && (f.isLocked() == false) ){
				int ret = JOptionPane.showConfirmDialog(env.getWindow(), "Etes-vous sur de vouloir supprimer définitivement ce filtre ?", "Confirmation ?", JOptionPane.OK_CANCEL_OPTION, JOptionPane.INFORMATION_MESSAGE); 
				if( ret == JOptionPane.OK_OPTION ){
					env.getSimul().getSpecs().getFilters().remove(f);
					optMonoFilter.setSelectedIndex(mem-1);
				}
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "removeSelectedFilter()", ex);}
	}
	
	
	
	
	
	
	private final void showFilterEditor(final SFilter filter)
	{
		try{
			// s'assure de bien avoir un bruit
			SFilter localFilter = null;
			if(filter == null) localFilter = new SFilter("Nouveau filtre", 0, 0);
			else localFilter = filter;
			
			// affiche la fenetre
			FilterDialog fm;
			if( this.getActualWindowMode() == SOSDIALOG_MODE_WINDOW ) fm = new FilterDialog(this.getWindowModeDialog(), localFilter);
			else fm = new FilterDialog(this.getPanelModeFrame(), localFilter);	
			
			fm.setVisible(true);
			
			if(fm.retournedValue() == SOSDialog.BUTTON_OK){
				
				// on ajoute le filtre si c'est un nouveau
				if(filter == null) env.getSimul().getSpecs().getFilters().add(localFilter);
				optMonoFilter.updateUI();
				optMonoFilter.setSelectedIndex(optMonoFilter.getItemCount()-1);
				
				
			}
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "showFilterEditor()", ex);}
	}
	
	
	
	
	
	

	public final void actionPerformed(ActionEvent ae)
	{
		super.actionPerformed(ae);
		try{
		
			if(ae.getSource() == btnAddFilter) {showFilterEditor(null); return;}
			if(ae.getSource() == btnRemoveFilter) {removeSelectedFilter(); return;}
			if(ae.getSource() == btnEditFilter) {showFilterEditor(getSelectedFilter()); return;}
			
			if(ae.getSource() == btnUpdate) {updateSimulation(); return;}
			if(ae.getSource() == btnSave) {saveSimulation(); return;}
			
			if(ae.getSource() == optMonoFilter) {toggleSelectedFilter(); return;}
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "actionPerformed()", ex);}
	}
	
	

	public void onSimulationStateChanged(boolean blIsSimulating)
	{
		btnUpdate.setEnabled(!blIsSimulating);
		btnSave.setEnabled(!blIsSimulating);
	}
	


}
