/*
 SOS Project Classes.

 Copyright (C) 2006-2008 SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sos.lib.fitswriter;

import sos.framework.maths.Maths;



public class FitsHeaderCard
{

	private static String SPACES_80 			= "                                                                                ";
	
	public static final int HEADER_CARD_UNDEF	= 0;
	public static final int HEADER_CARD_INT		= 1;
	public static final int HEADER_CARD_DOUBLE	= 2;
	public static final int HEADER_CARD_STRING	= 3;
	public static final int HEADER_CARD_BOOLEAN	= 4;
	
	
	private int iType							= HEADER_CARD_UNDEF;
	
	private String strKeyword					= null;
	private String strValue						= null;
	private String strComment					= null;
	
	
	
	
	
	

	public FitsHeaderCard(final String strKeyword, final String strValue, final String strComment)
	{
		this.strKeyword = strKeyword;
		this.strValue = "'" + strValue + "'";
		this.strComment = strComment;
		this.iType = HEADER_CARD_STRING;
	}
	
	public FitsHeaderCard(final String strKeyword, final double dblValue, final String strComment)
	{
		this.strKeyword = strKeyword;
		this.strValue = Maths.toString(dblValue, 3).toUpperCase();
		this.strComment = strComment;
		this.iType = HEADER_CARD_DOUBLE;
	}
	
	public FitsHeaderCard(final String strKeyword, final int intValue, final String strComment)
	{
		this.strKeyword = strKeyword;
		this.strValue = "" + intValue;
		this.strComment = strComment;
		this.iType = HEADER_CARD_INT;
	}
	
	public FitsHeaderCard(final String strKeyword, final boolean blValue, final String strComment)
	{
		this.strKeyword = strKeyword;
		this.strValue = (blValue ? "T" : "F");
		this.strComment = strComment;
		this.iType = HEADER_CARD_BOOLEAN;
	}
	
	public FitsHeaderCard(final String strKeyword)
	{
		this.strKeyword = strKeyword;
	}
	
	
	



	
	public String getStrComment()
	{
		return strComment;
	}

	public void setStrComment(String strComment)
	{
		this.strComment = strComment;
	}



	public String getStrKeyword()
	{
		return strKeyword;
	}

	public void setStrKeyword(String strKeyword)
	{
		this.strKeyword = strKeyword;
	}



	public String getStrValue()
	{
		return strValue;
	}

	public void setStrValue(String strValue)
	{
		this.strValue = strValue;
	}
	
	
	
	
	
	
	
	
	
	
	public final boolean write(final FitsOutputStream out)
	{
		try{
			StringBuffer sb = new StringBuffer(FitsFile.FITS_HEADER_CARD_SIZE);
			if(strKeyword != null){
				
				// taille du mot cle
				if(strKeyword.length() > FitsFile.FITS_HEADER_KEYWORD_SIZE){
					sb.append( strKeyword.substring(0, FitsFile.FITS_HEADER_KEYWORD_SIZE) );
				}else{
					sb.append( strKeyword );
					sb.append( SPACES_80.substring(0, FitsFile.FITS_HEADER_KEYWORD_SIZE - strKeyword.length()) );
				}

				// valeur
				if(strValue != null){
					sb.append( "= " );
					if(strValue.length() > FitsFile.FITS_HEADER_VALUE_SIZE){
						sb.append( strValue.substring(0, FitsFile.FITS_HEADER_VALUE_SIZE) );
					}else{
						if(iType == HEADER_CARD_STRING){
							// aligne a gauche
							sb.append( strValue );
							sb.append( SPACES_80.substring(0, FitsFile.FITS_HEADER_VALUE_SIZE - strValue.length()) );	
						}else{
							// aligne a droite
							sb.append( SPACES_80.substring(0, FitsFile.FITS_HEADER_VALUE_SIZE - strValue.length()) );
							sb.append( strValue );
						}
					}
					
					// commentaire
					if(strComment != null){
						sb.append( " / " );
						if(strComment.length() > FitsFile.FITS_HEADER_COMMENT_SIZE){
							sb.append( strComment.substring(0, FitsFile.FITS_HEADER_COMMENT_SIZE) );
						}else{
							sb.append( strComment );
							sb.append( SPACES_80.substring(0, FitsFile.FITS_HEADER_COMMENT_SIZE - strComment.length()) );
						}
					}else{
						sb.append( SPACES_80.substring(0, FitsFile.FITS_HEADER_COMMENT_SIZE + 3) );
					}
					
					
				}else{
					sb.append( SPACES_80.substring(0, FitsFile.FITS_HEADER_VALUE_SIZE + FitsFile.FITS_HEADER_COMMENT_SIZE + 5) );
				}

				if( !out.write( sb.toString().getBytes() ) ) return false;

			}
			return true;
		}catch(Exception ex) {}
		return false;
	}
	

	
	
}
