package sos.lib.mathparser;

import java.util.Hashtable;
import java.util.Vector;

import sos.lib.mathparser.functions.Function;
import sos.lib.mathparser.nodes.Node;
import sos.lib.mathparser.nodes.NodeFct;
import sos.lib.mathparser.nodes.NodeNb;
import sos.lib.mathparser.nodes.NodeOp;
import sos.lib.mathparser.nodes.NodeVar;





public class ParsedExpr
{
	
	public static final int TYPE_END 		= -1;
	public static final int TYPE_ERROR 		= -2;
	
	public static final int TYPE_NB 		= 100;
	public static final int TYPE_VAR 		= 101;
	public static final int TYPE_FCT 		= 102;
	
	public static final int TYPE_PLUS 		= 1;
	public static final int TYPE_MINUS 		= 2;
	public static final int TYPE_TIMES 		= 3;
	public static final int TYPE_DIVIDES 	= 4;
	public static final int TYPE_POWER	 	= 5;
	
	public static final int TYPE_AND	 	= 30;
	public static final int TYPE_OR		 	= 31;
	public static final int TYPE_NO		 	= 32;
	
	public static final int TYPE_EGAL	 	= 40;
	public static final int TYPE_EGALEGAL 	= 41;
	public static final int TYPE_INF	 	= 42;
	public static final int TYPE_INFEGAL 	= 43;
	public static final int TYPE_SUP	 	= 44;
	public static final int TYPE_SUPEGAL 	= 45;
	public static final int TYPE_DIFF	 	= 46;
	
	public static final int TYPE_BRAOPEN 	= 50;
	public static final int TYPE_BRACLOSE 	= 51;
	public static final int TYPE_CROOPEN 	= 52;
	public static final int TYPE_CROCLOSE 	= 53;
	public static final int TYPE_VIRG	 	= 54;
	
	
	
	
	private ParserFactory factory;
	private String strLastErr = null;

	private Node root = null;
	private Vector names = new Vector();
	private Vector nodes = new Vector();

	private boolean blParsed = false;
	
	
	private Hashtable vars = new Hashtable();
	
	public Hashtable getLocalVars() {return vars;}
	public void setVarValue(String name, double value) {vars.put(name, new Double(value));}
	public double getVarValue(String name) throws Exception
	{
		if(vars.containsKey(name)) return ((Double)(vars.get(name))).doubleValue();
		else return factory.getVarValue(name);
	}

	private Hashtable funcs = new Hashtable();
	public Hashtable getLocalFuncs() {return funcs;}
	public Function getFunction(String name) throws Exception
	{
		if(funcs.containsKey(name)) return (Function)(funcs.get(name));
		else return factory.getFunction(name);
	}
	
	
	
	public String getLastErr() {return strLastErr;}
	public Node getRoot() {return root;}
	public Vector getNodesNames() {return names;}
	public Vector getNodes() {return nodes;}
	
	
	
	
	public ParsedExpr(ParserFactory factory, String[] lines)
	{
		this.factory = factory;
		try{
			if( lines.length > 1 ){
				String line;
				for(int i=0; i<lines.length; i++){
					line = lines[i];
					if(line.toLowerCase().startsWith("return ")){
						root = stExp(new Reader(line.substring(7)));
					}else if(line.startsWith("//") == false && line.indexOf('=') > -1){
						names.add(line.substring(0, line.indexOf('=')).trim());
						nodes.add(stExp(new Reader(line.substring(line.indexOf('=')+1).trim())));
					}
				}
			}else{
				if(lines[0].toLowerCase().startsWith("return ")) root = stExp(new Reader(lines[0].substring(7)));
				else root = stExp(new Reader(lines[0]));
			}
			blParsed = true;
		}catch(Exception e) {strLastErr = e.getMessage();}
	}


	
	
	
	
	
	
	
	public synchronized double value()
	{
		if(root != null) return root.value();
		else return Double.NaN;
	}
	
	public synchronized boolean compute()
	{
		if(blParsed == false) return false;
		strLastErr = null;
		try{
			if(names.size() > 0){
				Node node;
				for(int i=0; i<names.size(); i++){
					node = (Node)(nodes.get(i));
					node.compile();
					setVarValue((String)(names.get(i)), node.value());
				}
			}
			root.compile();
			if(names.size() > 0) for(int i=0; i<names.size(); i++) vars.remove(names.get(i));
			return true;
		}catch(Exception e) {strLastErr = e.getMessage(); System.out.println("ERR = " + strLastErr); return false;}
	}

	

	
	
	private Node stExp(Reader r) throws Exception {return stExp(r, null);}
	private Node stExp(Reader r, Node n) throws Exception
	{
		if(n == null) n = stPrd(r, null); 
		switch(r.lastType()){
			case TYPE_PLUS:
			case TYPE_MINUS:
			case TYPE_OR:
				NodeOp op = new NodeOp(this, r.lastType());
				r.nextWord();
				op.addChild(n);
				op.addChild(stPrd(r, null));
				return stExp(r, op);
		}
		return n;
	}
	
	private Node stPrd(Reader r, Node n) throws Exception
	{
		if(n == null) n = stPow(r, null); 
		switch(r.lastType()){
			case TYPE_TIMES:
			case TYPE_DIVIDES:
			case TYPE_AND:
				NodeOp op = new NodeOp(this, r.lastType());
				r.nextWord();
				op.addChild(n);
				op.addChild(stPow(r, null));
				return stPrd(r, op);
		}
		return n;
	}
	
	private Node stPow(Reader r, Node n) throws Exception
	{
		if(n == null) n = stFct(r); 
		switch(r.lastType()){
			case TYPE_POWER:
			case TYPE_EGALEGAL:
			case TYPE_SUPEGAL:
			case TYPE_INFEGAL:
			case TYPE_SUP:
			case TYPE_INF:
				NodeOp op = new NodeOp(this, r.lastType());
				r.nextWord();
				op.addChild(n);
				op.addChild(stFct(r));
				return stPow(r, op);
		}
		return n;
	}
	
	private Node stFct(Reader r) throws Exception
	{
		switch(r.lastType()){
			case TYPE_MINUS:
			case TYPE_NO:
				NodeOp mn = new NodeOp(this, r.lastType());
				r.nextWord();
				mn.addChild(stFct(r));
				return mn;
			case TYPE_NB:
				NodeNb nb = new NodeNb(this, r.lastNb());
				r.nextWord();
				return nb;
			case TYPE_BRAOPEN:
				r.nextWord();
				Node n = stExp(r, null);
				if(r.lastType() == TYPE_BRACLOSE) {r.nextWord(); return n;}
				else throw new Exception("parse : caractere attendu : ')'");
			case TYPE_VAR:
				String name = r.lastVar();
				int tWait;
				r.nextWord();
				if(r.lastType() == TYPE_BRAOPEN) tWait = TYPE_BRACLOSE;
				else if(r.lastType() == TYPE_CROOPEN) tWait = TYPE_CROCLOSE;
				else return new NodeVar(this, name);
				NodeFct fct = new NodeFct(this, name);
				r.nextWord();
				while(r.lastType() != tWait){
					fct.addParam(stExp(r, null));
					if(r.lastType() == TYPE_VIRG) r.nextWord();
					else if(r.lastType() != tWait) throw new Exception("parse : caractere attendu : ',' ou '" + (tWait==TYPE_BRACLOSE?")":"]") + "'");
				}
				r.nextWord();
				return fct;
			case TYPE_ERROR:
				throw new Exception("parse : erreur syntaxique");
		}
		throw new Exception("parse : mot attendu : nombre, variable, fonction, expression");
	}


	
	
}
