/*
 SOS Project Classes.

 Copyright (C) 2006-2008 SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sos.lib.mathparser;

import java.util.Vector;

import sos.framework.maths.Maths;
import sos.lib.mathparser.nodes.Node;
import sos.lib.mathparser.nodes.NodeFct;
import sos.lib.mathparser.nodes.NodeNb;
import sos.lib.mathparser.nodes.NodeOp;
import sos.lib.mathparser.nodes.NodeVar;


public class Printer
{

	private static boolean[] tend = new boolean[100];
	private static int inc = 0;
	
	private static String out;
	
	
	
	public static void print(Node n)
	{
		System.out.println(explain(n));
	}
	
	
	
	
	
	public static String explainAll(ParsedExpr exp)
	{
		String strRet = "";
		for(int i=0; i<exp.getNodesNames().size(); i++){
			strRet += "<b><u>\"" + (String)( exp.getNodesNames().get(i) ) + "\" expression</u> :</b><br>";
			strRet += explain( (Node)( exp.getNodes().get(i) ) ) + "<br>";
		}
		strRet += "<b><u>return expression</u> :</b><br>";
		strRet += explain( exp.getRoot() );
		return strRet;
	}
	
	public static String explain(Node n)
	{
		out = "";
		compile(n, false);
		return out;
	}
	
	
	

	private static void compile(Node n, boolean end)
	{
		
		if(inc > 0) tend[inc-1] = end;
		if(n instanceof NodeFct){
			NodeFct nf = ((NodeFct)n);
			Vector p = nf.getParams();
			printLine(nf.getName()+"()&nbsp;<font color=\"gray\"><i>" + Maths.toString( n.value() ) + "</i></font>");
			inc++;
			for(int i=0; i<p.size(); i++){
				if(i == p.size()-1) compile((Node)(p.get(i)), true);
				else compile((Node)(p.get(i)), false);
			}
			inc--;
		}
		if(n instanceof NodeOp){
			NodeOp no = ((NodeOp)n);
			Vector c = no.getChildren();
			printLine("["+getOp(no.type())+"]&nbsp;<font color=\"gray\"><i>" + Maths.toString( n.value() ) + "</i></font>");
			inc++;
			tend[inc-1] = false;
			for(int i=0; i<c.size(); i++){
				if(i == c.size()-1) compile((Node)(c.get(i)), true);
				else compile((Node)(c.get(i)), false);
			}
			inc--;
		}
		if(n instanceof NodeNb){
			NodeNb nn = ((NodeNb)n);
			printLine("&nbsp;" + Maths.toString( nn.value() ) );
		}
		if(n instanceof NodeVar){
			NodeVar nv = ((NodeVar)n);
			printLine("&nbsp;*" + nv.getName() + "&nbsp;<font color=\"gray\"><i>" + Maths.toString( n.value() ) + "</i></font>");
		}
	}
	
	
	private static void printLine(String str)
	{
		for(int i=0; i<inc-1; i++){
			if(tend[i]) out += "&nbsp;&nbsp;&nbsp;&nbsp;";
			else out += "&nbsp;|&nbsp;&nbsp;";
		}
		if(inc > 0){
			if(tend[inc-1]) out += "&nbsp;\\--";
			else out += "&nbsp;|--";
		}
		out += str + "<br>";
	}
	
	
	private static String getOp(int type)
	{
		switch(type){
			case ParsedExpr.TYPE_PLUS: return "+";
			case ParsedExpr.TYPE_MINUS: return "-";
			case ParsedExpr.TYPE_TIMES: return "*";
			case ParsedExpr.TYPE_DIVIDES: return "/";
			case ParsedExpr.TYPE_POWER: return "^";
			case ParsedExpr.TYPE_EGALEGAL: return "==";
			case ParsedExpr.TYPE_SUPEGAL: return ">=";
			case ParsedExpr.TYPE_INFEGAL: return "<=";
			case ParsedExpr.TYPE_SUP: return ">";
			case ParsedExpr.TYPE_INF: return "<";
			case ParsedExpr.TYPE_OR: return "|";
			case ParsedExpr.TYPE_AND: return "&";
			case ParsedExpr.TYPE_NO: return "!";
			default: return ""+type;
		}
	}


	
}

