/*
 SOS Project Classes.

 Copyright (C) 2006-2008 SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sos.lib.uicommon;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Insets;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

public class SOSNumberedTextPane extends JPanel
{
	private static final long serialVersionUID 	= 1L;
	
	public final int NUMBERS_PANEL_WIDTH 		= 30;
	
	public final Color NUMBERS_COLOR	 		= Color.lightGray;
	public final Color NUMBERS_BACK_COLOR 		= Color.white;
	
	public final Font TEXT_FONT;
	public final Font NUMBERS_FONT;
	
	private JTextArea text;
	private JScrollPane scroll;
	private JNumbersPane numbers;
	
	
	
	public SOSNumberedTextPane()
	{
		super(new BorderLayout());
		
		TEXT_FONT = new Font("Monospaced", Font.PLAIN, this.getFont().getSize());
		NUMBERS_FONT = new Font("Monospaced", Font.PLAIN, this.getFont().getSize()-1);
		
		text = new JTextArea2();
		text.setMargin(new Insets(5, 0, 5, 5));
		text.setFont(TEXT_FONT);
		
		scroll = new JScrollPane(text);
		scroll.setBorder(null);
		
		numbers = new JNumbersPane();
		numbers.setPreferredSize(new Dimension(NUMBERS_PANEL_WIDTH, NUMBERS_PANEL_WIDTH));

		this.add(numbers, BorderLayout.WEST);
		this.add(scroll, BorderLayout.CENTER);
	}
	
	
	
	public final JTextArea getTextArea() {return text;}
	
	
	public final void setText(final String strText) {text.setText(strText);}
	public final String getText() {return text.getText();}
	
	
	
	private class JTextArea2 extends JTextArea
	{
		private static final long serialVersionUID			= 1L;
		
		public void paintComponent(Graphics g)
		{
			super.paintComponent(g);
			numbers.repaint();
		}
	}

	private class JNumbersPane extends JPanel
	{
		private static final long serialVersionUID			= 1L;
		
		private Image buffNumbers = null;
		private int iMemHeight = 0;
		private int iWidth = NUMBERS_PANEL_WIDTH;
		
		public void paintComponent(Graphics g)
		{
			super.paintComponent(g);

			int iNewHeight = text.getHeight();
			if (buffNumbers == null || iNewHeight != iMemHeight) {
				iMemHeight = iNewHeight;
				drawNumbers();
			}
			
			g.drawImage(buffNumbers, 0, -scroll.getViewport().getViewRect().y, null);
			g.setColor(NUMBERS_BACK_COLOR);
			g.fillRect(0, scroll.getViewport().getViewRect().height, iWidth, iMemHeight-scroll.getViewport().getViewRect().height);
			g.dispose();
		}

		public void drawNumbers()
		{
			buffNumbers = this.createImage(iWidth, iMemHeight);
			Graphics g = buffNumbers.getGraphics();
			
			g.setFont(NUMBERS_FONT);
			g.setColor(NUMBERS_BACK_COLOR);
			g.fillRect(0, 0, iWidth, iMemHeight);
			g.setColor(NUMBERS_COLOR);
			g.drawLine(iWidth - 1, 0, iWidth - 1, iMemHeight);
			
			int top = text.getMargin().top;
	
			FontMetrics tfm = g.getFontMetrics(TEXT_FONT);
			FontMetrics nfm = g.getFontMetrics(NUMBERS_FONT);
			int tfh = tfm.getHeight();
			int nbLinesNew = (iMemHeight / tfh) + 1;
	
			String str;
			for (int i = 1; i < nbLinesNew; i++) {
				str = ""+i;
				g.drawString(str, iWidth - 2 - nfm.stringWidth(str), top + i * tfh - nfm.getDescent());
			}
		}

	}

}