/*
 SOS Project Classes.

 Copyright (C) 2006-2008 SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sos.lib.uicommon;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.LayoutManager;
import java.awt.event.ActionListener;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeListener;


public class UI extends JPanel
{

	private static final long		serialVersionUID		= -4739407578937135512L;

	public static final Dimension	SIZE_BUTTON				= new Dimension(100, 25);
	public static final Dimension	SIZE_ICON				= new Dimension(28, 28);
	public static final Dimension	SIZE_SMALL_ICON			= new Dimension(24, 24);
	
	public static Font				FONT_PLAIN				= null;
	public static Font				FONT_PLAIN_SMALL		= null;
	public static Font				FONT_PLAIN_VERY_SMALL	= null;
	public static Font				FONT_PLAIN_MONOSPACED	= null;
	public static Font				FONT_PLAIN_ITALIC		= null;

	public static Font				FONT_BOLD				= null;
	public static Font				FONT_BOLD_SMALL			= null;
	public static Font				FONT_BOLD_BIG			= null;
	public static Font				FONT_BOLD_ITALIC		= null;



	
	
	
	
	
	
	public static final void init()
	{
		try{
			Font f = new JPanel(null).getFont();
			FONT_PLAIN 				= new Font(f.getName(), Font.PLAIN, f.getSize());
			FONT_PLAIN_ITALIC 		= new Font(f.getName(), Font.PLAIN + Font.ITALIC, f.getSize());
			FONT_PLAIN_SMALL 		= new Font(f.getName(), Font.PLAIN, 11);
			FONT_PLAIN_VERY_SMALL 	= new Font(f.getName(), Font.PLAIN, 9);
			FONT_PLAIN_MONOSPACED 	= new Font("Monospaced", Font.PLAIN, 12);
			FONT_BOLD 				= new Font(f.getName(), Font.BOLD, f.getSize());
			FONT_BOLD_BIG 			= new Font(f.getName(), Font.BOLD, f.getSize() + 3);
			FONT_BOLD_SMALL 		= new Font(f.getName(), Font.BOLD, 10);
			FONT_BOLD_ITALIC 		= new Font(f.getName(), Font.BOLD + Font.ITALIC, f.getSize());
		}catch(Exception ex) {}
	}

	
	
	
	
	
	

	public static final JPanel createButtonsPanel(final JComponent[] cpns)
	{
		try{
			GridBagConstraints c = new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 3, 0, 0), 0, 0);
			JPanel pnlGridBag = UI.createPanel(new GridBagLayout());
			for(int i=0; i<cpns.length; i++) {pnlGridBag.add(cpns[i], c); c.gridx++;}
			JPanel pnlContent = UI.createPanel(new BorderLayout());
			pnlContent.add(UI.createPanel(null), BorderLayout.CENTER);
			pnlContent.add(pnlGridBag, BorderLayout.EAST);
			return pnlContent;
		}catch(Exception ex) {}
		return UI.createPanel(null);
	}
	
	
	public static final JComponent createInputFunctionPanel(final JTextField txtTop, final JComponent txtMain, final JTextField txtBottom, final JButton btnTop, final JButton btnBottom)
	{
		try{
			JPanel pnlMail = UI.createPanel(new BorderLayout());
			Color c = new Color(220, 220, 220);
			
			txtTop.setBorder(new EmptyBorder(3, 3, 3, 0)); txtTop.setEditable(false); txtTop.setBackground(c);
			txtBottom.setBorder(new EmptyBorder(3, 3, 3, 0)); txtBottom.setEditable(false); txtBottom.setBackground(c);

			//JScrollPane scroll = UI.createScrollPane(txtMain);
			
			pnlMail.setBorder(new EtchedBorder());
			//scroll.setBorder(null);
			
			JPanel pnlTop = createPanel(new BorderLayout());
			pnlTop.add(txtTop, BorderLayout.CENTER);
			if(btnTop != null) pnlTop.add(btnTop, BorderLayout.EAST);
			
			JPanel pnlBottom = createPanel(new BorderLayout());
			pnlBottom.add(txtBottom, BorderLayout.CENTER);
			if(btnBottom != null) pnlBottom.add(btnBottom, BorderLayout.EAST);
			
			pnlMail.add(pnlTop, BorderLayout.NORTH);
			pnlMail.add(txtMain, BorderLayout.CENTER);
			pnlMail.add(pnlBottom, BorderLayout.SOUTH);

			return pnlMail;
	
		}catch(Exception ex) {}
		return UI.createPanel(null);
	}

	
	
	
	
	
	
	
	
	

	
	public static final JTextArea createTextArea(final String strData)
	{
		JTextArea txt = null;
		try {
			txt = new JTextArea(strData);
			txt.setLineWrap(true);
			txt.setFont(FONT_PLAIN_MONOSPACED);
		} catch (Exception ex) {
		}
		return txt;
	}

	public static final SOSComboBox createComboBox(final Object[] strData, final ActionListener lst, final int iSelected)
	{
		SOSComboBox cb = null;
		try{
			if(strData == null) cb = new SOSComboBox();
			else cb = new SOSComboBox(strData);
			if(lst != null) cb.addActionListener(lst);
			if(iSelected > -1) cb.setSelectedIndex(iSelected);
		}catch(Exception ex) {}
		return cb;
	}

	public static final JRadioButton createRadioButton(final String strText, final ActionListener lst, final boolean blSel, final ButtonGroup g, final Font ft)
	{
		JRadioButton opt = null;
		try{
			opt = new JRadioButton(strText, blSel);
			opt.setOpaque(false);
			opt.addActionListener(lst);
			if(ft != null) opt.setFont(ft);
			if(g != null) g.add(opt);
		}catch(Exception ex) {}
		return opt;
	}

	public static final JLabel createLabel(final String strText) {return createLabel(strText, null);}
	public static final JLabel createLabel(final String strText, final Font ft)
	{
		JLabel lbl = new JLabel(strText);
		try{
			if(ft != null) lbl.setFont(ft);
		}catch(Exception ex) {}
		return lbl;
	}
	
	
	
	public static final JTextField createTextField(final String strText)
	{
		return new JTextField(strText);
	}

	public static final SOSTextField createTextField(final String strText, final String strUnit)
	{
		return new SOSTextField(strText, strUnit);
	}
	

	
	
	
	
	
	public static final JButton createIcon(final String strText, final ActionListener lst, final ImageIcon icon0, final ImageIcon icon1)
	{
		JButton btn = null;
		try{
			btn = new JButton("");
			btn.setToolTipText(strText);
			btn.setPreferredSize(SIZE_ICON);
			btn.setMaximumSize(SIZE_ICON);
			btn.setMinimumSize(SIZE_ICON);
			btn.setBorder(null);
			btn.setIcon(icon0);
			if(icon1 != null) btn.setRolloverIcon(icon1);
			btn.addActionListener(lst);
			btn.setMargin(new Insets(0, 0, 0, 0));
		}catch(Exception ex) {}
		return btn;
	}

	public static final JButton createButton(final String strText, final ActionListener lst, final Dimension d) {return createButton(strText, lst, d, null);}
	public static final JButton createButton(final String strText, final ActionListener lst, final Dimension d, final ImageIcon icon)
	{
		JButton btn = null;
		try{
			btn = new JButton(strText);
			if(d != null) {
				btn.setPreferredSize(d);
				btn.setMaximumSize(d);
				btn.setMinimumSize(d);
			}
			if(icon != null){
				btn.setIcon(icon);
				btn.setToolTipText(strText);
				btn.setText("");
			}
			btn.addActionListener(lst);
			btn.setMargin(new Insets(1, 1, 1, 1));
		}catch(Exception ex) {}
		return btn;
	}

	
	
	
	
	public static final JMenuItem createMenuItem(final String strText,
			final ActionListener lst, final ImageIcon img)
	{
		return createMenuItem(strText, lst, img, null);
	}

	public static final JMenuItem createMenuItem(final String strText, final ActionListener lst, final ImageIcon img, final String ActionCommand)
	{
		JMenuItem item = null;
		try{
			item = new JMenuItem(strText);
			item.addActionListener(lst);
			if(img != null) item.setIcon(img);
			if(ActionCommand != null) item.setActionCommand(ActionCommand);
			return item;
		}catch (Exception ex) {}
		return item;
	}

	public static final JCheckBox createCheckBox(final String strText,
			final ActionListener lst, final boolean blSel, final Font ft)
	{
		JCheckBox chk = null;
		try {
			chk = new JCheckBox(strText, blSel);
			chk.setOpaque(false);
			chk.addActionListener(lst);
			if (ft != null) chk.setFont(ft);
		} catch (Exception ex) {
		}
		return chk;
	}

	public static final JPanel createPanel(final LayoutManager layout)
	{
		return createPanel(layout, null);
	}

	public static final JPanel createPanel(final LayoutManager layout,
			final Dimension d)
	{
		JPanel pnl = null;
		try {
			pnl = new JPanel(layout);
			pnl.setOpaque(false);
			if (d != null) {
				pnl.setPreferredSize(d);
				pnl.setMaximumSize(d);
				pnl.setMinimumSize(d);
			}
		} catch (Exception ex) {
		}
		return pnl;
	}

	
	
	
	public static final JScrollPane createScrollPane(final Component cpn) {return createScrollPane(cpn, null);}
	public static final JScrollPane createScrollPane(final Component cpn, final Dimension d)
	{
		JScrollPane scr = null;
		try {
			scr = new JScrollPane(cpn);
			scr.setOpaque(false);
			scr.getViewport().setOpaque(false);
			if (d != null) {
				scr.setPreferredSize(d);
				scr.setMaximumSize(d);
				scr.setMinimumSize(d);
			}
		}catch(Exception ex) {}
		return scr;
	}

	
	
	
	
	
	
	
	public static final JSlider createSlider(final int orientation, final int min, final int max, final int value, final ChangeListener lst)
	{
		JSlider sld = null;
		try{
			sld = new JSlider(orientation, min, max, value);
			sld.setMajorTickSpacing(10);
			sld.setMinorTickSpacing(1);
			sld.setPaintTicks(false);
			sld.setPaintLabels(false);
			sld.setBorder(null);
			if(lst != null) sld.addChangeListener(lst);
		}catch(Exception ex) {}
		return sld;
	}
	
	

        
	
	
	
}
