/*
  SOS Project Classes.

  Copyright (C) SOS Project <http://sos.project.free.fr>
  SOS Project is :
    Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>
    Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package sos.lib.uicommon.graphics;

import java.awt.Color;

import sos.framework.maths.Maths;
import sos.lib.logger.Logger;


public class SPointsFunction extends SFunction
{
	
	public static final String SIMPLE_POINTS_FUNCTION			= "points:";
	public static final String LOG_POINTS_FUNCTION				= "lpoints:";

	// pour les points
	private String strPoints									= null;
	private double[] xValues									= null;
	private double[] yValues									= null;
	private boolean blLogY										= false;
	
	

	
	
	
	
	public SPointsFunction(final String strName, final String strDesc, final String strFunc)
	{
		super(strName, strDesc, null);
		this.setPoints(strFunc);
	}

	public SPointsFunction(final String strName, final String strDesc, final Color color, final String strFunc)
	{
		super(strName, strDesc, color);
		this.setPoints(strFunc);
	}
	
	
	
	
	
	public final void setPoints(final String strNewPoints)
	{
		try{
			xValues	= null;
			yValues = null;
			blLogY = false;
			strPoints = null;

			if((strNewPoints == null) || strNewPoints.length() < 2) return;
			
			// detection du type
			String strType = strNewPoints.substring(0, strNewPoints.indexOf(':')+1);
			if( strType.equalsIgnoreCase(SIMPLE_POINTS_FUNCTION) == true ) blLogY = false;
			else if( strType.equalsIgnoreCase(LOG_POINTS_FUNCTION) == true ) blLogY = true;
			else return;

			this.strPoints = strNewPoints;
			String[] tabl = strNewPoints.substring(strNewPoints.indexOf(':')+2, strNewPoints.length()-1).split("\\]\\[");
				
			xValues = new double[tabl.length];
			yValues = new double[tabl.length];
			String[] val;

			for(int i=0; i<tabl.length; i++){
				val = tabl[i].split(";");
				xValues[i] = Double.parseDouble(val[0]);
				if("NAN".equalsIgnoreCase(val[1])){
					yValues[i] = Double.NaN;
				}else{
					yValues[i] = Double.parseDouble(val[1]);
				}

			}

		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "setPoints()", ex);}
	}

	public final String getPoints() {return strPoints;}

	
	
	
	
	
	public final double yValue(double xValue)
	{
		try{
			if(strPoints == null) return Double.NaN;
			
			// on recherche le bon x
			int i=0;
			while( (i < xValues.length) && (xValues[i] < xValue) ) i++;
			
			// si c'est avant on prend la premiere valeur
			if(i == 0) return yValues[0];
			
			// si c'est apres on prend la derniere valeur
			if(i == xValues.length) return yValues[xValues.length-1];
			
			// si une valeur d'a cote n'est pas un nombre, on calcule rien
			if( Double.isNaN(yValues[i]) || Double.isNaN(yValues[i-1]) ) return Double.NaN;
			
			// on calcule la valeur sur une droite reliant les deux points
			double x1 = xValues[i-1];
			double y1 = yValues[i-1];
			
			double x2 = xValues[i];
			double y2 = yValues[i];

			if(blLogY == true){
				x1 = Maths.log( x1 );
				x2 = Maths.log( x2 );
				y1 = Maths.log( y1 );
				y2 = Maths.log( y2 );
			}
			
			double a = (y2 - y1) / (x2 - x1);
			double b = y1 - a * x1;
			
			if(blLogY == true) return Maths.pow( 10.0 , a * Maths.log( xValue ) + b );
			else return a * xValue + b;
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "yValue()", ex);}
		return Double.NaN;
	}
	
	



}
